/*
 ============================================================================
 Name        : AnalyzeRNAMotifs.c
 Author      : Shahidul Islam
 Version     :
 Copyright   : Your copyright notice
 Description : Hello World in C, Ansi-style
 ============================================================================
 */

#include "./inc/CommonDefinitions.h"
#include "./inc/UtilityFunctions.h"
#include "./inc/MappingBasePairs.h"
#include "./inc/ReadPDBData.h"
#include "./inc/ReadMCAnnotateData.h"
#include "./inc/ReadRNAViewData.h"
#include "./inc/ReadFR3DData.h"
#include "./inc/ReadDSSRData.h"
#include "./inc/ReadClaRNAData.h"
#include "./inc/MyersMillerAlign.h"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

 //LinuxPath
char pathInput[] = "./io_data/Input/";
char pathOutput[] = "./io_data/Output/";

//Windows Path
//char pathInput[] = "io_data\\Input\\";
//char pathOutput[] = "io_data\\Output\\";

//Variables to store atom infos (defined in ReadPDBData.h)
extern atomInfo atomList[MAX_ATOM_COUNT];
extern int atomCount;

extern atomInfo atomMultiList[MAX_ATOM_LIST][MAX_ATOM_COUNT];
extern int atomMultiCount[MAX_ATOM_LIST];

//Store sequence alignment for the fasta files (defined MyersMillerAlign.h)
extern sequenceMap refSeqsMap[MAX_ALIGN_PDB_COUNT];
extern int mapPDBsCount;

extern sequenceMap refMap[MAX_ALIGN_PDB_COUNT];
extern sequenceMap pdbMap[MAX_ALIGN_PDB_COUNT];
extern sequenceMap pdbTempMap[MAX_ALIGN_PDB_COUNT];

extern basePairInfo basePairMap[MAX_PDB_FILES][MAX_BASE_PAIR];
extern int basePairCountsMap[MAX_PDB_FILES];

extern basePairInfo basePairAccepted[MAX_BASE_PAIR];
extern int bpAccCount;


extern int updatePDBAlign_Star3D(sequenceMap pdbMap[], char path[], char pdbId1[], char chainId1[], char pdbId2[], char chainId2[]);

void copyAtoms(atomInfo *atomMultiList, int *atomMCount, atomInfo atomList[], int atomCount);
int processArguments(int argc, char *argv[], char pdbIds[][12][6], char targetChain[][12][MAX_CHAIN_NAME_LENGTH + 1], annotationMethod requestedMethods[], int *requestMethodCount, int *keepOrigFormat, int *printMotifs);
void testCompAnnotate(char pdbIds[][12][6], char targetChain[][12][MAX_CHAIN_NAME_LENGTH + 1], int *alignEndInd, int *startPos, int *increment, int *isRegular, annotationMethod requestedMethods[], int *requestMethodCount);
void testAlignOutput(char pathOutput[], annotationMethod aMethod, sequenceMap pdbMap[MAX_ALIGN_PDB_COUNT]);
void methodListPrint(annotationMethod requestedMethods[], int requestMethodCount);

int main(int argc, char *argv[]) {

	//Variables related to atom and refine parameters from PDB file
	int atomListCount = 2, isRegular;
	char pdbIds[2][12][6], targetChain[2][12][MAX_CHAIN_NAME_LENGTH + 1];
	PDB_RefineInfo PDB_RefineInfoList[2][12];
	double dpiPass[2];
	//int fastaFileFound;

	//Variables related to method and output format selection
	annotationMethod requestedMethods[5];
	int requestMethodCount = 0, keepOriginalFomat = 0;
	int alignEndInd = 0, startPos = 0, increment = 1;

	//Initialization related to benchmark result		
	int printMotifs = 0;		
	testCodeFlag = testCodeFlag2 = 0;		
	
	
	if(!processArguments(argc, argv, pdbIds, targetChain, requestedMethods, &requestMethodCount, &keepOriginalFomat, &printMotifs))
		return 0;

	char fileName[MAX_FILE_SZ] = "";
	char outputFileName[MAX_FILE_SZ] = "";
	char fileNameBPSrc[MAX_FILE_SZ] = "";	
	
	char file1[MAX_FILE_SZ] = "";//, file2[MAX_FILE_SZ] = "";

	char pdbsRefAlign[2][20];

	char *proposedBP = "ProposedBasePair";

	char tempStackFile[] = "tempStack.txt";

	int i, j, k, m;
	PRINT_ORIGINAL_SIMPLE = 0;

	//extractRelevantFR3D_BP();

	//Motif Extracting Process
	if(printMotifs)
		motifInitialization(motifs, &motifCount, pathOutput);

	//Method-wise processing
	annotationMethod aMethod;
	char methodName[30];
	
	if(!(printMotifs == 1 || testCodeFlag == 1 || testCodeFlag2 == 1))
		printf("Reference: %s %s\nTarget: %s %s\n", pdbIds[0][0], targetChain[0][0], pdbIds[1][0], targetChain[1][0]);

	int printedAlignWarnMessage = 0;

	//Test CompAnnotate for different methods
	testCodeLog = testCodeLog2 = NULL;
	writeCompResult = 1;

	//Set parameters and files if running the code in test mode
	if(testCodeFlag)
	{
		strcpy(fileName, pathOutput);
		strcat(fileName, "log.txt");
		testCodeLog = fopen(fileName, "w");
		if(!testCodeLog)
		{
			printf("%s file could not be created.\n", fileName);
			return 0;
		}
		writeCompResult = 0;

		strcpy(fileName, pathOutput);
		strcat(fileName, "log2.txt");
		testCodeLog2 = fopen(fileName, "w");
		if(!testCodeLog2)
		{
			printf("%s file could not be created.\n", fileName);
			return 0;
		}
	}

	/*Run loop for data set type
	 * (writeCompResult == 1) implies running CompAnnotate for the real reference and target data
	 * (writeCompResult == 0) implies running CompAnnotate for the test data
	 */
	for(; writeCompResult <= 1; writeCompResult++)
	{
		COMP_EDGE_ONLY = writeCompResult;
		if(testCodeLog2)
		{
			if(writeCompResult)
				fprintf(testCodeLog2, "\n\nCompAnnotate\n\n");
			else
				fprintf(testCodeLog2, "Regular Annotate\n\n");
		}

		if(testCodeLog)
		{
			if(writeCompResult)
				fprintf(testCodeLog, "\n\nCompAnnotate\n\n");
			else
				fprintf(testCodeLog, "Regular Annotate\n\n");
			testCompAnnotate(pdbIds, targetChain, &alignEndInd, &startPos, &increment, &isRegular, requestedMethods, &requestMethodCount);
		}
		// Run CompAnnotate for different method (ie, MC-Annotate, RNAView etc)
		for(m = 0; m < requestMethodCount; m++)
		{
			aMethod = requestedMethods[m];
		    methodPrintToFile(aMethod, testCodeLog);
			methodPrintToFile(aMethod, testCodeLog2);

			/* Run CompAnnotate for different pair of Reference and Target data.
			 * For real user input scenario, only one pair is expected.
			 * More pairs can come for the test data.
			 */
			for(k = startPos; k <= alignEndInd; k+=increment)
			{
			    for(i = 0; i < atomListCount; i++)
				{
					strcpy(pdbsRefAlign[i], pdbIds[i][k]);
					//strcat(pdbsRefAlign[i], ".pdb");
				}

			    //Read atom and base-pair info for Reference and Target PDB
				for(i = 0; i < atomListCount; i++)
				{
					//Read Base Pairs
					strcpy(fileName, pathInput);
					strcat(fileName, pdbIds[i][k]);

					//Considering the 1st pdb value of odd index will be propagated from the previous level
					if(k % 2 == 1 && i == 0)
					{
						basePairCountsMap[i] = bpAccCount;
						for(j = 0; j < bpAccCount; j++)
							basePairMap[i][j] = basePairAccepted[j];
					}
					else
					{
						if(k % 2 == 0)
							bpAccCount = 0;
						if(aMethod == mcAnnotate)
						{
							strcpy(methodName, "MC-Annotate Regular");
							strcat(fileName, ".mca.out");
							if(!readMCAnnotateOutput(fileName, basePairMap[i], &basePairCountsMap[i], targetChain[i][k], tempStackFile))
								return 0;
						}
						else
						{
						    if(aMethod == fr3d)
							{
								strcat(fileName, ".fr3d.out");
								strcpy(methodName, "FR3D Regular");
								if(!readFR3DOutput(fileName, basePairMap[i], &basePairCountsMap[i], pdbIds[i][k], targetChain[i][k], tempStackFile))
									return 0;
							}
							else if(aMethod == dssr)
							{
								strcat(fileName, ".dssr.out");
								strcpy(methodName, "DSSR Regular");
								if(!readDSSROutput(fileName, basePairMap[i], &basePairCountsMap[i], pdbIds[i][k], targetChain[i][k], tempStackFile))
									return 0;
							}
							else if(aMethod == clarna)
							{
								strcat(fileName, ".cr.out");
								strcpy(methodName, "ClaRNA Regular");
								if(!readClaRNAOutput(fileName, basePairMap[i], &basePairCountsMap[i], pdbIds[i][k], targetChain[i][k], tempStackFile))
									return 0;
							}
							else
							{
							    strcat(fileName, ".rv.out");
								strcpy(methodName, "RNAView Regular");
								if(!readRNAViewOutput(fileName, basePairMap[i], &basePairCountsMap[i], targetChain[i][k], tempStackFile))
									return 0;
							}
						}
						if(printMotifs)
							extractMotifInfo(motifs, motifCount, pdbIds[i][k], methodName, basePairMap[i], basePairCountsMap[i], pathOutput);
					}
				}
				//Read fasta Sequence
				strcpy(file1, pathInput);
				strcat(file1, "pdb_seqres.na.fa");

				if(readFastaSequence(file1, pdbIds[0][k], targetChain[0][k], pdbIds[1][k], targetChain[1][k], refSeqsMap))
				{
					mapPDBsCount = 2;
					fastaFileFound = 1;
				}
				else
				{
					printf("The chain sequence for given PDB was not found in %s.\nPlease check your input, or update the file from PDB server.\n", file1);
					//printf("Attempting to read the sequence from the PDB file anyway.\n", file1);
					fastaFileFound = 0;
					return 0;
				}
				for(i = 0; i < atomListCount; i++)
				{
					//Read AtomInfo and align with reference seq
					//strcpy(fileName, pathInput);
					if(!readPDBFile(pathInput, pdbsRefAlign[i], &PDB_RefineInfoList[i][k], targetChain[i][k]))
						return 0;
					copyAtoms(atomMultiList[i], &atomMultiCount[i], atomList, atomCount);

					//Defined in MyersMillerAlign.h
					alignPDBWithRef(refSeqsMap[i], atomMultiList[i], atomMultiCount[i], &refMap[i], &pdbTempMap[i]);
				}
                if(!testCodeFlag && (k % 2 == 0) && PDB_RefineInfoList[0][k].resolution > PDB_RefineInfoList[1][k].resolution)
				{
					printf("\nWarning!!! Undesired data resolution - ");
					printf("Reference (%s - %.2lfA) is lower resolution than Target (%s - %.2lfA) (you may have given the PDB ids in reverse order), continuing anyway.\n",
							pdbIds[0][k], PDB_RefineInfoList[0][k].resolution, pdbIds[1][k], PDB_RefineInfoList[1][k].resolution);
				}

                ////Defined in MyersMillerAlign.h
				alignFasta(refSeqsMap, refMap, pdbTempMap, basePairMap, basePairCountsMap);
                //Align pdb seq with atom info
				dpiPass[0] = PDB_RefineInfoList[0][k].dpi;
				dpiPass[1] = PDB_RefineInfoList[1][k].dpi;
				//Defined in MappingBasePairs.h
				alignPDBs(refSeqsMap, refMap, pdbTempMap, pdbMap, dpiPass);

				//testAlignOutput(pathOutput, aMethod, pdbMap);

				//Incorporate Ping's PDB Align
				if(!updatePDBAlign_Star3D(pdbMap, pathInput, pdbMap[0].seqId, pdbMap[0].aChainMap[0].chainId, pdbMap[1].seqId, pdbMap[1].aChainMap[0].chainId))
				{
					if(!printedAlignWarnMessage)
					{
						printf("\nWarning!!! Structure alignment file \"%s_%s_%s_%s.aln\" was not found, ",
							pdbMap[0].seqId, pdbMap[0].aChainMap[0].chainId, pdbMap[1].seqId, pdbMap[1].aChainMap[0].chainId);
						printf("continuing with alternate built-in sequence alignment.\n");
						printedAlignWarnMessage = 1;
					}
				}
				strcpy(fileName, pathOutput);
				strcat(fileName, proposedBP);
				if(aMethod == mcAnnotate)
					strcat(fileName, "_MC_Annotate_");
				else
				{
					if(aMethod == fr3d)
						strcat(fileName, "_FR3D_");
					else if(aMethod == dssr)
						strcat(fileName, "_DSSR_");
					else if(aMethod == clarna)
						strcat(fileName, "_ClaRNA_");
					else
						strcat(fileName, "_RNAView_");
				}

				if(k % 2 == 1)
				{
					//print Benchmark BP Classification
					int cano = 0, nonCano = 0, oth = 0;
					printf("Total Benchmark BP = %d\n", basePairCountsMap[1]);
					for(j = 0; j < basePairCountsMap[1]; j++)
					{
						if(isCanonicalBP(basePairMap[1][j]))
							cano++;
						else if(isWHS_BP(basePairMap[1][j]))
							nonCano++;
						else
						{
							oth++;
							//printPairMCAnnotateOnScreen(basePairMap[1][j]);
							//printf("Canonical = %d, Noncanonical = %d, Others = %d\n", cano, nonCano, oth);
							//getchar();
						}
					}
					printf("--------Benchmark (%s) BP classification--------\n", pdbIds[1][k]);
					printf("Canonical = %d, Noncanonical = %d, Others = %d\n", cano, nonCano, oth);
					if(testCodeLog)
					{
						fprintf(testCodeLog, "Benchmark %s\n", pdbIds[1][k]);
						fprintf(testCodeLog, "Canonical:%d, Noncanonical:%d, Others:%d\n", cano, nonCano, oth);
					}
				}

				if(isRegular)
					COMPARE_EDGE = 1;
				else
				{
					if(k % 2 == 0)
						COMPARE_EDGE = 0;
					else
						COMPARE_EDGE = 1;
				}
				mapBasePairs(atomMultiList, atomMultiCount, pdbMap, basePairMap, basePairCountsMap, fileName, !(k % 2));
				//for(i = 0; i < atomListCount; i++)
					//if(atomMultiList[i])
						//free(atomMultiList[i]);
				if(PRINT_ALIGN_BP_INFO)
				{
					printf("%s ref seq count = %d\n", refMap[0].seqId, refMap[0].aChainMap[0].resCount);
					printf("%s ref seq count = %d\n", refMap[1].seqId, refMap[1].aChainMap[0].resCount);
					printf("%s PDB seq count = %d\n", pdbMap[0].seqId, pdbMap[0].aChainMap[0].resCount);
					printf("%s PDB seq count = %d\n", pdbMap[1].seqId, pdbMap[1].aChainMap[0].resCount);
				}
				//char updatedBasePairs[MAX_FILE_SZ] = "updatedBPs_";

				for(i = 0; i <= 1; i++)
				{
				    for(PRINT_ORIGINAL_SIMPLE = 0; PRINT_ORIGINAL_SIMPLE <= 0; PRINT_ORIGINAL_SIMPLE++)
					{
						strcpy(fileName, pathOutput);
						strcpy(fileNameBPSrc, pathOutput);
						
						if(strcmp(pathOutput, "./io_data/Output/") == 0)
							strcat(fileNameBPSrc, "BP_Src/");
						else
							strcat(fileNameBPSrc, "\\BP_Src");
						strcpy(outputFileName, "");
						strcat(outputFileName, pdbIds[i][k]);
						strcat(outputFileName, "_");
						strcat(outputFileName, targetChain[i][k]);
						if(aMethod == mcAnnotate)
						{
							if(SIMPLE_OUTPUT_FORMAT)
							{
								if(PRINT_ORIGINAL_SIMPLE)
									strcat(outputFileName, ".mcaOrig.pairs");
								else
									strcat(outputFileName, ".mca.pairs");
							}
							else
							{
								if(keepOriginalFomat)
									strcat(outputFileName, ".mca.orig.rca");
								else
									strcat(outputFileName, ".mca.rca");
							}
						}
						else
						{
							if(SIMPLE_OUTPUT_FORMAT)
							{
								if(aMethod == fr3d)
								{
									if(PRINT_ORIGINAL_SIMPLE)
										strcat(outputFileName, ".FR3DOrig.pairs");
									else
										strcat(outputFileName, ".FR3D.pairs");
								}
								else if(aMethod == dssr)
								{
									if(PRINT_ORIGINAL_SIMPLE)
										strcat(outputFileName, ".DSSROrig.pairs");
									else
										strcat(outputFileName, ".DSSR.pairs");
								}
								else if(aMethod == clarna)
								{
									if(PRINT_ORIGINAL_SIMPLE)
										strcat(outputFileName, ".ClaRNAOrig.pairs");
									else
										strcat(outputFileName, ".ClaRNA.pairs");
								}
								else
								{
									if(PRINT_ORIGINAL_SIMPLE)
										strcat(outputFileName, ".rvOrig.pairs");
									else
										strcat(outputFileName, ".rv.pairs");
								}
							}
							else
							{
								if(aMethod == fr3d)
									strcat(outputFileName, ".fr3d.rca");
								else if(aMethod == dssr)
									strcat(outputFileName, ".dssr.rca");
								else if(aMethod == clarna)
									strcat(outputFileName, ".clarna.rca");
								else
								{
									if(keepOriginalFomat)
										strcat(outputFileName, ".rv.orig.rca");
									else
										strcat(outputFileName, ".rv.rca");
								}
							}
						}
						//printf("i = %d, k = %d, file = %s\n", i, k, outputFileName);
						//getchar();
						strcat(fileName, outputFileName);
						strcat(fileNameBPSrc, outputFileName);
						strcat(fileNameBPSrc, ".bpsrc");
						if(writeCompResult)
						{
							if(PRINT_ORIGINAL_SIMPLE)
							{
								//For simplified output to be used in alignment tool like ARTS
								if(!(k % 2 == 1 && i == 1))
									printAcceptedBPInteractions(fileName, fileNameBPSrc, pdbIds[i][k], aMethod, 1, tempStackFile, basePairMap[i], basePairCountsMap[i], 0, pathOutput, 0);
							}
							else
							{
								if(k % 2 == 0 && i == 1)
									printAcceptedBPInteractions(fileName, fileNameBPSrc, pdbIds[1][k], aMethod, 1, tempStackFile, basePairMap[1], basePairCountsMap[1], printMotifs, pathOutput, keepOriginalFomat);
							}
						}
					}
				}
			}
		}
		printf("\nSuccessfully enhanced the basepair annotation for PDB %s, chain %s.\n", pdbIds[1][0], targetChain[1][0]);
		printf("The result for method");
		methodListPrint(requestedMethods, requestMethodCount);
		printf("is placed in folder: %s\n", pathOutput);
	}

	remove(tempStackFile);

	if(testCodeLog)
		fclose(testCodeLog);

	if(testCodeLog2)
		fclose(testCodeLog2);

	if(writeCompResult && testCodeFlag)
		readLogWriteTables(pathOutput, "log.txt");

	if(writeCompResult && testCodeFlag2)
		processInferabilityLog(pathOutput, "log2.txt");

	return 0;
}

void methodListPrint(annotationMethod requestedMethods[], int requestMethodCount)
{
	int m;
	for(m = 0; m < requestMethodCount; m++)
	{
		if(requestMethodCount > 1 && m == requestMethodCount - 1)
			printf(" and");
		else if(m > 0)
			printf(",");
		if(requestedMethods[m] == mcAnnotate)
			printf(" MC-Annotate");
		if(requestedMethods[m] == rnaView)
			printf(" RNAView");
		if(requestedMethods[m] == fr3d)
			printf(" FR3D");
		if(requestedMethods[m] == dssr)
			printf(" DSSR");
		if(requestedMethods[m] == clarna)
			printf(" ClaRNA");
	}
	printf(" ");
}

int gotNonAlphaNumeric(char str[])
{
	int i, len = strlen(str);
	for(i = 0; i < len; i++)
		if(!isalnum(str[i]))
			return 1;
	return 0;
}

#define REF_COUNT 20

void compareAlignScore(char fastaFile[], char targetSeq[], char refIds[][PDB_ID_SZ], char refChains[][MAX_CHAIN_NAME_LENGTH], int count, char refSequence[][MAX_SEQ_LEN], char bestRefId[], char bestChain[], double *bestScore)
{
	doScoreOnlyAlign = 1;
	readFastaSeqList(fastaFile, refIds, refChains, count, refSequence);
	 
	int i;
	double score = 0.0;
	int targetLen = strlen(targetSeq), refLen;
	char *state = NULL;
	
	for(i = 0; i < count; i++)
	{
		refLen = strlen(refSequence[i]);
		state = (char *)malloc((targetLen + refLen) * sizeof(char));
		
		MMAlign(targetSeq, targetLen, refSequence[i], refLen, state, &score);
		
		//printf("%s %s %0.2lf\n", refIds[i], refChains[i], score);
		
		if(score > *bestScore)
		{
			strcpy(bestRefId, refIds[i]);
			strcpy(bestChain, refChains[i]);
			*bestScore = score;
		}
		
		if(state)
		{
			free(state);
			state = NULL;
		}
	}
	doScoreOnlyAlign = 0;
}

void findBestReference(char referenceListFile[])
{
	char fileName[MAX_FILE_SZ], fastaFile[MAX_FILE_SZ];
	
	strcpy(fileName, pathInput);
	strcat(fileName, referenceListFile);
	FILE *fpt = fopen(fileName, "r");
	if(!fpt)
	{
		printf("%s File Not Found\n", fileName);
		return;
	}
	
	strcpy(fastaFile, pathInput);
	strcat(fastaFile, "pdb_seqres.na.fa");
	
	char target[PDB_ID_SZ], targetChain[MAX_CHAIN_NAME_LENGTH];
	char refIds[REF_COUNT][PDB_ID_SZ], refChains[PDB_ID_SZ][MAX_CHAIN_NAME_LENGTH];
	char targetSeq[MAX_SEQ_LEN], refSequence[REF_COUNT][MAX_SEQ_LEN];
	char bestRefId[PDB_ID_SZ], bestChain[MAX_CHAIN_NAME_LENGTH];
	double bestScore = -1e9;
	
	fscanf(fpt, "%s %s", target, targetChain);
	
	readFastaSeqList(fastaFile, &target, &targetChain, 1, &targetSeq);
	
	strcpy(bestRefId, target);
	strcpy(bestChain, targetChain);
	
	int count = 0;
	printf("Candidate Reference List:\n");
	while(fscanf(fpt, "%s %s", refIds[count], refChains[count]) == 2)
	{
		printf("Candidate %d = %s %s\n", count + 1, refIds[count], refChains[count]);
		count++;
		
		if(count == REF_COUNT)
		{
			compareAlignScore(fastaFile, targetSeq, refIds, refChains, count, refSequence, bestRefId, bestChain, &bestScore);
			count = 0;
		}
	}
	
	if(count > 0)
		compareAlignScore(fastaFile, target, refIds, refChains, count, refSequence, bestRefId, bestChain, &bestScore);
	
	printf("\nTarget = %s %s\n", target, targetChain);
	printf("Recommended reference: %s %s\n", bestRefId, bestChain);
	
	if(fpt)
		fclose(fpt);
}

int processArguments(int argc, char *argv[], char pdbIds[][12][6], char targetChain[][12][MAX_CHAIN_NAME_LENGTH + 1], annotationMethod requestedMethods[], int *requestMethodCount, int *keepOrigFormat, int *printMotifs)
{
	*keepOrigFormat = 0;
	int i, j, len;
	char originalArg2[MAX_FILE_SZ];
	strcpy(originalArg2, argv[2]);
	for(i = 0; i < argc; i++)
	{
		len = strlen(argv[i]);
		for(j = 0; j < len; j++)
			argv[i][j] = toupper(argv[i][j]);
	}
	if(argc < 5)
	{
		int flag = 0; 
		
		if(argc > 2 && strcmp(argv[1], "-R") == 0)
		{
			findBestReference(originalArg2);
			return 0;
		}
		
		if(argc > 2 && strcmp(argv[1], "-B") == 0)
		{
			if(strcmp(argv[2], "BENCHMARK-MOTIF") == 0)
			{
				*printMotifs = 1;
				testCodeFlag = testCodeFlag2 = 1;
				flag = 1;
			}
			else if(strcmp(argv[2], "BENCHMARK") == 0)
			{
				testCodeFlag = testCodeFlag2 = 1;
				flag = 1;
			}
		}
		if(!flag)
		{
			printf("Required valid arguments not found.\nPlease check the 'readme' file for details.\n");
			return 0;
		}
	}
	else
	{
		int i, j, len;
		for(i = 0; i < argc; i++)
		{
			len = strlen(argv[i]);
			for(j = 0; j < len; j++)
				argv[i][j] = toupper(argv[i][j]);
		}
		if(strlen(argv[1]) != 4 || strlen(argv[3]) != 4 || gotNonAlphaNumeric(argv[1]) || gotNonAlphaNumeric(argv[3]) )
		{
			if(strlen(argv[1]) != 4 || gotNonAlphaNumeric(argv[1]))
				printf("Reference PDB id doesn't seem correct.\n");
			if(strlen(argv[3]) != 4 || gotNonAlphaNumeric(argv[3]))
				printf("Target PDB id doesn't seem correct.\n");
			return 0;
		}
		strcpy(pdbIds[0][0], argv[1]);
		strcpy(targetChain[0][0], argv[2]);
		strcpy(pdbIds[1][0], argv[3]);
		strcpy(targetChain[1][0], argv[4]);
		if(strlen(argv[2]) > 1 || strlen(argv[4]) > 1)
			printf("Warning!!! Single character chain id expected, continuing  with first character anyway.\n");
		//printf("%s %c %s %c\n", pdbIds[0][0], targetChain[0][0], pdbIds[1][0], targetChain[1][0]);
		*requestMethodCount = 0;
		int hasMethodArgument = 0;
		for(i = 5; i < argc; i++)
		{
			if(strcmp(argv[i], "-M") == 0)
			{
				if(i + 1 < argc)
				{
					if(strcmp(argv[i + 1], "ALL") == 0)
					{
						requestedMethods[(*requestMethodCount)++] = mcAnnotate;
						requestedMethods[(*requestMethodCount)++] = rnaView;
						requestedMethods[(*requestMethodCount)++] = fr3d;
						requestedMethods[(*requestMethodCount)++] = dssr;
						requestedMethods[(*requestMethodCount)++] = clarna;
						hasMethodArgument = 1;
					}
					else
					{
						char delim[3] = ", ", token[10][200];
						int tokenCount, flag[5] = {0};
						myTokenize(argv[i + 1], delim, token, &tokenCount, 1);
						for(j = 0; j < tokenCount; j++)
						{
							if(!flag[0] && strstr(token[j],"MC"))
							{
								requestedMethods[(*requestMethodCount)++] = mcAnnotate;
								flag[0] = 1;
								hasMethodArgument = 1;
							}
							else if(!flag[1] && strstr(token[j],"RV"))
							{
								requestedMethods[(*requestMethodCount)++] = rnaView;
								flag[1] = 1;
								hasMethodArgument = 1;
							}
							else if(!flag[2] && strstr(token[j],"FR"))
							{
								requestedMethods[(*requestMethodCount)++] = fr3d;
								flag[2] = 1;
								hasMethodArgument = 1;
							}
							else if(!flag[3] && strstr(token[j],"DS"))
							{
								requestedMethods[(*requestMethodCount)++] = dssr;
								flag[3] = 1;
								hasMethodArgument = 1;
							}
							else if(!flag[4] && strstr(token[j],"CLA"))
							{
								requestedMethods[(*requestMethodCount)++] = clarna;
								flag[4] = 1;
								hasMethodArgument = 1;
							}
							else
								printf("Warning!!! Unknown method %s.\n", token[j]);
							//printf("%s\n", token[j]);
						}
					}
				}
			}
			else if(strcmp(argv[i], "-F") == 0)
			{
				if(i + 1 < argc && strstr(argv[i + 1],"ORIG"))
					*keepOrigFormat = 1;
			}
		}
		if(!hasMethodArgument)
		{
			*requestMethodCount = 1;
			requestedMethods[0] = mcAnnotate;
		}
	}

	//for(i = 0; i < argc; i++)
	//	printf("%s\n", argv[i]);
	//getchar();
	return 1;
}

void copyAtoms(atomInfo *atomMList, int *atomMCount, atomInfo atomList[], int atomCount)
{
	int i;
	//atomMList = (atomInfo*)malloc(atomCount*sizeof(atomInfo));
	*atomMCount = atomCount;
	for(i = 0; i < atomCount; i++)
		atomMList[i] = atomList[i];
}

void testCompAnnotate(char pdbIds[][12][6], char targetChain[][12][MAX_CHAIN_NAME_LENGTH + 1], int *alignEndInd, int *startPos, int *increment, int *isRegular, annotationMethod requestedMethods[], int *requestMethodCount)
{
	*requestMethodCount = 0;
	requestedMethods[(*requestMethodCount)++] = mcAnnotate;
	requestedMethods[(*requestMethodCount)++] = rnaView;
	requestedMethods[(*requestMethodCount)++] = fr3d;
	requestedMethods[(*requestMethodCount)++] = dssr;
	requestedMethods[(*requestMethodCount)++] = clarna;
	int lim;

	//Regular Target compare to Benchmark
	if(!writeCompResult)
	{
		*alignEndInd = 9;
		*startPos = 2;
		*increment = 2;//3
		*isRegular = 1;
		lim = 12;
	}
	char tempPdbIdsReg[2][12][6] = 									{{"1PNY", "1PNY", "2B9N", "2B9N", "2B9N", "2B9N", "1YL4", "1YL4", "2B9M", "2B9M", "3JYX", "3JYX"},
								 	     	 	 	 	 	 	 	 {"2ZJR", "2ZJR", "3V2F", "3V2F", "3V2F", "3V2F", "2VQE", "2VQE", "2VQE", "2VQE", "3JYX", "3JYX"}};
	char temptargetChainReg[2][12][MAX_CHAIN_NAME_LENGTH + 1] = 	{{   "0",    "0",    "A",    "A",    "A",    "A",    "A",    "A",    "A",    "A",    "5",    "5"},
									     	 	 	 	 	 	 	 {   "X",    "X",    "A",    "A",    "A",    "A",    "A",    "A",    "A",    "A",    "5",    "5"}};


	//Comparative Target compare to Benchmark
	if(writeCompResult)
	{
		*alignEndInd = 9;
		*startPos = 2;
		*increment = 1;//3
		*isRegular = 0;
		lim = 12;
	}
	char tempPdbIds[2][12][6] = 									{{"3V2F", "1PNY", "3R8S", "2B9N", "2ZJR", "2B9N", "4GD1", "1YL4", "4GD1", "2B9M", "4GD1", "3JYX"},
							 	 	 	 	 	 	 	 	 	 	 {"1PNY", "2ZJR", "2B9N", "3V2F", "2B9N", "3V2F", "1YL4", "2VQE", "2B9M", "2VQE", "3JYX", "3JYX"}};
	char temptargetChain[2][12][MAX_CHAIN_NAME_LENGTH + 1] = 	{    {   "A",    "0",    "A",    "A",    "X",    "A",    "A",    "A",    "A",    "A",    "A",    "5"},
								 	 	 	 	 	 	 	 	 	 {   "0",    "X",    "A",    "A",    "A",    "A",    "A",    "A",    "A",    "A",    "5",    "5"}};

	//Kink Turn Check
	/*
	*alignEndInd = 2;
	*startPos = 2;
	*increment = 1;//3
	*isRegular = 1;
	lim = 8;
	char tempPdbIds[2][8][6] = 			{{"3U5D", "3J62", "1VQ8", "3J2L", "3U5D", "3JYX", "3V2D", "3BBO"},
									 {"3J62", "3U5D", "3J2L", "1VQ8", "3JYX", "3U5D", "3BBO", "3V2D"}};
	char temptargetChain[2][8][3] = 	{{   "1",    "a",    "0",    "1",    "1",    "5",    "A",    "A"},
									 {   "a",    "1",    "1",    "0",    "5",    "1",    "A",    "A"}};
	double tempResolution[2][8] = 		{{   3.0,    5.5,    2.3,    6.6,    3.0,    8.9,    2.7,    9.4},
									 {   5.5,    3.9,    6.6,    2.2,    8.9,    3.0,    9.4,    2.7}};
	*/
	
	int i, j;
	if(writeCompResult)
	{
		for(i = 0; i < 2; i++)
			for(j = 0; j < lim; j++)
			{
				strcpy(pdbIds[i][j], tempPdbIds[i][j]);
				strcpy(targetChain[i][j], temptargetChain[i][j]);
			}
	}
	else
	{
		for(i = 0; i < 2; i++)
			for(j = 0; j < lim; j++)
			{
				strcpy(pdbIds[i][j], tempPdbIdsReg[i][j]);
				strcpy(targetChain[i][j], temptargetChainReg[i][j]);
			}
	}
}

void testAlignOutput(char pathOutput[], annotationMethod aMethod, sequenceMap pdbMap[MAX_ALIGN_PDB_COUNT])
{
	char fileName[MAX_FILE_SZ] = "";
	char *pdbMapOutput = "PDBAlign";

	strcpy(fileName, pathOutput);
	strcat(fileName, pdbMapOutput);

	//Print PDB aligns
	if(aMethod == mcAnnotate)
		strcat(fileName, "_MC_Annotate_");
	else
	{
		if(aMethod == fr3d)
			strcat(fileName, "_FR3D_");
		else if(aMethod == dssr)
			strcat(fileName, "_DSSR_");
		else if(aMethod == clarna)
			strcat(fileName, "_ClaRNA_");
		else
			strcat(fileName, "_RNAView_");
	}
	strcat(fileName, "_");
	strcat(fileName, pdbMap[0].seqId);
	strcat(fileName, "_");
	strcat(fileName, pdbMap[1].seqId);
	strcat(fileName, ".out");

	printPdbAlign(fileName, pdbMap[0], pdbMap[1], 50);
}
