/*
 * MappingBasePairs.c
 *
 *  Created on: Apr 17, 2014
 *  Author: Shahidul
 */

#include "./inc/MappingBasePairs.h"
#include "./inc/UtilityFunctions.h"
#include "./inc/ReadPDBData.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#define ATOM_PAIR_COUNT 2
#define POS_ALIGN 0
#define TEST_PRINT_SETDIFF 0
#define PRINT_MAPPING 0

#define NO_ALIGN_STAT -1 //-1 (Exclude), 0 (Include)


int commonCBP, commonNCBP, commonOthers;
int diffCBP, diffNCBP, diffOthers;

int probable2ndListDiffCnt;
double root2;
int bpSource;

void generateSetDifference(sequenceMap pdbMap[], int refPDBIndex, basePairInfo basePairMapping1[], int count1, basePairInfo basePairMapping2[], int count2,
		basePairMapping *basePairSetDiff, basePairMapping *basePairCommon, basePairMapping *basePairNoAlign, basePairMapping *basePairTargetSetDiff, basePairMapping *basePairTargetNoAlign, int updateAcceptBPList);
int getCompatibleBasePair(basePairInfo x, sequenceMap pdbMap[], int refPDBIndex, basePairInfo *compatibleBP);
void classifyTargetCounts(basePairInfo compatibleBP, basePairMapping *basePairSet);
void classifyCounts(basePairInfo compatibleBP, basePairMapping *basePairSet, int isCommon);
void makeGeometricAssesment(atomInfo atomMultilist[][MAX_ATOM_COUNT], int atomMultiCount[], int refPDBIndex, basePairMapping aMapping, statisticsBP aStat[], int *statCount, FILE* fpt, int updateAcceptBPList, double resolutions[], int selfMapping);
int isSymmetricBP(basePairInfo a, basePairInfo b, int n, edgeAtomPair aEdgeAtomPairActual[], edgeAtomCoordinates edgeAtomsProposed[], double threshold, int isUpperBound);
int compBPIndices(const void *a, const void *b);
int compBPIndicesAndResidue(const void *a, const void *b);
int compBPResidue(const void *a, const void *b);
int compAtomInfoMap(const void *a, const void *b);
int compStatsDistDiff(const void *a, const void *b);
int compStatsDistPropose(const void *a, const void *b);
int compStatsDistActual(const void *a, const void *b);
int compStatsCoOrdDiffX(const void *a, const void *b);
int compStatsCoOrdDiffY(const void *a, const void *b);
int compStatsCoOrdDiffZ(const void *a, const void *b);
void printCorrectnessStatistics(FILE* fpt, int ind1, int ind2, sequenceMap pdbMap[]);
void printInferabilityInFile(FILE* fpt, int ind1, int ind2, sequenceMap pdbMap[]);
void printStats(int statCount, statisticsBP aStat[], FILE* fpt);

int printPrefix(FILE *fpt, int alignCount, int residuePerLine, char seqId[]);
void generateAlignSequence(sequenceMap pdbMap1, sequenceMap pdbMap2, char seq[], int *alignCount, char seqAlign[], int updateSeqAlign);

int mapBasePairs(atomInfo atomMultilist[][MAX_ATOM_COUNT], int atomMultiCount[], sequenceMap pdbMap[], basePairInfo basePairMapping[][MAX_BASE_PAIR], int basePairCountsMapping[], char outputFile[], int updateAcceptBPList)
{
    int i, j;

	double resolutions[2];
	resolutions[0] = pdbMap[0].resolution;
	resolutions[1] = pdbMap[1].resolution;
	root2 = sqrt(2);
			
	//Counters for different types (Canonical, non-Canonical etc) of Basepairs
	bpAccCount = 0;
	commonCBP = commonNCBP = commonOthers = 0;
	diffCBP = diffNCBP = diffOthers = 0;

	for(i = 0; i < 2; i++)
	{
		qsort(basePairMapping[i], basePairCountsMapping[i], sizeof(basePairInfo), compBPIndicesAndResidue);
		//Never sort these pdbs here, as the Mapping is based on relative position, and soting would ruin that
		//for(j = 0; j < pdbMap[i].chainCount; j++)
			//qsort(pdbMap[i].aChainMap[j].resMap, pdbMap[i].aChainMap[j].resCount - 2, sizeof(posMap), mappedSeqComp);
	}
	int refPDBIndex = 0;
	probable2ndListDiffCnt = 0;

	//Generate subsets of reference and target pdb basepairs, which is similar and dissimilar to each other
	if(refPDBIndex == 0)
		generateSetDifference(pdbMap, 0, basePairMapping[0], basePairCountsMapping[0], basePairMapping[1], basePairCountsMapping[1],
			&basePairSetDiff[0], &basePairCommon[0], &basePairNoAlign[0], &basePairSetDiff[1], &basePairNoAlign[1], updateAcceptBPList);
	else
		generateSetDifference(pdbMap, 1, basePairMapping[1], basePairCountsMapping[1], basePairMapping[0], basePairCountsMapping[0],
			&basePairSetDiff[1], &basePairCommon[1], &basePairNoAlign[1], &basePairSetDiff[0], &basePairNoAlign[0], updateAcceptBPList);

	if(testCodeLog)
		printCorrectnessStatistics(testCodeLog, refPDBIndex, (refPDBIndex + 1) % 2, pdbMap);

	if(testCodeLog2)
		printInferabilityInFile(testCodeLog2, refPDBIndex, (refPDBIndex + 1) % 2, pdbMap);

	if(PRINT_ALIGN_BP_INFO)
		printf("!!!---Probable 2nd List Diff Cnt  = %d---!!!\n", probable2ndListDiffCnt);
	//Test print of BP Mapping
	if(TEST_PRINT_SETDIFF)
	{
		int start = 0;
		for(j = start; j < start + 2; j++)
		{
			printPairMCAnnotateOnScreen(basePairSetDiff[refPDBIndex].proposedBasePairs[j]);
			printPairMCAnnotateOnScreen(basePairSetDiff[refPDBIndex].actualBasePairs[j]);
		}
		printf("\n");
		for(j = basePairSetDiff[refPDBIndex].bpCounts - 2; j < basePairSetDiff[refPDBIndex].bpCounts; j++)
		{
			printPairMCAnnotateOnScreen(basePairSetDiff[refPDBIndex].proposedBasePairs[j]);
			printPairMCAnnotateOnScreen(basePairSetDiff[refPDBIndex].actualBasePairs[j]);
		}
		printf("---------------------------\n");
	}

	//Generate Proposals of Base Pairs

	//for(i = 0; i < 2; i++)
		//qsort(atomMultilist[i], atomMultiCount[i], sizeof(atomInfo), compAtomInfoMap);

	//Proposal analysis output
	char fileName[MAX_FILE_SZ];
	strcpy(fileName, outputFile);
	strcat(fileName, "_");
	strcat(fileName, pdbMap[refPDBIndex].seqId);
	strcat(fileName, "_vs_");
	strcat(fileName, pdbMap[(refPDBIndex + 1) % 2].seqId);
	strcat(fileName, ".out");
	FILE *fpt;
	if(PRINT_MAPPING)
	{
		fpt = fopen(fileName, "w");
		if(!fpt)
		{
			printf("%s File Not Found\n", fileName);
			return 0;
		}
	}

	if(PRINT_MAPPING)
		fprintf(fpt, "\n\nCommon Set Statistics\n");
	if(testCodeLog2)
		fprintf(testCodeLog2, "Common BP Accept Count\n");

	//Geometric Analysis to filter potential basepairs
	/* updateAcceptBPList == 1 >> dealing with inferable reference BP, and accept if flexibility within threshold,
	 * updateAcceptBPList == 2 >> accept directly,
	 * updateAcceptBPList == 4 >> dealing with inferable target BP, and accept if flexibility is greater than threshold,
	 */

	bpSource = 2;
	if(updateAcceptBPList)
		makeGeometricAssesment(atomMultilist, atomMultiCount, refPDBIndex, basePairCommon[refPDBIndex], commonStat, &commonStatCount, fpt, 2, resolutions, 0);
	if(PRINT_MAPPING)
		fprintf(fpt, "Difference Set Statistics\n");
	
	bpSource = 1;
	if(testCodeLog2)
		fprintf(testCodeLog2, "Reference Inferable BP Accept Count\n");
	if(updateAcceptBPList)
		makeGeometricAssesment(atomMultilist, atomMultiCount, refPDBIndex, basePairSetDiff[refPDBIndex], diffStat, &diffStatCount, fpt, /*updateAcceptBPList*/1, resolutions, 0);

	bpSource = 0;
	if(PRINT_MAPPING)
		fprintf(fpt, "\n\nTarget Set Difference Statistics\n");
	//Categorize method based on BP type
	if(testCodeLog2)
		fprintf(testCodeLog2, "Target Inferable BP Accept Count\n");
	if(updateAcceptBPList)
		makeGeometricAssesment(atomMultilist, atomMultiCount, refPDBIndex, basePairSetDiff[(refPDBIndex + 1) % 2], targetSetDiffStat, &targetSetDiffStatCount, fpt, 4, resolutions, 0);
	if(testCodeLog2)
		fprintf(testCodeLog2, "Target Non-inferable BP Accept Count\n");
	if(updateAcceptBPList)
		makeGeometricAssesment(atomMultilist, atomMultiCount, refPDBIndex, basePairNoAlign[(refPDBIndex + 1) % 2], targetSetDiffStat, &targetSetDiffStatCount, fpt, 2, resolutions, 1);
	if(testCodeLog2)
		fprintf(testCodeLog2, "\n");

	if(PRINT_MAPPING)
	{
		if(fpt)
			fclose(fpt);
	}
	return 1;
}


void classifyAcceptCount(basePairInfo compatibleBP, int *cano, int *nonCano, int *others)
{
	if(isCanonicalBP(compatibleBP))
		(*cano)++;
	else if(isWHS_BP(compatibleBP))
		(*nonCano)++;
	else
		(*others)++;
}

void findAtoms(atomInfo atomMultilist[][MAX_ATOM_COUNT], int atomMultiCount[], basePairInfo aBasePair, statisticsBP aStat[], int *statCount, atomInfo *atom[], atomInfo *c2Atoms[], atomInfo *c5Atoms[], int index, edgeAtomName edgeAtomNames[2], edgeAtomCoordinates edgeAtoms[2], int *edgeAtomFlag, int isProposed)
{
	atomInfo tempAtom;
	int j, k;
	for(j = 0; j < 2; j++)
	{
		strcpy(tempAtom.chainId, aBasePair.chainResidue[j]);
		tempAtom.resSeqNo = aBasePair.residueID[j];
		tempAtom.iCode = aBasePair.iCode[j];

		//Get Triangle Atoms (Atoms available in all nucleotides - C1', C2, C5)
		strcpy(tempAtom.atomName, C_ALPHA);
		atom[j] = (atomInfo *)bsearch(&tempAtom, atomMultilist[index], atomMultiCount[index], sizeof(atomInfo), compAtomInfoMap);
		strcpy(tempAtom.atomName, C2);
		c2Atoms[j] = (atomInfo *)bsearch(&tempAtom, atomMultilist[index], atomMultiCount[index], sizeof(atomInfo), compAtomInfoMap);
		strcpy(tempAtom.atomName, C5);
		c5Atoms[j] = (atomInfo *)bsearch(&tempAtom, atomMultilist[index], atomMultiCount[index], sizeof(atomInfo), compAtomInfoMap);
		//Get participating edge atoms in base pairing
		for(k = 0; k < edgeAtomNames[j].acceptorCount; k++)
		{
			strcpy(tempAtom.atomName, edgeAtomNames[j].acceptorAtomNames[k]);
			edgeAtoms[j].acceptorAtomsInfo[k] = (atomInfo *)bsearch(&tempAtom, atomMultilist[index], atomMultiCount[index], sizeof(atomInfo), compAtomInfoMap);
			if(!edgeAtoms[j].acceptorAtomsInfo[k])
				*edgeAtomFlag = 0;
		}
		for(k = 0; k < edgeAtomNames[j].donorCount; k++)
		{
			strcpy(tempAtom.atomName, edgeAtomNames[j].donorAtomNames[k]);
			edgeAtoms[j].donorAtomsInfo[k] = (atomInfo *)bsearch(&tempAtom, atomMultilist[index], atomMultiCount[index], sizeof(atomInfo), compAtomInfoMap);
			if(!edgeAtoms[j].donorAtomsInfo[k])
				*edgeAtomFlag = 0;
		}
		//Update Statistics Info
		if(atom[j])
		{
			if(isProposed)
			{
				aStat[*statCount].residueNamePropose[j] = atom[j]->residueName;
				strcpy(aStat[*statCount].chainIdPropose[j], atom[j]->chainId);
				aStat[*statCount].resSeqNoPropose[j] = atom[j]->resSeqNo;
				aStat[*statCount].iCodePropose[j] = atom[j]->iCode;
			}
			else
			{
				aStat[*statCount].residueNameActual[j] = atom[j]->residueName;
				strcpy(aStat[*statCount].chainIdActual[j], atom[j]->chainId);
				aStat[*statCount].resSeqNoActual[j] = atom[j]->resSeqNo;
				aStat[*statCount].iCodeActual[j] = atom[j]->iCode;
			}
		}
	}
}

/* Do the geometric comparison to decide if a base pair should be includes or excluded.
 * updateAcceptBPList = 1 >> dealing with inferable reference BP, and accept if flexibility within threshold,
 * updateAcceptBPList = 2 >> accept directly,
 * updateAcceptBPList = 3 >> attempt to handle canonical and non canonical differently (NOT in use now),
 * updateAcceptBPList = 4 >> dealing with inferable target BP, and accept if flexibility is greater than threshold,
 */
void makeGeometricAssesment(atomInfo atomMultilist[][MAX_ATOM_COUNT], int atomMultiCount[], int refPDBIndex, basePairMapping aMapping, statisticsBP aStat[], int *statCount, FILE* fpt, int updateAcceptBPList, double resolutions[], int selfMapping)
{
	int i, j, k;
	int edgeAtomFlag;
	int currentBaseAcceptCount = 0, canoAcceptCount = 0, nonCanoAcceptCount = 0, otherAcceptCount = 0;
	atomInfo *atomPropose[2], *atomActual[2], tempAtom;
	atomInfo *c2AtomsPropose[2], *c2AtomsActual[2];
	atomInfo *c5AtomsPropose[2], *c5AtomsActual[2];
	edgeAtomCoordinates edgeAtomsProposed[2], edgeAtomsActual[2];
	edgeAtomName edgeAtomNamesProposed[2], edgeAtomNamesActual[2];
	*statCount = 0;
	double threshold = 0.0;//4.5;//5.50;//4.5//4.875;
	//threshold = ((resolutions[0] + resolutions[1]) / 2);
	//if(resolutions[0] > ERR && resolutions[1] > ERR)
    threshold = (resolutions[0] + resolutions[1]) * root2;//(resolutions[0] + resolutions[1]) * 2;//sqrt(resolutions[0] * resolutions[0] + resolutions[1] * resolutions[1]);

	//printf("Distance difference threshold = %0.2lf\n", threshold);
	if(PRINT_ALIGN_BP_INFO)
		printf("Basepair Count =  %d\n", aMapping.bpCounts);
	for(i = 0; i < aMapping.bpCounts; i++)
	{
		if(updateAcceptBPList == 3 && !isCanonicalBP(aMapping.proposedBasePairs[i]))
			continue;
		edgeAtomFlag = 1;

		//Get participating edge atom names in base pairing

		//For the proposed base pair
		edgeAtomNamesProposed[0] = getEdgeAtomName(aMapping.proposedBasePairs[i].aEdges[0].edge1[0], aMapping.proposedBasePairs[i].residue[0]);
		edgeAtomNamesProposed[1] = getEdgeAtomName(aMapping.proposedBasePairs[i].aEdges[0].edge2[0], aMapping.proposedBasePairs[i].residue[1]);
		edgeAtomsProposed[0].edgeAtomNamesRef = &edgeAtomNamesProposed[0];
		edgeAtomsProposed[1].edgeAtomNamesRef = &edgeAtomNamesProposed[1];
		//For the actual base pair
		edgeAtomNamesActual[0] = getEdgeAtomName(aMapping.actualBasePairs[i].aEdges[0].edge1[0], aMapping.actualBasePairs[i].residue[0]);
		edgeAtomNamesActual[1] = getEdgeAtomName(aMapping.actualBasePairs[i].aEdges[0].edge2[0], aMapping.actualBasePairs[i].residue[1]);
		edgeAtomsActual[0].edgeAtomNamesRef = &edgeAtomNamesActual[0];
		edgeAtomsActual[1].edgeAtomNamesRef = &edgeAtomNamesActual[1];

		//Find Atom Info of 'Proposed' Base Pairs
		int proposedIndex = (refPDBIndex + 1) % 2;
		//if(selfMapping)
			//proposedIndex = refPDBIndex;

		findAtoms(atomMultilist, atomMultiCount, aMapping.proposedBasePairs[i], aStat, statCount, atomPropose, c2AtomsPropose, c5AtomsPropose, proposedIndex, edgeAtomNamesProposed, edgeAtomsProposed, &edgeAtomFlag, 1);
		//Find Atom Info of 'Actual' Base Pairs
		int actualIndex = refPDBIndex;
		if(selfMapping)
			actualIndex = proposedIndex; //???
		findAtoms(atomMultilist, atomMultiCount, aMapping.actualBasePairs[i], aStat, statCount, atomActual, c2AtomsActual, c5AtomsActual, actualIndex, edgeAtomNamesActual, edgeAtomsActual, &edgeAtomFlag, 0);
		//If all C' atoms are found
		if(atomPropose[0] && atomActual[0] && atomPropose[1] && atomActual[1])
		{
			int n = ATOM_PAIR_COUNT;
			double distance[2], c2C5Flag = 0;

			//Distance between C' atoms in the actual and proposed base pairs
			aStat[*statCount].aDist.actualDist = distance3D(atomActual[0]->coOrd, atomActual[1]->coOrd);
			aStat[*statCount].aDist.proposedDist = distance3D(atomPropose[0]->coOrd, atomPropose[1]->coOrd);
			aStat[*statCount].distDiff = aStat[*statCount].aDist.proposedDist - aStat[*statCount].aDist.actualDist;
			edgeAtomPair aEdgeAtomPairProposed[ATOM_PAIR_COUNT];


			//Find n closest atom for the proposed pair (will be compared with actual pairs later)
			if(edgeAtomFlag)
				closestNInteractions(edgeAtomsProposed, &n, aEdgeAtomPairProposed);

			//If C2 and C5 atoms are found
			if(c2AtomsPropose[0] && c2AtomsActual[0] && c2AtomsPropose[1] && c2AtomsActual[1]
			   && c5AtomsPropose[0] && c5AtomsActual[0] && c5AtomsPropose[1] && c5AtomsActual[1])
			{
				distance[0] = distance3D(c2AtomsActual[0]->coOrd, c2AtomsActual[1]->coOrd) - distance3D(c2AtomsPropose[0]->coOrd, c2AtomsPropose[1]->coOrd);
				distance[1] = distance3D(c5AtomsActual[0]->coOrd, c5AtomsActual[1]->coOrd) - distance3D(c5AtomsPropose[0]->coOrd, c5AtomsPropose[1]->coOrd);
				c2C5Flag = 1;
			}

			//Accept base pair when it's within a threshlond distance
			if(
				updateAcceptBPList == 2
				||
				(
						updateAcceptBPList == 4 && fabs(aStat[*statCount].distDiff) >= threshold
						&& (!c2C5Flag || (fabs(distance[0]) >= threshold && fabs(distance[1]) >= threshold))
						&& (edgeAtomFlag && isSymmetricBP(aMapping.actualBasePairs[i], aMapping.proposedBasePairs[i], n, aEdgeAtomPairProposed, edgeAtomsActual, threshold, 0))
				)
				||
				(
						(updateAcceptBPList == 1 || updateAcceptBPList == 3) && fabs(aStat[*statCount].distDiff) <= threshold
						&&(!c2C5Flag || (fabs(distance[0]) <= threshold && fabs(distance[1]) <= threshold))
						&& (edgeAtomFlag && isSymmetricBP(aMapping.actualBasePairs[i], aMapping.proposedBasePairs[i], n, aEdgeAtomPairProposed, edgeAtomsActual, threshold, 1))
				)
			  )
			{
				basePairAccepted[bpAccCount++] = aMapping.proposedBasePairs[i];
				basePairAccepted[bpAccCount - 1].bpSrc = bpSource;
				classifyAcceptCount(aMapping.proposedBasePairs[i], &canoAcceptCount, &nonCanoAcceptCount, &otherAcceptCount);
				currentBaseAcceptCount++;
			}

			for(j = 0; j < 2; j++)
			{
				aStat[*statCount].coOrdDiff[j].x = atomActual[j]->coOrd.x - atomPropose[j]->coOrd.x;
				aStat[*statCount].coOrdDiff[j].y = atomActual[j]->coOrd.y - atomPropose[j]->coOrd.y;
				aStat[*statCount].coOrdDiff[j].z = atomActual[j]->coOrd.z - atomPropose[j]->coOrd.z;
				aStat[*statCount].coOrdsPropose[j] = atomPropose[j]->coOrd;
				aStat[*statCount].coOrdsActual[j] = atomActual[j]->coOrd;
			}
			aStat[*statCount].aEdgeActual = aMapping.actualBasePairs[i].aEdges[0];
			aStat[*statCount].glycosidicOrientation = aMapping.actualBasePairs[i].glycosidicOrientation;
			(*statCount)++;

		}
		else
		{
			//If the case of direct include, previous atom finding conditions does not matter
			if(updateAcceptBPList == 2)
			{
				basePairAccepted[bpAccCount++] = aMapping.proposedBasePairs[i];
				basePairAccepted[bpAccCount - 1].bpSrc = bpSource;
				classifyAcceptCount(aMapping.proposedBasePairs[i], &canoAcceptCount, &nonCanoAcceptCount, &otherAcceptCount);
				currentBaseAcceptCount++;
			}
			//basePairAccepted[bpAccCount++] = aMapping.proposedBasePairs[i];
			if(PRINT_BASE_MAPPING_ERRORS)
			{
				printf("!!!Error in base pairs finding!!!\n");
				printPairInfoOnScreen(aMapping.proposedBasePairs[i]);
				printPairInfoOnScreen(aMapping.actualBasePairs[i]);
				getchar();
			}
		}
	}

	qsort(aStat, *statCount, sizeof(statisticsBP), compStatsDistDiff);
	//qsort(aStat, *statCount, sizeof(statisticsBP), compStatsCoOrdDiffZ);
	if(PRINT_MAPPING)
		printStats(*statCount, aStat, fpt);
	if(testCodeLog2)
	{
		fprintf(testCodeLog2, "Total = %d\n", currentBaseAcceptCount);
		fprintf(testCodeLog2, "Canonical = %d, Non-Canonical = %d, Others = %d\n", canoAcceptCount, nonCanoAcceptCount, otherAcceptCount);
	}
	//printf("Stat Count = %d, Accepted BP Count = %d\n", *statCount, bpAccCount);
}

int isSymmetricBP(basePairInfo a, basePairInfo b, int n, edgeAtomPair aEdgeAtomPairProposed[], edgeAtomCoordinates edgeAtomsActual[], double threshold, int isUpperBound)
{
	//return 1;
	int isSymmetric = 0;
	/*if(((isPurine(a.residue[0]) && isPurine(b.residue[0])) || (isPyrimidine(a.residue[0]) && isPyrimidine(b.residue[0]))) &&
		((isPurine(a.residue[1]) && isPurine(b.residue[1])) || (isPyrimidine(a.residue[1]) && isPyrimidine(b.residue[1]))))
		isSymmetric = 1;*/
	/*else if(((isPurine(a.residue[0]) && isPurine(b.residue[1])) || (isPyrimidine(a.residue[0]) && isPyrimidine(b.residue[1]))) &&
		((isPurine(a.residue[1]) && isPurine(b.residue[0])) || (isPyrimidine(a.residue[1]) && isPyrimidine(b.residue[0]))))
		isSymmetric = 1;*/
	int i, flag = 0;
	double distDiff;
	edgeAtomPair aEdgeAtomPairActual[ATOM_PAIR_COUNT];

	//If isSymmetric or identically ordered base base-pair
	if((a.residue[0] == b.residue[0] && a.residue[1] == b.residue[1]) || isSymmetric)
	{
		//returns 1e9 if distance could not be found
		getCorrespondingDistance(edgeAtomsActual, 0, n, aEdgeAtomPairProposed, aEdgeAtomPairActual);

		flag = 1;

		for(i = 0; i < n; i++)
		{
			distDiff = fabs(aEdgeAtomPairActual[i].distance - aEdgeAtomPairProposed[i].distance);
			if(distDiff > INF_DOUBLE / 2.0)
				continue;
			if((isUpperBound && !(distDiff <= threshold)) ||
				(!isUpperBound && !(distDiff >= threshold)))
			{
				flag = 0;
				break;
			}
		}
	}
	//If not already approved in the previous conditions
	if(!flag)
		//If isSymmetric or reverse ordered base base-pair
		if((a.residue[0] == b.residue[1] && a.residue[1] == b.residue[0]) || isSymmetric)
		{
			//returns 1e9 if distance could not be found
			getCorrespondingDistance(edgeAtomsActual, 1, n, aEdgeAtomPairProposed, aEdgeAtomPairActual);

			flag = 1;
			for(i = 0; i < n; i++)
			{
				distDiff = fabs(aEdgeAtomPairActual[i].distance - aEdgeAtomPairProposed[i].distance);
				if(distDiff > INF_DOUBLE / 2.0)
					continue;
				if( (isUpperBound && !(distDiff <= threshold)) ||
					(!isUpperBound && !(distDiff >= threshold))
					)
				{
					flag = 0;
					break;
				}
			}
		}
	return flag;
}

void generateSetDifference(sequenceMap pdbMap[], int refPDBIndex, basePairInfo basePairMapping1[], int count1, basePairInfo basePairMapping2[], int count2,
		basePairMapping *basePairSetDiff, basePairMapping *basePairCommon, basePairMapping *basePairNoAlign, basePairMapping *basePairTargetSetDiff, basePairMapping *basePairTargetNoAlign, int updateAcceptBPList)
{
	//If needed to be attempted to refine alignment based on helix based anchors
	//getAnchors(pdbMap[0].seqId, basePairMapping1, count1, pdbMap[1].seqId, basePairMapping2, count2);

	int i = 0, j = 0, cmp;
	basePairInfo compBPInfo;
	basePairSetDiff->bpCounts = basePairCommon->bpCounts = basePairNoAlign->bpCounts = 0;
	basePairSetDiff->cannonicalCount = basePairCommon->cannonicalCount = basePairNoAlign->cannonicalCount = 0;
	basePairSetDiff->nonCannonicalCount = basePairCommon->nonCannonicalCount = basePairNoAlign->nonCannonicalCount = 0;
	basePairSetDiff->otherCount = basePairCommon->otherCount = basePairNoAlign->otherCount = 0;

	basePairTargetNoAlign->bpCounts = basePairTargetSetDiff->bpCounts = 0;
	basePairTargetNoAlign->cannonicalCount = basePairTargetSetDiff->cannonicalCount = 0;
	basePairTargetNoAlign->nonCannonicalCount = basePairTargetSetDiff->nonCannonicalCount = 0;
	basePairTargetNoAlign->otherCount = basePairTargetSetDiff->otherCount = 0;

	int printFlag = 0, notEqualOnlyEdgeDiff = 0, prevCompEdge = COMPARE_EDGE;

	while(i < count1 && j < count2)
	{
		//if(isCanonicalBP(basePairMapping1[i]) || isWHS_BP(basePairMapping1[i]))
		COMPARE_EDGE = 1;

		if(getCompatibleBasePair(basePairMapping1[i], pdbMap, refPDBIndex, &compBPInfo))
		{
			if(POS_ALIGN == 0)// && !isCanonicalBP(basePairMapping1[i]) && !isWHS_BP(basePairMapping1[i]))
				cmp = compBPResidue(&compBPInfo, &basePairMapping2[j]); //????
			else
				cmp = compBPIndicesAndResidue(&compBPInfo, &basePairMapping2[j]); //????

			//if(COMP_EDGE_ONLY && updateAcceptBPList)
				//cmp = compareEdges(compBPInfo, basePairMapping2[j]);

			if(cmp == 0)
			{
				/*if(compBPIndicesAndResidue(&compBPInfo, &basePairMapping2[j]) == 0)
				{
					compBPInfo.edgeCount = basePairMapping2[j].edgeCount;
					for(k = 0; k < basePairMapping2[j].edgeCount; k++)
						compBPInfo.aEdges[k] = basePairMapping2[j].aEdges[k];
				}*/
				if(printFlag)
				{
					printf("Equal i = %d, j = %d\n", i, j);
					printPairMCAnnotateOnScreen(basePairMapping1[i]);
					printPairMCAnnotateOnScreen(compBPInfo);
					printPairMCAnnotateOnScreen(basePairMapping2[j]);
					getchar();
				}
				//for(k = 0; k < compBPInfo.edgeCount; k++)
					//compBPInfo.aEdges[k] = basePairMapping2[j].aEdges[k];
				basePairCommon->actualBasePairs[basePairCommon->bpCounts] = basePairMapping1[i];
				basePairCommon->proposedBasePairs[basePairCommon->bpCounts] = compBPInfo;
				classifyCounts(compBPInfo, basePairCommon, 1);
				i++;
				j++;
				//printFlag = 0;
			}
			else
			{
				//printFlag = 1;
				//Update the edges
				if(basePairMapping1[i].residue[0] == basePairMapping2[j].residue[0] && basePairMapping1[i].residue[1] == basePairMapping2[j].residue[1])
					notEqualOnlyEdgeDiff++;

				cmp = compBPIndices(&compBPInfo, &basePairMapping2[j]); //????

				if(printFlag)// && cmp2 == 0)
				{
					printf("Not Equal i = %d, j = %d\n", i, j);
					printPairMCAnnotateOnScreen(basePairMapping1[i]);
					printPairMCAnnotateOnScreen(compBPInfo);
					printPairMCAnnotateOnScreen(basePairMapping2[j]);
					getchar();
					//printFlag = 1;
				}

				if(cmp <= 0)// || cmp2 == 0)
				{
					basePairSetDiff->actualBasePairs[basePairSetDiff->bpCounts] = basePairMapping1[i];
					basePairSetDiff->proposedBasePairs[basePairSetDiff->bpCounts] = compBPInfo;
					classifyCounts(compBPInfo, basePairSetDiff, 0);
					i++;
					if(cmp == 0) //When mapped to a target BP, but mismatched edges
						j++;
				}
				else
				{
					//Mismatched basepairs from 2nd list???
					if(getCompatibleBasePair(basePairMapping2[j], pdbMap, (refPDBIndex + 1) % 2, &compBPInfo))
					{
						basePairTargetSetDiff->actualBasePairs[basePairTargetSetDiff->bpCounts] = compBPInfo;
						basePairTargetSetDiff->proposedBasePairs[basePairTargetSetDiff->bpCounts] = basePairMapping2[j];
						classifyTargetCounts(compBPInfo, basePairTargetSetDiff);
						//basePairTargetSetDiff->bpCounts++;
					}
					else
					{
						basePairTargetNoAlign->actualBasePairs[basePairTargetNoAlign->bpCounts] = basePairMapping2[j];
						basePairTargetNoAlign->proposedBasePairs[basePairTargetNoAlign->bpCounts] = basePairMapping2[j];
						classifyTargetCounts(basePairMapping2[j], basePairTargetNoAlign);
						//basePairTargetNoAlign->bpCounts++;
					}
					probable2ndListDiffCnt++;
					j++;
				}
			}
		}
		else
		{
			if(printFlag)
			{
				printf("No Mapping i = %d, j = %d\n", i, j);
				printPairMCAnnotateOnScreen(basePairMapping1[i]);
				getchar();
			}
			basePairNoAlign->actualBasePairs[basePairNoAlign->bpCounts] = basePairMapping1[i];
			basePairNoAlign->proposedBasePairs[basePairNoAlign->bpCounts] = basePairMapping1[i];
			classifyCounts(basePairMapping1[i], basePairNoAlign, NO_ALIGN_STAT);
			//basePairNoAlign->bpCounts++;
			i++;
		}
		COMPARE_EDGE = prevCompEdge;
	}

	while(i < count1)
	{
		if(getCompatibleBasePair(basePairMapping1[i], pdbMap, refPDBIndex, &compBPInfo))
		{
			basePairSetDiff->actualBasePairs[basePairSetDiff->bpCounts] = basePairMapping1[i];
			basePairSetDiff->proposedBasePairs[basePairSetDiff->bpCounts] = compBPInfo;
			classifyCounts(compBPInfo, basePairSetDiff, 0);
			//printf("Test1\n");
			//getchar();
		}
		else
		{
			basePairNoAlign->actualBasePairs[basePairNoAlign->bpCounts] = basePairMapping1[i];
			basePairNoAlign->proposedBasePairs[basePairNoAlign->bpCounts] = basePairMapping1[i];
			classifyCounts(basePairMapping1[i], basePairNoAlign, NO_ALIGN_STAT);
			//basePairNoAlign->bpCounts++;
		}
		i++;
	}

	while(j < count2)
	{
		if(getCompatibleBasePair(basePairMapping2[j], pdbMap, (refPDBIndex + 1) % 2, &compBPInfo))
		{
			basePairTargetSetDiff->actualBasePairs[basePairTargetSetDiff->bpCounts] = compBPInfo;
			basePairTargetSetDiff->proposedBasePairs[basePairTargetSetDiff->bpCounts] = basePairMapping2[j];
			classifyTargetCounts(compBPInfo, basePairTargetSetDiff);
			//basePairTargetSetDiff->bpCounts++;
			//printf("Test1\n");
			//getchar();
		}
		else
		{
			basePairTargetNoAlign->actualBasePairs[basePairTargetNoAlign->bpCounts] = basePairMapping2[j];
			basePairTargetNoAlign->proposedBasePairs[basePairTargetNoAlign->bpCounts] = basePairMapping2[j];
			classifyTargetCounts(basePairMapping2[j], basePairTargetNoAlign);
			//basePairTargetNoAlign->bpCounts++;
		}
		j++;
	}
	if(PRINT_ALIGN_BP_INFO)
	{
		printf("---------------Not Equal, Only Edge Diff = %d---------------\n", notEqualOnlyEdgeDiff);
		//getchar();
		printf("i = %d count1 = %d count2 = %d\nbpDiffCnt = %d bpCommonCount = %d bpNoAlignCount = %d\n", i, count1, count2, basePairSetDiff->bpCounts, basePairCommon->bpCounts, basePairNoAlign->bpCounts);
		//printf("i = %d count1 = %d count2 = %d bpDiffCount = %d bpCommonCount = %d\n", i, count1, count2, basePairSetDiff->bpCounts, basePairCommon->bpCounts);
	}
}

int getCompatibleBasePair(basePairInfo x, sequenceMap pdbMap[], int refPDBIndex, basePairInfo *compatibleBP)
{
	//Taking Edge (??) and Other bp interaction info from reference
	//Only chain, resPos and residue is going to be replaced from target
	//Consider updating the Edge from target too!!!
	*compatibleBP = x;
	int i, j, k;
	posMap *aPos, tempPos;

	for(i = 0; i < 2; i++)
	{
		tempPos.resPos = x.residueID[i];
		tempPos.iCode = x.iCode[i];
		for(j = 0; j < pdbMap[refPDBIndex].chainCount; j++)
			if(strcmp(pdbMap[refPDBIndex].aChainMap[j].chainId, x.chainResidue[i]) == 0)
			{
				aPos = NULL;
				//Find a residue in the residue list
				//RNAView gives error due to iCode mismanagement
				aPos = (posMap *)bsearch(&tempPos, pdbMap[refPDBIndex].aChainMap[j].resMap, pdbMap[refPDBIndex].aChainMap[j].resCount, sizeof(posMap), mappedSeqComp);
				if(aPos)
				{
					strcpy(compatibleBP->chainResidue[i], pdbMap[refPDBIndex].aChainMap[j].mappedChainId);
					//Mapped array index
					compatibleBP->residueID[i] = aPos->seq2MappedPos;
					if(aPos->seq2MappedPos != NO_ALIGN_INDEX)
					{
						//For a valid mapped index in target sequence, get corresponding
						for(k = 0; k < pdbMap[(refPDBIndex + 1) % 2].chainCount; k++)
							if(strcmp(pdbMap[(refPDBIndex + 1) % 2].aChainMap[k].chainId, compatibleBP->chainResidue[i]) == 0)
							{
								//Mapped PDB index
								//compatibleBP->residueID[i] =  pdbMap[(refPDBIndex + 1) % 2].aChainMap[k].resMap[aPos->seq2MappedPos].resPos;
								//compatibleBP->iCode[i] =  pdbMap[(refPDBIndex + 1) % 2].aChainMap[k].resMap[aPos->seq2MappedPos].iCode;
								//compatibleBP->residue[i] =  pdbMap[(refPDBIndex + 1) % 2].aChainMap[k].resMap[aPos->seq2MappedPos].residue;
								compatibleBP->residueID[i] =  pdbMap[(refPDBIndex + 1) % 2].aChainMap[k].resMap[aPos->seq2MappedPos].resPos;
								compatibleBP->iCode[i] =  pdbMap[(refPDBIndex + 1) % 2].aChainMap[k].resMap[aPos->seq2MappedPos].iCode;
								compatibleBP->residue[i] =  pdbMap[(refPDBIndex + 1) % 2].aChainMap[k].resMap[aPos->seq2MappedPos].residue;
							}
					}
				}
				else
				{
					if(PRINT_BASE_MAPPING_ERRORS)
					{
						printf("Error!! Base pair generating method has generated pairs with invalid '%s'%d - \'%s\'%d of %s\n", x.chainResidue[i], x.residueID[i], x.chainResidue[(i + 1)%2], x.residueID[(i + 1)%2], pdbMap[refPDBIndex].seqId);
						printPairInfoOnScreen(x);
					}
				}
				break;
			}
	}

	if(compatibleBP->residueID[0] != NO_ALIGN_INDEX && compatibleBP->residueID[1] != NO_ALIGN_INDEX)
	{
		//printPairMCAnnotateOnScreen(x);
		//printPairMCAnnotateOnScreen(*compatibleBP);
		return 1;
	}
	else
		return 0;
}

void classifyTargetCounts(basePairInfo compatibleBP, basePairMapping *basePairSet)
{
	if(isCanonicalBP(compatibleBP))
	{
		basePairSet->bpCounts++;
		basePairSet->cannonicalCount++;
	}
	else if(isWHS_BP(compatibleBP))
	{
		basePairSet->bpCounts++;
		basePairSet->nonCannonicalCount++;
	}
	else
	{
		basePairSet->bpCounts++;
		basePairSet->otherCount++;
	}
}

void classifyCounts(basePairInfo compatibleBP, basePairMapping *basePairSet, int isCommon)
{
	if(isCanonicalBP(compatibleBP))
	{
		if(isCommon == 1)
			commonCBP++;
		else if(isCommon == 0)
			diffCBP++;
		basePairSet->bpCounts++;
		basePairSet->cannonicalCount++;
	}
	else if(isWHS_BP(compatibleBP))
	{
		if(isCommon == 1)
			commonNCBP++;
		else if(isCommon == 0)
			diffNCBP++;
		basePairSet->bpCounts++;
		basePairSet->nonCannonicalCount++;
	}
	else
	{
		if(isCommon == 1)
			commonOthers++;
		else if(isCommon == 0)
			diffOthers++;
		basePairSet->bpCounts++;
		basePairSet->otherCount++;
	}
}

int compBPIndices(const void *a, const void *b)
{
	basePairInfo *x = (basePairInfo*)a;
	basePairInfo *y = (basePairInfo*)b;
	if(strcmp(x->chainResidue[0], y->chainResidue[0]) == 0)
		if(strcmp(x->chainResidue[1], y->chainResidue[1]) == 0)
			if(x->residueID[0] == y->residueID[0])
				if(x->iCode[0] == y->iCode[0])
					if(x->residueID[1] == y->residueID[1])
						return x->iCode[1] - y->iCode[1];
					else
						return x->residueID[1] - y->residueID[1];
				else
					return x->iCode[0] - y->iCode[0];
			else
				return x->residueID[0] - y->residueID[0];
		else
			return strcmp(x->chainResidue[1], y->chainResidue[1]);
	else
		return strcmp(x->chainResidue[0], y->chainResidue[0]);
}

int compBPIndicesAndResidue(const void *a, const void *b)
{
	basePairInfo *x = (basePairInfo*)a;
	basePairInfo *y = (basePairInfo*)b;
	if(strcmp(x->chainResidue[0], y->chainResidue[0]) == 0)
		if(strcmp(x->chainResidue[1], y->chainResidue[1]) == 0)
			if(x->residueID[0] == y->residueID[0])
				if(x->iCode[0] == y->iCode[0])
					if(x->residueID[1] == y->residueID[1])
						if(x->iCode[1] == y->iCode[1])
							if(x->residue[0] == y->residue[0])
							{
								if(COMPARE_EDGE)
								{
									if(x->residue[1] == y->residue[1])
										return compareEdges(*x, *y);
								}
								return x->residue[1] - y->residue[1];
							}
							else
								return x->residue[0] - y->residue[0];
						else
							return x->iCode[1] - y->iCode[1];
					else
						return x->residueID[1] - y->residueID[1];
				else
					return x->iCode[0] - y->iCode[0];
			else
				return x->residueID[0] - y->residueID[0];
		else
			return strcmp(x->chainResidue[1], y->chainResidue[1]);
	else
		return strcmp(x->chainResidue[0], y->chainResidue[0]);
}

int compBPResidue(const void *a, const void *b)
{
	basePairInfo *x = (basePairInfo*)a;
	basePairInfo *y = (basePairInfo*)b;
	if(x->residue[0] == y->residue[0])
	{
		if(COMPARE_EDGE)
		{
			if(x->residue[1] == y->residue[1])
				return compareEdges(*x, *y);
		}
		return x->residue[1] - y->residue[1];
	}
	else
		return x->residue[0] - y->residue[0];
}

int compStatsDistDiff(const void *a, const void *b)
{
	statisticsBP *x = (statisticsBP*)a;
	statisticsBP *y = (statisticsBP*)b;
	double distDiffX = fabs(x->distDiff);
	double distDiffY = fabs(y->distDiff);
	if(fabs(distDiffX - distDiffY) < ERR)
		return 0;
	else if(distDiffX > distDiffY)
		return 1;
	else
		return -1;
}

int compStatsDistPropose(const void *a, const void *b)
{
	statisticsBP *x = (statisticsBP*)a;
	statisticsBP *y = (statisticsBP*)b;
	if(fabs(x->aDist.proposedDist - y->aDist.proposedDist) < ERR)
	{
		if(fabs(x->aDist.actualDist - y->aDist.actualDist) < ERR)
			return 0;
		else if(x->aDist.actualDist > y->aDist.actualDist)
			return 1;
		else
			return -1;
	}
	else if(x->aDist.proposedDist > y->aDist.proposedDist)
		return 1;
	else
		return -1;

}

int compStatsDistActual(const void *a, const void *b)
{
	statisticsBP *x = (statisticsBP*)a;
	statisticsBP *y = (statisticsBP*)b;
	if(fabs(x->aDist.actualDist - y->aDist.actualDist) < ERR)
	{
		if(fabs(x->aDist.proposedDist - y->aDist.proposedDist) < ERR)
			return 0;
		else if(x->aDist.proposedDist > y->aDist.proposedDist)
			return 1;
		else
			return -1;
	}
	else if(x->aDist.actualDist > y->aDist.actualDist)
		return 1;
	else
		return -1;

}

int compStatsCoOrdDiffX(const void *a, const void *b)
{
	statisticsBP *x = (statisticsBP*)a;
	statisticsBP *y = (statisticsBP*)b;
	if(fabs(x->coOrdDiff[0].x - y->coOrdDiff[0].x) < ERR)
	{
		if(fabs(x->coOrdDiff[1].x - y->coOrdDiff[1].x) < ERR)
			return 0;
		else if(x->coOrdDiff[1].x > y->coOrdDiff[1].x)
			return 1;
		else
			return -1;
	}
	else if(x->coOrdDiff[0].x > y->coOrdDiff[0].x)
		return 1;
	else
		return -1;

}

int compStatsCoOrdDiffY(const void *a, const void *b)
{
	statisticsBP *x = (statisticsBP*)a;
	statisticsBP *y = (statisticsBP*)b;
	if(fabs(x->coOrdDiff[0].y - y->coOrdDiff[0].y) < ERR)
	{
		if(fabs(x->coOrdDiff[1].y - y->coOrdDiff[1].y) < ERR)
			return 0;
		else if(x->coOrdDiff[1].y > y->coOrdDiff[1].y)
			return 1;
		else
			return -1;
	}
	else if(x->coOrdDiff[0].y > y->coOrdDiff[0].y)
		return 1;
	else
		return -1;

}

int compStatsCoOrdDiffZ(const void *a, const void *b)
{
	statisticsBP *x = (statisticsBP*)a;
	statisticsBP *y = (statisticsBP*)b;
	if(fabs(x->coOrdDiff[0].z - y->coOrdDiff[0].z) < ERR)
	{
		if(fabs(x->coOrdDiff[1].z - y->coOrdDiff[1].z) < ERR)
			return 0;
		else if(x->coOrdDiff[1].z > y->coOrdDiff[1].z)
			return 1;
		else
			return -1;
	}
	else if(x->coOrdDiff[0].z > y->coOrdDiff[0].z)
		return 1;
	else
		return -1;

}

int compAtomInfoMap(const void *a, const void *b)
{
	atomInfo *x = (atomInfo*)a;
	atomInfo *y = (atomInfo*)b;
	if(strcmp(x -> chainId, y -> chainId) == 0)
		if(x -> resSeqNo == y -> resSeqNo)
			if(x -> iCode == y -> iCode)
				return strcmp(x -> atomName, y -> atomName);
			else
				return x -> iCode - y -> iCode;
		else
			return x -> resSeqNo - y -> resSeqNo;
	else
		return strcmp(x -> chainId, y -> chainId);
}

void alignPDBs(sequenceMap refSeqMap[], sequenceMap refMap[], sequenceMap pdbTempMap[], sequenceMap pdbMap[], double resolution[])
{
	int i, j, k, seqCount = 2;

	for(i = 0; i < seqCount; i++)
	{
		pdbMap[i] = pdbTempMap[i];
		strcpy(pdbMap[i].seqId, refMap[i].seqId);
		//Manually updating the resolution info (TEMPORARY CODE)
		pdbMap[i].resolution = resolution[i];
		for(j = 0; j < pdbTempMap[i].chainCount; j++)
		{
			strcpy(pdbMap[i].aChainMap[j].mappedChainId, refMap[(i + 1) % seqCount].aChainMap[j].chainId);
			for(k = 0; k < pdbTempMap[i].aChainMap[j].resCount; k++)
			{
				pdbMap[i].aChainMap[j].resMap[k].isMatch = 0;
				pdbMap[i].aChainMap[j].resMap[k].seq2MappedPos = NO_ALIGN_INDEX;

				int mappedPos = pdbTempMap[i].aChainMap[j].resMap[k].seq2MappedPos;
				//If PDB residue does have gap with reference
				if(mappedPos != NO_ALIGN_INDEX)
				{
					int mappedPos2 = refSeqMap[i].aChainMap[j].resMap[mappedPos].seq2MappedPos;
					//If reference1 has gap with reference2
					if(mappedPos2 != NO_ALIGN_INDEX)
					{
						//Check the all matched case
						if(pdbTempMap[i].aChainMap[j].resMap[k].isMatch
						&& refSeqMap[i].aChainMap[j].resMap[mappedPos].isMatch
						&& refMap[(i + 1) % seqCount].aChainMap[j].resMap[mappedPos2].isMatch)
								pdbMap[i].aChainMap[j].resMap[k].isMatch = 1;
						pdbMap[i].aChainMap[j].resMap[k].seq2MappedPos = refMap[(i + 1) % seqCount].aChainMap[j].resMap[mappedPos2].seq2MappedPos;
					}
				}

			}
		}
	}
}

//Need fixing to give desired output
int printAlign(char outputFile[], sequenceMap clMap1, sequenceMap clMap2, int residuePerLine)
{
	int j, k, mismatchCount = 0, prevIndex = -1;
	//strcpy(pdbDummyMap1.seqId, clMap1.PDBId);
	//pdbDummyMap1.chainCount = clMap1.chainCount;

	for(j = 0; j < clMap1.chainCount; j++)
	{
		mismatchCount = 0;
		for(k = 0; k < clMap1.aChainMap[j].resCount; k++)
		{
			if(clMap1.aChainMap[j].resMap[k].seq2MappedPos == NO_ALIGN_INDEX)
				mismatchCount++;
			else
			{
				mismatchCount += clMap1.aChainMap[j].resMap[k].seq2MappedPos - prevIndex - 1;

				if(clMap1.aChainMap[j].resMap[k].residue != clMap2.aChainMap[j].resMap[clMap1.aChainMap[j].resMap[k].seq2MappedPos].residue)
					mismatchCount++;
				prevIndex = clMap1.aChainMap[j].resMap[k].seq2MappedPos;
			}
		}
		printf("Ref Sequence Map Mismatch Count = %d\n", mismatchCount);
	}
	return 1;
}

int printPdbAlign(char outputFile[], sequenceMap pdbMap1, sequenceMap pdbMap2, int residuePerLine)
{
	//residuePerLine = 65;
	FILE *fpt = fopen(outputFile, "w");
	if(!fpt)
	{
		printf("%s File Not Found\n", outputFile);
		return 0;
	}

	int i, j, k, seqCount = 2, alignCount[2] = {0, 0}, alignLen = 0;
	char *seq[2];
	char *seqAlign;
	char seqId[2][PDB_ID_SZ];

	for(j = 0; j < pdbMap1.chainCount; j++)
		alignLen += pdbMap1.aChainMap[j].resCount;
	for(j = 0; j < pdbMap1.chainCount; j++)
		alignLen += pdbMap2.aChainMap[j].resCount;
	seq[0] = (char *) malloc(alignLen * sizeof(char));
	seq[1] = (char *) malloc(alignLen * sizeof(char));
	seqAlign = (char *) malloc(alignLen * sizeof(char));

	strcpy(seqId[0], pdbMap1.seqId);
	strcpy(seqId[1], pdbMap2.seqId);

	generateAlignSequence(pdbMap1, pdbMap2, seq[0], &alignCount[0], seqAlign, 1);
	generateAlignSequence(pdbMap2, pdbMap1, seq[1], &alignCount[1], seqAlign, 0);

	int count[2] = {0, 0};

	int mismatchCount = 0, gap[2] = {0, 0}, match[2] = {0, 0};

	for(k = 0; k < alignCount[0]; k += residuePerLine)
	{
		for(i = 0; i < seqCount; i++)
		{
			fprintf(fpt, "%4s%4s", seqId[i], " ");
			if(i == 0)
			{
				printResSeqFile(pdbMap1.aChainMap[0].resMap[count[i]].resPos, pdbMap1.aChainMap[0].resMap[count[i]].iCode, 6, fpt);
				fprintf(fpt, "(%4d)%4s", count[i] + 1, " ");
			}
			else
			{
				printResSeqFile(pdbMap2.aChainMap[0].resMap[count[i]].resPos, pdbMap2.aChainMap[0].resMap[count[i]].iCode, 6, fpt);
				fprintf(fpt, "(%4d)%4s", count[i] + 1, " ");
			}

			for(j = k; j < k + residuePerLine && j < alignCount[0]; j++)
			{
				fprintf(fpt, "%c", seq[i][j]);
				if(seq[i][j] != '-')
				{
					count[i]++;
					match[i]++;
				}
				else
				{
					gap[i]++;
					match[(i + 1) % 2]--;
					//mismatchCount++;
				}
			}
			fprintf(fpt, "\n");
		}
		fprintf(fpt, "%24s"," ");
		for(j = k; j < k + residuePerLine && j < alignCount[0]; j++)
		{
			if(seqAlign[j] == 'x')
			{
				mismatchCount++;
				match[0]--;
				match[1]--;
			}
			fprintf(fpt, "%c", seqAlign[j]);
		}
		fprintf(fpt, "\n\n");
	}

	fprintf(fpt, "Match = %d/%d Mismatch = %d, Gap1 = %d Gap2 = %d\n", match[0], match[1], mismatchCount, gap[1], gap[0]);
	fprintf(fpt, "Diff_Ref1 = %d Diff_Ref2 = %d\n", mismatchCount + gap[1], mismatchCount + gap[0]);
	fprintf(fpt, "Seq1 Len = %d Seq2 Len = %d\n", pdbMap1.aChainMap[0].resCount, pdbMap2.aChainMap[0].resCount);
	fprintf(fpt, "%d %d %d\n", alignCount[0], alignCount[1], alignLen);

	int seq1Len = (match[0] + gap[1] + mismatchCount);
	int seq2Len = (match[1] + gap[0] + mismatchCount);
	double identity1 = (match[0] * 100.0) / seq1Len;
	double identity2 = (match[1] * 100.0)/ seq2Len;
	printf("%4s vs %4s\n", seqId[0], seqId[1]);
	printf("Seq1 = %d, Seq2 = %d, Match = %d Mismatch = %d, Gap1 = %d Gap2 = %d: \nIdentity 1 = %0.2lf%% Identity 2 = %0.2lf%%\n", seq1Len, seq2Len, match[0], mismatchCount, gap[1], gap[0], identity1, identity2);

	if(seq[0])
		free(seq[0]);
	if(seq[1])
		free(seq[1]);
	if(seqAlign)
		free(seqAlign);

	if(fpt)
		fclose(fpt);
	return 1;
}

void generateAlignSequence(sequenceMap pdbMap1, sequenceMap pdbMap2, char seq[], int *alignCount, char seqAlign[], int updateSeqAlign)
{
	int j, k, l, prevIndex = -1;
	for(j = 0; j < pdbMap1.chainCount; j++)
	{
		for(k = 0; k < pdbMap1.aChainMap[j].resCount; k++)
		{
			for(l = prevIndex + 1; l < pdbMap1.aChainMap[j].resMap[k].seq2MappedPos; l++)
			{
				if(updateSeqAlign == 1)
					sprintf(&seqAlign[*alignCount], " ");
				sprintf(&seq[(*alignCount)++], "-");
			}
			if(updateSeqAlign == 1)
			{
				if(pdbMap1.aChainMap[j].resMap[k].seq2MappedPos != NO_ALIGN_INDEX &&
						pdbMap1.aChainMap[j].resMap[k].residue != pdbMap2.aChainMap[j].resMap[pdbMap1.aChainMap[j].resMap[k].seq2MappedPos].residue)
					sprintf(&seqAlign[*alignCount], "x");
				else
					sprintf(&seqAlign[*alignCount], " ");
			}
			sprintf(&seq[(*alignCount)++], "%c", pdbMap1.aChainMap[j].resMap[k].residue);
			if(pdbMap1.aChainMap[j].resMap[k].seq2MappedPos != NO_ALIGN_INDEX)
				prevIndex = pdbMap1.aChainMap[j].resMap[k].seq2MappedPos;
		}

		for(l = prevIndex + 1; l < pdbMap2.aChainMap[j].resCount; l++)
		{
			if(updateSeqAlign == 1)
				sprintf(&seqAlign[*alignCount], " ");
			sprintf(&seq[(*alignCount)++], "-");
		}
	}
}

int printPrefix(FILE *fpt, int alignCount, int residuePerLine, char seqId[])
{
	if(alignCount % residuePerLine == 0)
	{
		if(alignCount > 0)
			fprintf(fpt, "\n");
		fprintf(fpt, "%s\t", seqId);
		if(alignCount > 0)
			return 1;
	}
	return 0;
}

void printBpSrc(FILE * fpt, annotationMethod aMethod, int bpSrc)
{
	if(bpSrc == 2)
		fprintf(fpt, "  Both\n");
	else if(bpSrc == 1)
		fprintf(fpt, "  CompAnnotate\n");
	else
	{
		if(aMethod == mcAnnotate)
			fprintf(fpt, "  MC-Annotate\n");
		else if(aMethod == fr3d)
			fprintf(fpt, "  FR3D\n");
		else if(aMethod == dssr)
			fprintf(fpt, "  DSSR\n");
		else if(aMethod == rnaView)
			fprintf(fpt, "  RNAView\n");
		else if(aMethod == clarna)
			fprintf(fpt, "  ClaRNA\n");
		else
			fprintf(fpt, "  Undefined Method\n");
	}
}

int printAcceptedBPInteractions(char outputFile[], char bpSrcOutput[], char PBDId[], annotationMethod aMethod, int sort, char stackInfoFileName[], basePairInfo basePairMapping[MAX_BASE_PAIR], int basePairCountsMapping, int printMotifs, char pathCW[], int originalFormat)
{
	int i;
	char line[LINE_SIZE];

	FILE *fpt = fopen(outputFile, "w");
	if(!fpt)
	{
		printf("%s file could not be created\n", outputFile);
		return 0;
	}
	
	FILE *fptBpSrc = fopen(bpSrcOutput, "w");
	if(!fptBpSrc)
	{
		printf("%s file could not be created\n", bpSrcOutput);
		return 0;
	}

	FILE *stackInfoFile = fopen(stackInfoFileName, "r");
	if(!stackInfoFile)
	{
		printf("%s file not found\n", stackInfoFileName);
		return 0;
	}
	if(sort)
		qsort(basePairAccepted, bpAccCount, sizeof(basePairInfo), compBasePairId);
	if(SIMPLE_OUTPUT_FORMAT)
	{
		fprintf(fpt, "# FORMAT: <chain>:<residue>-<chain>:<residue>\n");
		fprintf(fptBpSrc, "# FORMAT: <chain>:<residue>-<chain>:<residue>\n");
		if(PRINT_ORIGINAL_SIMPLE)
		{
			for(i = 0; i < basePairCountsMapping; i++)
			{
				printPairSimpleFormat(fpt, basePairMapping[i], 0);
				printPairSimpleFormat(fptBpSrc, basePairMapping[i], 1);
				printBpSrc(fptBpSrc, aMethod, basePairMapping[i].bpSrc);
			}
		}
		else
		{
			for(i = 0; i < bpAccCount; i++)
			{
				printPairSimpleFormat(fpt, basePairAccepted[i], 0);
				printPairSimpleFormat(fptBpSrc, basePairAccepted[i], 1);
				printBpSrc(fptBpSrc, aMethod, basePairAccepted[i].bpSrc);
			}
		}
	}
	else
	{
		char methodName[30];
		if(aMethod == mcAnnotate)
		{
			strcpy(methodName, "MC-Annotate CompAnnotate");
			if(originalFormat)
			{
				while(fscanf(stackInfoFile, " %[^\n]", line) == 1)
				{
					fprintf(fpt, "%s\n", line);
					fprintf(fptBpSrc, "%s", line);
					printBpSrc(fptBpSrc, aMethod, 0);
				}
				fprintf(fpt, "Base-pairs ------------------------------------------------------\n");
				fprintf(fptBpSrc, "Base-pairs ------------------------------------------------------\n");
				for(i = 0; i < bpAccCount; i++)
				{
					printPairMCAnnotateFormat(fpt, basePairAccepted[i], 0);
					printPairMCAnnotateFormat(fptBpSrc, basePairAccepted[i], 1);
					printBpSrc(fptBpSrc, aMethod, basePairAccepted[i].bpSrc);
				}
			}
			else
				for(i = 0; i < bpAccCount; i++)
				{
					printPairRCAFormat(fpt, basePairAccepted[i], 0);
					printPairRCAFormat(fptBpSrc, basePairAccepted[i], 1);
					printBpSrc(fptBpSrc, aMethod, basePairAccepted[i].bpSrc);
				}
		}
		else
		{
			if(aMethod == fr3d)
				strcpy(methodName, "FR3D CompAnnotate");
			else if(aMethod == dssr)
				strcpy(methodName, "DSSR CompAnnotate");
			else if(aMethod == rnaView)
				strcpy(methodName, "RNAView CompAnnotate");
			else if(aMethod == clarna)
				strcpy(methodName, "ClaRNA CompAnnotate");
			else
				strcpy(methodName, "Undefined Method");

			if(aMethod == rnaView && originalFormat == 1)
			{
				fprintf(fpt, "PDB data file name: %s.pdb\n", PBDId);
				fprintf(fpt, "BEGIN_base-pair\n");
				
				fprintf(fptBpSrc, "PDB data file name: %s.pdb\n", PBDId);
				fprintf(fptBpSrc, "BEGIN_base-pair\n");
				
				while(fscanf(stackInfoFile, " %[^\n]", line) == 1)
				{
					fprintf(fpt, "%s\n", line);
					fprintf(fptBpSrc, "%s", line);
					printBpSrc(fptBpSrc, aMethod, 0);
				}
				for(i = 0; i < bpAccCount; i++)
				{
					printPairRNAViewFormat(fpt, basePairAccepted[i], 0);
					printPairRNAViewFormat(fptBpSrc, basePairAccepted[i], 1);
					printBpSrc(fptBpSrc, aMethod, basePairAccepted[i].bpSrc);
				}
				fprintf(fpt, "END_base-pair\n");
				fprintf(fptBpSrc, "END_base-pair\n");
			}
			else
				for(i = 0; i < bpAccCount; i++)
				{
					printPairRCAFormat(fpt, basePairAccepted[i], 0);
					printPairRCAFormat(fptBpSrc, basePairAccepted[i], 1);
					printBpSrc(fptBpSrc, aMethod, basePairAccepted[i].bpSrc);
				}
		}
		if(printMotifs)
			extractMotifInfo(motifs, motifCount, PBDId, methodName, basePairAccepted, bpAccCount, pathCW);
	}
	if(stackInfoFile)
		fclose(stackInfoFile);
	if(fptBpSrc)
		fclose(fptBpSrc);
	if(fpt)
		fclose(fpt);
	return 1;
}

int updatePDBAlign_Star3D(sequenceMap pdbMap[], char path[], char pdbId1[], char chainId1[], char pdbId2[], char chainId2[])
{
	char align3D[MAX_FILE_SZ];
	strcpy(align3D, path);
	strcat(align3D, pdbId1);
	strcat(align3D, "_");
	strcat(align3D, chainId1);
	strcat(align3D, "_");
	strcat(align3D, pdbId2);
	strcat(align3D, "_");
	strcat(align3D, chainId2);
	strcat(align3D, ".aln");
	FILE *fpt = fopen(align3D, "r");
	if(!fpt)
	{
		//printf("File \"%s\" was not found\n", align3D);
		return 0;
	}
	else
	{
		int i = 0, j = 0, k, count = 0, len, mapMismatchCount = 0;
		int resId1, resId2;
		char chain1[MAX_CHAIN_NAME_LENGTH + 1], iCode1, chain2[MAX_CHAIN_NAME_LENGTH + 1], iCode2;
		char line[LINE_SIZE];

		while(fscanf(fpt,"%s", line) == 1)
		{
			if(strstr(line, "<->") == NULL)
				continue;
			sscanf(line," %[^:\n\r ]:%d", chain1, &resId1);
			k = 0;
			while(line[k] != '<')
				k++;
			if(k == 0 || isdigit(line[k - 1]))
				iCode1 = NO_ICODE;
			else
				iCode1 = line[k - 1];
			while(line[k] != '>')
				k++;
			sscanf(&line[k + 1]," %[^:\n\r ]:%d", chain2, &resId2);
			len = strlen(line);
			if(isdigit(line[len - 1]))
				iCode2 = NO_ICODE;
			else
				iCode2 = line[len - 1];

			if(strcmp(pdbMap[0].aChainMap[0].chainId, chain1) != 0 || strcmp(pdbMap[1].aChainMap[0].chainId, chain2) != 0)
			{
				printf("Chain mismatch %s-%s, %s-%s\n", pdbMap[0].aChainMap[0].chainId, chain1, pdbMap[1].aChainMap[0].chainId, chain2);
				getchar();
				return 0;
			}

			while((pdbMap[0].aChainMap[0].resMap[i].resPos < resId1 ||
				(pdbMap[0].aChainMap[0].resMap[i].resPos == resId1 && pdbMap[0].aChainMap[0].resMap[i].iCode < iCode1)) &&
				i < pdbMap[0].aChainMap[0].resCount)
			{
				pdbMap[0].aChainMap[0].resMap[i].seq2MappedPos = NO_ALIGN_INDEX;
				i++;
			}

			while((pdbMap[1].aChainMap[0].resMap[j].resPos < resId2 ||
				(pdbMap[1].aChainMap[0].resMap[j].resPos == resId2 && pdbMap[1].aChainMap[0].resMap[j].iCode < iCode2)) &&
				j < pdbMap[1].aChainMap[0].resCount)
			{
				pdbMap[1].aChainMap[0].resMap[j].seq2MappedPos = NO_ALIGN_INDEX;
				j++;
			}

			if(pdbMap[0].aChainMap[0].resMap[i].seq2MappedPos != j)
				mapMismatchCount++;

			if(pdbMap[0].aChainMap[0].resMap[i].resPos == resId1 && pdbMap[0].aChainMap[0].resMap[i].iCode == iCode1 && i < pdbMap[0].aChainMap[0].resCount &&
					pdbMap[1].aChainMap[0].resMap[j].resPos == resId2 && pdbMap[1].aChainMap[0].resMap[j].iCode == iCode2 && j < pdbMap[1].aChainMap[0].resCount)
			{
				pdbMap[0].aChainMap[0].resMap[i].seq2MappedPos = j;
				pdbMap[1].aChainMap[0].resMap[j].seq2MappedPos = i;
				i++;
				j++;
			}
		}
		while(i < pdbMap[0].aChainMap[0].resCount)
			pdbMap[0].aChainMap[0].resMap[i++].seq2MappedPos = NO_ALIGN_INDEX;
		while(j < pdbMap[1].aChainMap[0].resCount)
			pdbMap[1].aChainMap[0].resMap[j++].seq2MappedPos = NO_ALIGN_INDEX;

		while(i < pdbMap[0].aChainMap[0].resCount)
		{
			pdbMap[0].aChainMap[0].resMap[i].seq2MappedPos = NO_ALIGN_INDEX;
			i++;
		}

		while(j < pdbMap[1].aChainMap[0].resCount)
		{
			pdbMap[1].aChainMap[0].resMap[j].seq2MappedPos = NO_ALIGN_INDEX;
			j++;
		}

		for(i = 0; i < pdbMap[0].aChainMap[0].resCount; i++)
		{
			if(pdbMap[0].aChainMap[0].resMap[i].seq2MappedPos != NO_ALIGN_INDEX)
				count++;
		}
		if(PRINT_ALIGN_BP_INFO)
		{
			printf("Total match = %d\n", count);
			printf("Total map misMatch = %d\n", mapMismatchCount);
		}
	}
	return 1;
}

void printCorrectnessStatistics(FILE* fpt, int ind1, int ind2, sequenceMap pdbMap[])
{
	//Print statistics of accuracy of predictions
	double totCount = (basePairCommon[ind1].bpCounts + basePairSetDiff[ind1].bpCounts + basePairNoAlign[ind1].bpCounts);
	//double mismatchPercent = (basePairSetDiff[refPDBIndex].bpCounts + basePairNoAlign[refPDBIndex].bpCounts) * 100.0 / totCount;
	double matchPercent = basePairCommon[ind1].bpCounts * 100.0 / totCount;
	double percentCBP = (100.0 * commonCBP) / (commonCBP + diffCBP), percentNCBP = (100.0 * commonNCBP) / (commonNCBP + diffNCBP),
	percentOthers = (100.0 * commonOthers) / (commonOthers + diffOthers);

	if(PRINT_ALIGN_BP_INFO)
	{
		printf("%s vs %s : BP Match = %d BP Mismatch = %d noAlign = %d match %% = %0.2lf\n",
				pdbMap[ind1].seqId, pdbMap[ind2].seqId, basePairCommon[ind1].bpCounts, basePairSetDiff[ind1].bpCounts, basePairNoAlign[ind1].bpCounts, matchPercent);
		printf("%s vs %s : BP Match = %d BP Mismatch = %d match %% = %0.2lf\n", pdbMap[ind1].seqId, pdbMap[ind2].seqId, commonStatCount, diffStatCount, matchPercent);
		printf("Match: Canonical = %d (%0.2lf%%), WHS = %d (%0.2lf%%), Others = %d (%0.2lf%%)\n", commonCBP, percentCBP, commonNCBP, percentNCBP, commonOthers, percentOthers);
		printf("Mismatch: Canonical = %d, WHS = %d, Others = %d\n", diffCBP, diffNCBP, diffOthers);
	}
	if(testCodeLog)
	{
		fprintf(testCodeLog, "%s (%s)\n", pdbMap[ind1].seqId, pdbMap[ind2].seqId);
		fprintf(testCodeLog, "Canonical:%d/%d %0.2lf\n", commonCBP, diffCBP, percentCBP);
		fprintf(testCodeLog, "Non-Canonical:%d/%d %0.2lf\n", commonNCBP, diffNCBP, percentNCBP);
	}
}


void printInferabilityInFile(FILE* fpt, int ind1, int ind2, sequenceMap pdbMap[])
{
	fprintf(testCodeLog2, "%s vs %s : BP Canonical Common = %d BP Canonical Inferable = %d Canonical Non-inferable = %d\n",
		pdbMap[ind1].seqId, pdbMap[ind2].seqId, basePairCommon[ind1].cannonicalCount, basePairSetDiff[ind1].cannonicalCount, basePairNoAlign[ind1].cannonicalCount);
	fprintf(testCodeLog2, "%s vs %s : BP Non-Canonical Common = %d BP Non-Canonical Inferable = %d Non-Canonical Non-inferable = %d\n",
		pdbMap[ind1].seqId, pdbMap[ind2].seqId, basePairCommon[ind1].nonCannonicalCount, basePairSetDiff[ind1].nonCannonicalCount, basePairNoAlign[ind1].nonCannonicalCount);
	fprintf(testCodeLog2, "%s vs %s : BP Other Common = %d BP Other Inferable = %d Other Non-inferable = %d\n",
		pdbMap[ind1].seqId, pdbMap[ind2].seqId, basePairCommon[ind1].otherCount, basePairSetDiff[ind1].otherCount, basePairNoAlign[ind1].otherCount);

	//fprintf(testCodeLog2, "%s vs %s : BP Common = %d BP Inferable = %d Non-inferable = %d\n",
		//pdbMap[ind2].seqId, pdbMap[ind1].seqId, basePairCommon[ind1].bpCounts, basePairSetDiff[ind2].bpCounts, basePairNoAlign[ind2].bpCounts);

	fprintf(testCodeLog2, "%s vs %s : BP Canonical Common = %d BP Canonical Inferable = %d Canonical Non-inferable = %d\n",
		pdbMap[ind2].seqId, pdbMap[ind1].seqId, basePairCommon[ind1].cannonicalCount, basePairSetDiff[ind2].cannonicalCount, basePairNoAlign[ind2].cannonicalCount);
	fprintf(testCodeLog2, "%s vs %s : BP Non-Canonical Common = %d BP Non-Canonical Inferable = %d Non-Canonical Non-inferable = %d\n",
		pdbMap[ind2].seqId, pdbMap[ind1].seqId, basePairCommon[ind1].nonCannonicalCount, basePairSetDiff[ind2].nonCannonicalCount, basePairNoAlign[ind2].nonCannonicalCount);
	fprintf(testCodeLog2, "%s vs %s : BP Other Common = %d BP Other Inferable = %d Other Non-inferable = %d\n",
		pdbMap[ind2].seqId, pdbMap[ind1].seqId, basePairCommon[ind1].otherCount, basePairSetDiff[ind2].otherCount, basePairNoAlign[ind2].otherCount);
}

void printStats(int statCount, statisticsBP aStat[], FILE* fpt)
{
	int i;
	for(i = 0; i < statCount; i++)
	{
		fprintf(fpt, "(%8.4lf) %8.4lf <- %8.4lf", aStat[i].distDiff, aStat[i].aDist.proposedDist, aStat[i].aDist.actualDist);

		fprintf(fpt, "  :::: \'%s\'", aStat[i].chainIdPropose[0]);
		printResSeqFile(aStat[i].resSeqNoPropose[0], aStat[i].iCodePropose[0], 6, fpt);
		fprintf(fpt, " %c - %c \'%s\'", aStat[i].residueNamePropose[0], aStat[i].residueNamePropose[1], aStat[i].chainIdPropose[1]);
		printResSeqFile(aStat[i].resSeqNoPropose[1], aStat[i].iCodePropose[1], 6, fpt);

		fprintf(fpt, " <- \'%s\'", aStat[i].chainIdActual[0]);
		printResSeqFile(aStat[i].resSeqNoActual[0], aStat[i].iCodeActual[0], 6, fpt);
		fprintf(fpt, " %c - %c \'%s\'", aStat[i].residueNameActual[0],	aStat[i].residueNameActual[1], aStat[i].chainIdActual[1]);
		printResSeqFile(aStat[i].resSeqNoActual[1], aStat[i].iCodeActual[1], 6, fpt);

		fprintf(fpt, " :::: %c%s/%s", aStat[i].glycosidicOrientation, aStat[i].aEdgeActual.edge1, aStat[i].aEdgeActual.edge2);

		fprintf(fpt, " :::: %9.3lf %9.3lf %9.3lf  ~ %9.3lf %9.3lf %9.3lf", aStat[i].coOrdDiff[0].x, aStat[i].coOrdDiff[0].y, aStat[i].coOrdDiff[0].z,
			aStat[i].coOrdDiff[1].x, aStat[i].coOrdDiff[1].y, aStat[i].coOrdDiff[1].z);

		fprintf(fpt,"\n");
	}
}
