/*
 * ReadClaRNAData.c
 *
 *  Created on: Mar 14, 2014
 *      Author: Shahidul
 */

#include "./inc/ReadRNAViewData.h"
#include "./inc/UtilityFunctions.h"
#include "./inc/ReadClaRNAData.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define PDB_COUNT 8
#define ANALYZE_INTERACTIONS 0
#define INCLUDE_NEAR_ANNOTATION 1
#define PRINT_TOKENS 0
#define REMOVE_DUPLICATE 1

void parseAndProcessClaRNATokens(char line[], char targetChain[], basePairInfo basePairClaRNA[MAX_BASE_PAIR], int *basePairCountClaRNA, FILE *stackOut);
void analyzeClaRNATokens(char token[][LINE_SIZE], int count);
void findAndAddItemClaRNA(char arr[][10], int *count, char str[]);
void printClaRNAInteractions(char arr[][10], int count, char heading[]);
//int isMultipleOrNoInteractionTag(char token[][LINE_SIZE], int count);


int readClaRNAOutput(char inputFile[], basePairInfo basePairClaRNA[MAX_BASE_PAIR], int *basePairCountClaRNA, char PDBId[], char targetChain[], char stackOutput[])
{
	*basePairCountClaRNA = 0;
	FILE *fpt = fopen(inputFile, "r");
	if(!fpt)
	{
		printf("%s File Not Found\n", inputFile);
		return 0;
	}

	FILE *stackOut = fopen(stackOutput, "w");
	if(!stackOut)
	{
		printf("%s File Not Found\n", stackOutput);
		return 0;
	}

	//basePairInfo temp;
	int i; //distinctPairTypeCount = 0, len, firstLinePrinted = 0;
	char line[LINE_SIZE];
	//int printed = 0;

	char *start = "\"residue1\",\"residue2\",\"n_type\",\"weight\",\"description\",\"full description\"";
	int startedBP = 0;
	while(fscanf(fpt, " %[^\n\r]", line) == 1)
	{
		if(startedBP)
		{
			if(line[0] == 0)
				break;
			parseAndProcessClaRNATokens(line, targetChain, basePairClaRNA, basePairCountClaRNA, stackOut);
		}
		else if(strcmp(line, start) == 0)
			startedBP = 1;
	}

	//printf("BP Count = %d\n", *basePairCountClaRNA);
	//getchar();

	qsort(basePairClaRNA, *basePairCountClaRNA, sizeof(basePairInfo), compBasePairId);
	if(TEST_PRINT_COUNT > 0)
		printf("ClaRNA Pair Sample\n");
	for(i = 0; i < TEST_PRINT_COUNT; i++)
	{
		printPairInfoOnScreen(basePairClaRNA[i]);
		//char ch = getchar();
		//if(ch == 'X')
			//break;
	}

	if(ANALYZE_INTERACTIONS)
	{
		//qsort(basePair, bpCount, 10*sizeof(char), compStr);
		//printClaRNAInteractions(basePair, bpCount, "Base Pairs");
		//getchar();
	}

	if(stackOut)
		fclose(stackOut);
    if(fpt)
        fclose(fpt);
    return 1;
}

//Return if acceptable interaction, chain-wise
int insertClaRNAInteractionInfo(basePairInfo *info, char tokenMain[][LINE_SIZE], char targetChainID[])
{
	int len, flag[2] = {0, 0};

	//strcpy(info->adjacent, "");
	strcpy(info->stackDirection, "");
	strcpy(info->MC_SymNo, "");
	info->isParallel = -1;
	info->edgeCount = 0;
	info->glycosidicOrientation = 0;
	strcpy(info->chainResidue[0], targetChainID);
	strcpy(info->chainResidue[1], targetChainID);

	info->iCode[0] = info->iCode[1] = NO_ICODE;

	//myTokenize(tokenMain[1], s, tokens, &tokenCount, 1);
	if(strncmp(&tokenMain[0][1], targetChainID, strlen(targetChainID)) == 0)
		flag[0] = 1;
	len = strlen(tokenMain[0]);
	tokenMain[0][len-1] = 0;

	//No iCode Management

	sscanf(&tokenMain[0][strlen(targetChainID) + 1], "%d", &info->residueID[0]);
	info->residue[0] = tokenMain[2][1];

	if(strncmp(&tokenMain[1][1], targetChainID, strlen(targetChainID)) == 0)
		flag[1] = 1;

	len = strlen(tokenMain[1]);
	tokenMain[1][len-1] = 0;
	//No iCode Management

	sscanf(&tokenMain[1][strlen(targetChainID) + 1], "%d", &info->residueID[1]);
	info->residue[1] = tokenMain[2][2];

	if(info->residueID[0] > info->residueID[1])
		return 0;

	//Pick Only Confirm (Weight 1.0) Interactions
	double weight;
	sscanf(tokenMain[3], "%lf", &weight);

	//if(myFabs(1.0 - weight) > 0.5)
		//return 0;
	info->confidenceWeight = weight;

	//Remove Stack Info's
	if(tokenMain[4][1] == '<' || tokenMain[4][1] == '>')
		return 0;


	if(flag[0] && flag[1])
	{
		//printPairInfoOnScreen(*info);
		//getchar();
		return 1;
	}
	else
		return 0;
}

//Return if BP is acceptable, edge info -wise
int insertClaRNAEdgeInfo(basePairInfo *info, char tokenEdge[])
{
	char *edge = &tokenEdge[0];

	//printf("Step1 - %s\n", edge);
	//getchar();

	if(!(edge[1] == 'W' || edge[1] == 'H' || edge[1] == 'S') ||
		!(edge[2] == 'W' || edge[2] == 'H' || edge[2] == 'S'))
		return 0;

	info->aEdges[0].edge1[0] = toupper(edge[1]);
	info->aEdges[0].edge2[0] = toupper(edge[2]);
	info->aEdges[0].edge1[1] = info->aEdges[0].edge2[1] = '\0';
	info->glycosidicOrientation = edge[4];

	//printf("%0.2lf -- ", info->confidenceWeight);
	//printPairInfoOnScreen(*info);

	//getchar();

	info->edgeCount = 1;
	return 1;
}

void parseAndProcessClaRNATokens(char line[], char targetChain[], basePairInfo basePairClaRNA[MAX_BASE_PAIR], int *basePairCountClaRNA, FILE *stackOut)
{
	basePairInfo temp;
	char s[3] = {" ,"};
	char tokenMain[6][LINE_SIZE];
	int mtCount, j;//, k;
	int acceptable;
	myTokenize(line, s, tokenMain, &mtCount, 1);
	if(ANALYZE_INTERACTIONS)
		analyzeClaRNATokens(tokenMain, mtCount);

	acceptable = insertClaRNAInteractionInfo(&temp, tokenMain, targetChain);

	//If chain is acceptable
	if(acceptable)
	{
		acceptable = insertClaRNAEdgeInfo(&temp, tokenMain[4]);
		if(acceptable)
			basePairClaRNA[(*basePairCountClaRNA)++] = temp;//printPairRNAViewFormat(fptOut, temp);
	}

	if(PRINT_TOKENS)
	{
		for(j = 0; j < mtCount; j++)
			printf("%s\n", tokenMain[j]);
		getchar();
	}
}

void analyzeClaRNATokens(char token[][LINE_SIZE], int count)
{
	//isMultipleOrNoInteractionTag(token, count);

	//findAndAddItemClaRNA(basePair, &bpCount, token[4]);
}

void findAndAddItemClaRNA(char arr[][10], int *count, char str[])
{
	int i;
	if(strlen(str) == 0)
		return;
	for(i = 0; i < *count; i++)
	{
		if(strcmp(arr[i], str) == 0)
			return;
	}
	strcpy(arr[*count], str);
	(*count)++;
	return;
}

void printClaRNAInteractions(char arr[][10], int count, char heading[])
{
	int i;
	printf("%s\n", heading);
	for(i = 0; i < count; i++)
		printf("%s\n", arr[i]);
}

//Unexpected behavior testing
/*int isMultipleOrNoInteractionTag(char token[][LINE_SIZE], int count)
{
	int i = 0, sum = 0;
	for(i = 2; i < count - 1; i++)
		sum += strlen(token[i]) > 0;
	if(sum > 1)
	{
		printf("Unwanted number of interaction\n");
		for(i = 0; i < count; i++)
			printf("%s ", token[i]);
		printf("\n");
		getchar();
		return 1;
	}
	return 0;
}*/

