/*
 * ReadDSSRData.c
 *
 *  Created on: Mar 14, 2014
 *      Author: Shahidul
 */

#include "./inc/ReadRNAViewData.h"
#include "./inc/UtilityFunctions.h"
#include "./inc/ReadDSSRData.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define PDB_COUNT 8
#define ANALYZE_INTERACTIONS 0
#define INCLUDE_NEAR_ANNOTATION 1
#define PRINT_TOKENS 0
#define REMOVE_DUPLICATE 1

void parseAndProcessDSSRTokens(char line[], char targetChain[], basePairInfo basePairDSSR[MAX_BASE_PAIR], int *basePairCountDSSR, FILE *stackOut);
void analyzeDSSRTokens(char token[][LINE_SIZE], int count);
void findAndAddItemDSSR(char arr[][10], int *count, char str[]);
void printDSSRInteractions(char arr[][10], int count, char heading[]);
//int isMultipleOrNoInteractionTag(char token[][LINE_SIZE], int count);


int readDSSROutput(char inputFile[], basePairInfo basePairDSSR[MAX_BASE_PAIR], int *basePairCountDSSR, char PDBId[], char targetChain[], char stackOutput[])
{
	*basePairCountDSSR = 0;
	FILE *fpt = fopen(inputFile, "r");
	if(!fpt)
	{
		printf("%s File Not Found\n", inputFile);
		return 0;
	}

	FILE *stackOut = fopen(stackOutput, "w");
	if(!stackOut)
	{
		printf("%s File Not Found\n", stackOutput);
		return 0;
	}

	//basePairInfo temp;
	int i; //distinctPairTypeCount = 0, len, firstLinePrinted = 0;
	char line[LINE_SIZE];
	//int printed = 0;

	char *start = "nt1            nt2           bp  name        Saenger    LW  DSSR";
	int startedBP = 0;
	while(fscanf(fpt, " %[^\n\r]", line) == 1)
	{
		if(startedBP)
		{
			if(line[0] == '*')
				break;
			parseAndProcessDSSRTokens(line, targetChain, basePairDSSR, basePairCountDSSR, stackOut);
		}
		else if(strcmp(line, start) == 0)
			startedBP = 1;
	}

	qsort(basePairDSSR, *basePairCountDSSR, sizeof(basePairInfo), compBasePairId);
	if(TEST_PRINT_COUNT > 0)
		printf("DSSR Pair Sample\n");
	for(i = 0; i < TEST_PRINT_COUNT; i++)
	{
		printPairInfoOnScreen(basePairDSSR[i]);
		//char ch = getchar();
		//if(ch == 'X')
			//break;
	}

	if(ANALYZE_INTERACTIONS)
	{
		//qsort(basePair, bpCount, 10*sizeof(char), compStr);
		//printDSSRInteractions(basePair, bpCount, "Base Pairs");
		//getchar();
	}

	if(stackOut)
		fclose(stackOut);
    if(fpt)
        fclose(fpt);
    return 1;
}

//Return if acceptable interaction, chain-wise
int insertDSSRInteractionInfo(basePairInfo *info, char tokenMain[][LINE_SIZE], char targetChainID[])
{
	char tokens[2][LINE_SIZE], s[] = ".";
	int tokenCount, len, flag[2] = {0, 0};

	//strcpy(info->adjacent, "");
	strcpy(info->stackDirection, "");
	strcpy(info->MC_SymNo, "");
	info->isParallel = -1;
	info->edgeCount = 0;
	info->glycosidicOrientation = 0;
	strcpy(info->chainResidue[0], targetChainID);
	strcpy(info->chainResidue[1], targetChainID);

	info->iCode[0] = info->iCode[1] = NO_ICODE;

	myTokenize(tokenMain[1], s, tokens, &tokenCount, 1);
	if(strcmp(tokens[0], targetChainID) == 0)
		flag[0] = 1;
	len = strlen(tokens[1]);
	if(tokens[1][len - 2] == '^')
	{
		info->iCode[0] = tokens[1][len - 1];
		tokens[1][len - 2] = 0;
	}
	sscanf(&tokens[1][1], "%d", &info->residueID[0]);
	info->residue[0] = tokens[1][0];

	myTokenize(tokenMain[2], s, tokens, &tokenCount, 1);
	if(strcmp(tokens[0], targetChainID) == 0)
		flag[1] = 1;
	len = strlen(tokens[1]);
	if(tokens[1][len - 2] == '^')
	{
		info->iCode[0] = tokens[1][len - 1];
		tokens[1][len - 2] = 0;
	}
	sscanf(&tokens[1][1], "%d", &info->residueID[1]);
	info->residue[1] = tokens[1][0];



	if(flag[0] && flag[1])
		return 1;
	else
		return 0;
}

//Return if BP is acceptable, edge info -wise
int insertDSSREdgeInfo(basePairInfo *info, char tokenEdge[])
{
	char *edge = &tokenEdge[0];

	info->glycosidicOrientation = edge[0];

	if(!(edge[1] == 'W' || edge[1] == 'H' || edge[1] == 'S') ||
		!(edge[2] == 'W' || edge[2] == 'H' || edge[2] == 'S'))
		return 0;
	info->aEdges[0].edge1[0] = toupper(edge[1]);
	info->aEdges[0].edge2[0] = toupper(edge[2]);

	info->aEdges[0].edge1[1] = info->aEdges[0].edge2[1] = '\0';
	info->edgeCount = 1;
	return 1;
}

void parseAndProcessDSSRTokens(char line[], char targetChain[], basePairInfo basePairDSSR[MAX_BASE_PAIR], int *basePairCountDSSR, FILE *stackOut)
{
	basePairInfo temp;
	char s[3] = {" \t"};
	char tokenMain[9][LINE_SIZE];
	int mtCount, j;//, k;
	int acceptable;
	myTokenize(line, s, tokenMain, &mtCount, 1);
	if(ANALYZE_INTERACTIONS)
		analyzeDSSRTokens(tokenMain, mtCount);

	acceptable = insertDSSRInteractionInfo(&temp, tokenMain, targetChain);

	//If chain is acceptable
	if(acceptable)
	{
		acceptable = insertDSSREdgeInfo(&temp, tokenMain[6]);
		if(acceptable)
			basePairDSSR[(*basePairCountDSSR)++] = temp;//printPairRNAViewFormat(fptOut, temp);
	}

	if(PRINT_TOKENS)
	{
		for(j = 0; j < mtCount; j++)
			printf("%s\n", tokenMain[j]);
		getchar();
	}
}

void analyzeDSSRTokens(char token[][LINE_SIZE], int count)
{
	//isMultipleOrNoInteractionTag(token, count);

	//findAndAddItemDSSR(basePair, &bpCount, token[6]);
}

void findAndAddItemDSSR(char arr[][10], int *count, char str[])
{
	int i;
	if(strlen(str) == 0)
		return;
	for(i = 0; i < *count; i++)
	{
		if(strcmp(arr[i], str) == 0)
			return;
	}
	strcpy(arr[*count], str);
	(*count)++;
	return;
}

void printDSSRInteractions(char arr[][10], int count, char heading[])
{
	int i;
	printf("%s\n", heading);
	for(i = 0; i < count; i++)
		printf("%s\n", arr[i]);
}

//Unexpected behavior testing
/*int isMultipleOrNoInteractionTag(char token[][LINE_SIZE], int count)
{
	int i = 0, sum = 0;
	for(i = 2; i < count - 1; i++)
		sum += strlen(token[i]) > 0;
	if(sum > 1)
	{
		printf("Unwanted number of interaction\n");
		for(i = 0; i < count; i++)
			printf("%s ", token[i]);
		printf("\n");
		getchar();
		return 1;
	}
	return 0;
}*/


