/*
 * ReadFR3DData.c
 *
 *  Created on: Feb 09, 2015
 *      Author: Shahidul
 */

#include "./inc/ReadRNAViewData.h"
#include "./inc/UtilityFunctions.h"
#include "./inc/ReadFR3DData.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define PDB_COUNT 8
#define ANALYZE_INTERACTIONS 0
#define INCLUDE_NEAR_ANNOTATION 1
#define PRINT_TOKENS 0
#define REMOVE_DUPLICATE 1

typedef struct
{
	char newPdbId[6];
	char newChainId[MAX_CHAIN_NAME_LENGTH + 1];
	char oldPdbId[6];
	char oldChainId[MAX_CHAIN_NAME_LENGTH + 1];
}PDB_Chain_Mapping;


void parseAndProcessTokens(char line[], PDB_Chain_Mapping aMap, basePairInfo basePairFR3D[MAX_BASE_PAIR], int *basePairCountFR3D, FILE *stackOut);
void parseProcessAndWrite(char line[], PDB_Chain_Mapping aMap, FILE *fptOut);
int validatePDBIdChainBP(basePairInfo *info, char tokens[][8][LINE_SIZE], int tokenCount[], PDB_Chain_Mapping aMap);
void analyzeTokens(char token[][LINE_SIZE], int count);
void findAndAddItem(char arr[][10], int *count, char str[]);
void printAllInteractionTypes(void);
void printInteractions(char arr[][10], int count, char heading[]);
int isMultipleOrNoInteractionTag(char token[][LINE_SIZE], int count);
void createSelfMapping(PDB_Chain_Mapping *aMap, char PDBId[], char chain[]);

int readFR3DOutput(char inputFile[], basePairInfo basePairFR3D[MAX_BASE_PAIR], int *basePairCountFR3D, char PDBId[], char targetChain[], char stackOutput[])
{
	*basePairCountFR3D = 0;
	FILE *fpt = fopen(inputFile, "r");
	if(!fpt)
	{
		printf("%s File Not Found\n", inputFile);
		return 0;
	}

	FILE *stackOut = fopen(stackOutput, "w");
	if(!stackOut)
	{
		printf("%s File Not Found\n", stackOutput);
		return 0;
	}

	PDB_Chain_Mapping aMap;
	createSelfMapping(&aMap, PDBId, targetChain);

	//basePairInfo temp;
	int i;//distinctPairTypeCount = 0, i, len, firstLinePrinted = 0;
	char line[LINE_SIZE];
	//int printed = 0;

    fscanf(fpt, " %[^\n]", line);
	while(fscanf(fpt, " %[^\n]", line) == 1)
		parseAndProcessTokens(line, aMap, basePairFR3D, basePairCountFR3D, stackOut);

	qsort(basePairFR3D, *basePairCountFR3D, sizeof(basePairInfo), compBasePairId);
	if(TEST_PRINT_COUNT > 0)
		printf("FR3D Pair Sample\n");
	for(i = 0; i < TEST_PRINT_COUNT; i++)
	{
		printPairInfoOnScreen(basePairFR3D[i]);
		//char ch = getchar();
		//if(ch == 'X')
			//break;
	}
	if(stackOut)
		fclose(stackOut);
    if(fpt)
        fclose(fpt);
    return 1;
}

void createSelfMapping(PDB_Chain_Mapping *aMap, char PDBId[], char chain[])
{
	strcpy(aMap->newPdbId, PDBId);
	strcpy(aMap->oldPdbId, PDBId);
	strcpy(aMap->oldChainId, chain);
	strcpy(aMap->newChainId, chain);
}

//Return if BP is acceptable, edge info -wise
int insertEdgeInfo(basePairInfo *info, char tokenEdge[])
{
	char *edge = &tokenEdge[0];
	if(tokenEdge[0] == 'n')
	{
		if(!INCLUDE_NEAR_ANNOTATION)
			return 0;
		else
			edge = &tokenEdge[1];
	}

	info->glycosidicOrientation = edge[0];

    info->aEdges[0].edge1[0] = toupper(edge[1]);
    info->aEdges[0].edge2[0] = toupper(edge[2]);
	info->aEdges[0].edge1[1] = info->aEdges[0].edge2[1] = '\0';
	info->edgeCount = 1;
	return 1;
}

void parseAndProcessTokens(char line[], PDB_Chain_Mapping aMap, basePairInfo basePairFR3D[MAX_BASE_PAIR], int *basePairCountFR3D, FILE *stackOut)
{
	basePairInfo temp;
	char s[3] = {","};
	char s2[3] = {"|"};
	char tokenMain[8][LINE_SIZE];
	char tokens[2][8][LINE_SIZE];
	int mtCount, tCount[2], j, k;
	int acceptable;
	myTokenize(line, s, tokenMain, &mtCount, 0);

	if(ANALYZE_INTERACTIONS)
		analyzeTokens(tokenMain, mtCount);

	for(j = 0; j < 2; j++)
		myTokenize(tokenMain[j], s2, tokens[j], &tCount[j], 0);

	acceptable = validatePDBIdChainBP(&temp, tokens, tCount, aMap);

	//If chain is acceptable
	if(acceptable)
	{
		if(strlen(tokenMain[2]) > 0)//BP Info
		{
			acceptable = insertEdgeInfo(&temp, tokenMain[2]);
			if(acceptable)
				basePairFR3D[(*basePairCountFR3D)++] = temp;
		}
		else if(strlen(tokenMain[3]) > 0) //Stack Info
			printPairRNAViewStack(stackOut, temp);
		else if(strlen(tokenMain[4]) > 0) // Decide later what to do about Phosphate Bond
			;
	}

	if(PRINT_TOKENS)
	{
		for(j = 0; j < mtCount; j++)
		{
			myTokenize(tokenMain[j], s2, tokens[0], &tCount[0], 0);
			for(k = 0; k < tCount[0]; k++)
				printf("Token %d.%d = %s\n", j, k, tokens[0][k]);
		}
		getchar();
	}
}

int extractRelevantFR3D_BP()
{
    //Linux Path
	char path[] = "./io_data/Input/FR3D_Annotations/";
	char path2[] = "./io_data/Input/";
	//Windows Path
	//char path[] = "io_data\\Input\\FR3D_Annotations\\";
	//char path2[] = "io_data\\Input\\";
	char inputFile[MAX_FILE_SZ] = "";
	char outputFile[MAX_FILE_SZ] = "";
	//CA, DA
	PDB_Chain_Mapping aMapping[PDB_COUNT] = {
									{"4V4S", "BA", "2B9N", "A"},
									{"4V4S", "AA", "2B9M", "A"},
									{"4V9D", "CA", "3R8S", "A"},
									{"4V9D", "AA", "4GD1", "A"},
									{"4V8I", "DA", "3V2F", "A"},
									{"2ZJR", "X", "2ZJR", "X"},
									{"2VQE", "A", "2VQE", "A"},
									{"4V4P", "BA", "1YL4", "A"}
								};
	int i;
	bpCount = bsCount = bPhCount = 0;
	for(i = 0; i < PDB_COUNT; i++)
	{
		strcpy(inputFile, path);
		strcat(inputFile, aMapping[i].newPdbId);
		strcat(inputFile, ".txt");
		FILE *fpt = fopen(inputFile, "r");
		if(!fpt)
		{
			printf("%s File Not Found\n", inputFile);
			continue;
		}

		strcpy(outputFile, path2);
		strcat(outputFile, aMapping[i].oldPdbId);
		strcat(outputFile, ".fr3d.out");
		FILE *fptOut = fopen(outputFile, "w");
		if(!fptOut)
		{
			printf("%s File Not Created\n", outputFile);
			continue;
		}

		//int len;
		char line[LINE_SIZE];

		//fprintf(fptOut, "PDB data file name: %s.pdb\n", aMapping[i].oldPdbId);
		//fprintf(fptOut, "BEGIN_base-pair\n");
		fprintf(fptOut, "Nucleotide 1,Nucleotide 2,Base-Pairs,Base-Stacking,Base-Phosphate\n");

		fscanf(fpt, " %[^\n]", line);
		while(fscanf(fpt, " %[^\n]", line) == 1)
			parseProcessAndWrite(line, aMapping[i], fptOut);

		//fprintf(fptOut, "END_base-pair\n");

		if(ANALYZE_INTERACTIONS)
			printAllInteractionTypes();


		if(fptOut)
			fclose(fptOut);

		if(fpt)
			fclose(fpt);
	}
	return 1;
}

int insertBPInfo(basePairInfo *info, char tokens[][8][LINE_SIZE], int tokenCount[], char chainID[])
{
    //strcpy(info->adjacent, "");
    strcpy(info->stackDirection, "");
    strcpy(info->MC_SymNo, "");
    info->isParallel = -1;
    info->edgeCount = 0;
    info->glycosidicOrientation = 0;
    strcpy(info->chainResidue[0], chainID);
    strcpy(info->chainResidue[1], chainID);
    sscanf(tokens[0][4], "%d", &info->residueID[0]);
    info->residue[0] = tokens[0][3][0];
    sscanf(tokens[1][4], "%d", &info->residueID[1]);
    info->residue[1] = tokens[1][3][0];
    info->iCode[0] = info->iCode[1] = NO_ICODE;
    if(tokenCount[0] > 7)
        info->iCode[0] = tokens[0][7][0];
    if(tokenCount[1] > 7)
        info->iCode[1] = tokens[1][7][0];

    if(REMOVE_DUPLICATE && (info->residueID[0] > info->residueID[1]))
        return 0;

    return 1;
}

//Return if acceptable interaction, chain-wise
int validatePDBIdChainBP(basePairInfo *info, char tokens[][8][LINE_SIZE], int tokenCount[], PDB_Chain_Mapping aMap)
{
	char newChains[3][LINE_SIZE], s[] = "/";
	int chainCount, flag[2] = {0, 0}, flag2[2] = {0, 0}, i, j;
	myTokenize(aMap.newChainId, s, newChains, &chainCount, 0);

	if(!(tokens[0][1][0] == '1' && tokens[1][1][0] == '1'))
		return 0;

	for(i = 0; i < 2; i++)
	{
		if(strcmp(tokens[i][0], aMap.newPdbId) == 0)
			flag2[i] = 1;
		for(j = 0; j < chainCount; j++)
			if(strcmp(tokens[i][2], newChains[j]) == 0)
				flag[i] = 1;
	}

	if(flag[0] && flag[1] && flag2[0] && flag2[1])
		return insertBPInfo(info, tokens, tokenCount, aMap.oldChainId);
	else
		return 0;
}

void printFR3DInteraction(FILE* fpt, basePairInfo info, char PDBId[], char BPType[], char stackType[], char phIntType[])
{
	//??Unknown what the 2nd value '1' means?? Model most likely
	fprintf(fpt, "%s|1|%s|%c|%d", PDBId, info.chainResidue[0], info.residue[0], info.residueID[0]);
	if(info.iCode[0] != NO_ICODE)
		fprintf(fpt, "|||%c", info.iCode[0]);
	fprintf(fpt, ",%s|1|%s|%c|%d", PDBId, info.chainResidue[1], info.residue[1], info.residueID[1]);
	if(info.iCode[0] != NO_ICODE)
		fprintf(fpt, "|||%c", info.iCode[1]);
	fprintf(fpt, ",%s,%s,%s\n", BPType, stackType, phIntType);
}

void parseProcessAndWrite(char line[], PDB_Chain_Mapping aMap, FILE *fptOut)
{
	basePairInfo temp;
	char s[3] = {","};
	char s2[3] = {"|"};
	char tokenMain[8][LINE_SIZE];
	char tokens[2][8][LINE_SIZE];
	int mtCount, tCount[2], j, k;
	int acceptable;
	myTokenize(line, s, tokenMain, &mtCount, 0);

	if(ANALYZE_INTERACTIONS)
		analyzeTokens(tokenMain, mtCount);

	for(j = 0; j < 2; j++)
		myTokenize(tokenMain[j], s2, tokens[j], &tCount[j], 0);

	acceptable = validatePDBIdChainBP(&temp, tokens, tCount, aMap);

	//If chain is acceptable
	if(acceptable)
	{
		if(strlen(tokenMain[2]) > 0)//BP Info
		{
			acceptable = insertEdgeInfo(&temp, tokenMain[2]);
			if(acceptable)
				printFR3DInteraction(fptOut, temp, aMap.oldPdbId, tokenMain[2], tokenMain[3], tokenMain[4]);
		}
		else if(strlen(tokenMain[3]) > 0) //Stack Info
			printFR3DInteraction(fptOut, temp, aMap.oldPdbId, tokenMain[2], tokenMain[3], tokenMain[4]);
		else if(strlen(tokenMain[4]) > 0) // Decide later what to do about Phosphate Bond
			;
	}

	if(PRINT_TOKENS)
	{
		for(j = 0; j < mtCount; j++)
		{
			myTokenize(tokenMain[j], s2, tokens[0], &tCount[0], 0);
			for(k = 0; k < tCount[0]; k++)
				printf("Token %d.%d = %s\n", j, k, tokens[0][k]);
		}
		getchar();
	}
}

void analyzeTokens(char token[][LINE_SIZE], int count)
{
	isMultipleOrNoInteractionTag(token, count);

	findAndAddItem(basePair, &bpCount, token[2]);
	findAndAddItem(baseStack, &bsCount, token[3]);
	findAndAddItem(basePhosphate, &bPhCount, token[4]);
}

void findAndAddItem(char arr[][10], int *count, char str[])
{
	int i;
	if(strlen(str) == 0)
		return;
	for(i = 0; i < *count; i++)
	{
		if(strcmp(arr[i], str) == 0)
			return;
	}
	strcpy(arr[*count], str);
	(*count)++;
	return;
}

void printAllInteractionTypes(void)
{
	qsort(basePair, bpCount, 10*sizeof(char), compStr);
	qsort(baseStack, bsCount, 10*sizeof(char), compStr);
	qsort(basePhosphate, bPhCount, 10*sizeof(char), compStr);
	printInteractions(basePair, bpCount, "Base Pairs");
	printInteractions(baseStack, bsCount, "Base Stacks");
	printInteractions(basePhosphate, bPhCount, "Base Phosphates");
	getchar();
}

void printInteractions(char arr[][10], int count, char heading[])
{
	int i;
	printf("%s\n", heading);
	for(i = 0; i < count; i++)
		printf("%s\n", arr[i]);
}

//Unexpected behavior testing
int isMultipleOrNoInteractionTag(char token[][LINE_SIZE], int count)
{
	int i = 0, sum = 0;
	for(i = 2; i < count - 1; i++)
		sum += strlen(token[i]) > 0;
	if(sum > 1)
	{
		printf("Unwanted number of interaction\n");
		for(i = 0; i < count; i++)
			printf("%s ", token[i]);
		printf("\n");
		getchar();
		return 1;
	}
	return 0;
}
