/*
 * ReadMCAnnotateData.c
 *
 *  Created on: Jan 6, 2014
 *      Author: Shahidul
 */

#include "./inc/ReadMCAnnotateData.h"
#include "./inc/UtilityFunctions.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

enum
{
	RESIDUE,
	STACK_ADJACENT,
	STACK_NONADJACENT,
	BASE_PAIR,
	OTHERS
}context = RESIDUE;

int isTitle(char line[], int len);
void setContext(char line[], int len);
void parseStackAdjacentLines(char line[], int len, stackAdjInfo *info);
void parseBasePairLine(char line[], int len, basePairInfo *info);

int isPairIds(char line[], int len);
int isResidue(char line[], int len);
int isEdges(char line[], int len);
int isAdjacent(char line[], int len);
int isStackDirection(char line[], int len);
int isParallelInfo(char line[], int len);
int hasApos(char arr[]);
int matchChainMCA(char line[], char targetChain[]);

extern int compBasePairId(const void *a, const void *b);

int readMCAnnotateOutput(char inputFile[], basePairInfo basePairList[MAX_BASE_PAIR], int *basePairCount, char targetChain[], char stackOutput[])
{
	*basePairCount = stackAdjacentCount = 0;
	FILE *fpt = fopen(inputFile, "r");
	if(!fpt)
	{
		printf("%s File Not Found\n", inputFile);
		return 0;
	}

	FILE *stackOut = fopen(stackOutput, "w");
	if(!stackOut)
	{
		printf("%s File could not be created\n", stackOutput);
		return 0;
	}


	stackAdjInfo temp1;
	basePairInfo temp2;
	int i, isOneBond, len;
	char *oneBond = "one_hbond";
	char line[LINE_SIZE];
	int oneBLen = strlen(oneBond);

	while(fscanf(fpt, " %[^\n]", line) == 1)
	{
		len = strlen(line);
		if(isTitle(line, len))
		{
			setContext(line, len);
			if(context == RESIDUE || context == STACK_ADJACENT || context == STACK_NONADJACENT)
				fprintf(stackOut, "%s\n", line);
		}
		else
		{
			if(context == RESIDUE)
			{
				if(matchChainMCA(line, targetChain))
					fprintf(stackOut, "%s\n", line);
			}
			else if(context == STACK_ADJACENT)
			{
				parseStackAdjacentLines(line, len, &temp1);
				if(temp1.isPair)
				{
					if(strcmp(temp1.stackDirection, "inward") == 0 || strcmp(temp1.stackDirection, "outward") == 0)
					{
						stack_adj_list[stackAdjacentCount++] = temp1;
						//printf("%s\n", temp1.pair);
					}
				}
				if(matchChainMCA(line, targetChain))
					fprintf(stackOut, "%s\n", line);
			}
			else if(context == STACK_NONADJACENT)
			{
				if(matchChainMCA(line, targetChain))
					fprintf(stackOut, "%s\n", line);
			}
			else if(context == BASE_PAIR)
			{
				parseBasePairLine(line, len, &temp2);
				//printf("Step1 %s\n", line);
				//printf("%c %c %c\n", temp2.chainResidue[0], temp2.chainResidue[1], targetChain);
				if(ONLY_TARGET_CHAIN && (strcmp(temp2.chainResidue[0], targetChain) != 0 || strcmp(temp2.chainResidue[1], targetChain) != 0))
					continue;
				if(!FILTER_CIS_TRANS_UNSPECIFIED_BONDS || (temp2.glycosidicOrientation == 'c' || temp2.glycosidicOrientation == 't'))
				{
					isOneBond = 1;
					for(i = 0; i < oneBLen; i++)
						if(temp2.MC_SymNo[i] != oneBond[i])
						{
							isOneBond = 0;
							break;
						}
					if(!FILTER_ONEBOND || !isOneBond)
					{
						//printf("Step5 %s\n", line);
						//getchar();
						if(!FILTER_NON_WHS_BONDS || isWHS_BP(temp2))
							basePairList[(*basePairCount)++] = temp2;
					}
				}
			}
			else
			{
			}
		}
	}

	qsort(basePairList, *basePairCount, sizeof(basePairInfo), compBasePairId);

	if(TEST_PRINT_COUNT > 0)
		printf("MC-Annotate Pair Sample\n");
	for(i = 0; i < TEST_PRINT_COUNT; i++)
		printf("%d-%d %s\n", basePairList[i].residueID[0], basePairList[i].residueID[1], basePairList[i].MC_SymNo);

	if(stackOut)
		fclose(stackOut);
    if(fpt)
        fclose(fpt);

    return 1;
}

int matchChainMCA(char line[], char targetChain[])
{
	char tokensMain[2][LINE_SIZE], tokens[2][LINE_SIZE], s1[] = ":", s2[] = "-";
	int tokensMainCount = 0, tokensCount = 0;
	myTokenize(line, s1, tokensMain, &tokensMainCount, 1);
	myTokenize(tokensMain[0], s2, tokens, &tokensCount, 1);
	char chains[2][MAX_CHAIN_NAME_LENGTH + 1];
	int i, j, len;
	for(i = 0; i < tokensCount && i < 2; i++)
	{
		//printf("%s\n", tokens[i]);
		//getchar();
		len = strlen(tokens[i]);
		if(tokens[i][0] == '\'')
		{
			for(j = 0; j < len && j < MAX_CHAIN_NAME_LENGTH && tokens[i][j + 1] != '\''; j++)
				chains[i][j] = tokens[i][j + 1];
			chains[i][j] = 0;
		}
		else
		{
			for(j = 0; j < len && j < MAX_CHAIN_NAME_LENGTH && isalpha(tokens[i][j]); j++)
				chains[i][j] = tokens[i][j];
			chains[i][j] = 0;
		}
		if(strcmp(chains[i], targetChain) != 0)
			return 0;
	}

	return 1;
}

int isTitle(char line[], int len)
{
	if(line[len-1] == '-' && line[len-2] == '-' && line[len-3] == '-')
		return 1;
	return 0;
}

void setContext(char line[], int len)
{
	char texts[TITLE_COUNT][LINE_SIZE] = {"Residue conformations", "Adjacent stackings", "Non-Adjacent stackings", "Base-pairs"};
	int i, j, lenText, flag[TITLE_COUNT] = {1, 1, 1, 1};

	for(i = 0; i < TITLE_COUNT; i++)
	{
		lenText = strlen(texts[i]);
		for(j = 0; j < lenText; j++)
			if(j >= len || line[j] != texts[i][j])
			{
				flag[i] = 0;
				break;
			}
		if(flag[i]) break;
	}
	if(flag[0])
		context = RESIDUE;
	else if(flag[1])
		context = STACK_ADJACENT;
	else if(flag[2])
		context = STACK_NONADJACENT;
	else if(flag[3])
		context = BASE_PAIR;
	else
		context = OTHERS;
	return;
}

void parseStackAdjacentLines(char line[], int len, stackAdjInfo *info)
{
	int i, count = 0;
	for(i = 0; i < len; i++)
	{
		if(line[i] == ':')
		{
			i++;
			while(isWhitespace(line[i]))
				i++;
			break;
		}
		if(!isWhitespace(line[i]))
			info->pair[count++] = line[i];
	}
	info->pair[count++] = 0;
	count = 0;
	for(; i < len; i++)
	{
		if(isWhitespace(line[i]))
		{
			while(isWhitespace(line[i]))
				i++;
			break;
		}
		info->adjacency[count++] = line[i];
	}
	info->adjacency[count++] = 0;
	count = 0;
	for(; i < len; i++)
	{
		if(isWhitespace(line[i]))
		{
			while(isWhitespace(line[i]))
				i++;
			break;
		}
		info->stackDirection[count++] = line[i];
	}
	info->stackDirection[count++] = 0;
	count = 0;

	char pair[PART_SIZE] = "";
	for(; i < len; i++)
	{
		if(isWhitespace(line[i]))
			break;
		pair[count++] = line[i];
	}
	pair[count++] = 0;
	if(strcmp(pair, "pairing") == 0)
		info->isPair = 1;
	else
		info->isPair = 0;
	return;
}

void parseBasePairLine(char line[], int len, basePairInfo *info)
{
	enum{
		INITIAL,
		DETAILS,
		CROSSED_RESIDUE,
		CROSSED_EDGES,
		CROSSED_PAIRING,
		CROSSED_GLYCOSIDIC
	}state = INITIAL;
	int i, MC_SymNo_Cnt = 0, offset;
	char temp[PART_SIZE];
	int temp_len = 0;
	//strcpy(info->adjacent, "");
	strcpy(info->stackDirection, "");
	strcpy(info->MC_SymNo, "");
	info->isParallel = -1;
	info->edgeCount = 0;
	info->glycosidicOrientation = '\0';
	for(i = 0; i < len; i++)
	{
		if(i == len || line[i] == ' ')
		{
			temp[temp_len] = 0;
			if(state == INITIAL)
			{
				if(isPairIds(temp, temp_len))
				{
					if(hasApos(temp))
					{
						info->iCode[0] = info->iCode[1] = NO_ICODE;

						//Read residue info
						sscanf(temp, " \' %[^\n\r\' ] \' %d", &info->chainResidue[0], &info->residueID[0]);

						//Read if there is iCode
						offset = 0;
						while(temp[offset] != '-')
						{
							if(temp[offset] == '.')
								info->iCode[0] = temp[offset + 1];
							offset++;
						}

						//Read residue info
						sscanf(&temp[offset + 1], "\' %[^\n\r\' ] \' %d", &info->chainResidue[1], &info->residueID[1]);

						//Read if there is iCode
						while(temp[offset] != 0)
						{
							if(temp[offset] == '.')
								info->iCode[1] = temp[offset + 1];
							offset++;
						}

					}
					else
					{
						info->iCode[0] = info->iCode[1] = NO_ICODE;

						//Read residue info
						sscanf(temp, " %[^\n\r0-9 ] %d", &info->chainResidue[0], &info->residueID[0]);

						//Read if there is iCode
						offset = 0;
						while(temp[offset] != '-')
						{
							if(temp[offset] == '.')
							{
								info->iCode[0] = temp[offset + 1];
								//printf("%c %d %c\n", info->chainResidue[0], info->residueID[0], info->iCode[0]);
								//getchar();
							}
							offset++;
						}

						//Read residue info
						sscanf(&temp[offset + 1], " %[^\n\r0-9 ] %d", &info->chainResidue[1], &info->residueID[1]);

						//Read if there is iCode
						while(temp[offset] != 0)
						{
							if(temp[offset] == '.')
							{
								info->iCode[1] = temp[offset + 1];
								//printf("%c %d %c\n", info->chainResidue[1], info->residueID[1], info->iCode[1]);
								//getchar();
							}
							offset++;
						}
					}
					state = DETAILS;
				}
			}
			else if(state == DETAILS)
			{
				if(isResidue(temp, temp_len))
				{
					sscanf(temp," %c - %c ", &info->residue[0], &info->residue[1]);
					state = CROSSED_RESIDUE;
				}
			}
			else if(state == CROSSED_RESIDUE)
			{
				if(isEdges(temp, temp_len))
				{
					int ind = info->edgeCount;
					if(ind < EDGE_COUNT_LIMIT)
					{
						sscanf(temp, " %[^/] / %s", info->aEdges[ind].edge1, info->aEdges[ind].edge2);
						info->edgeCount++;
					}
				}
				else
					state = CROSSED_EDGES;
			}

			if(state == CROSSED_EDGES)
			{
				if(strcmp(temp, "pairing") == 0)
					state = CROSSED_PAIRING;
				else if(isAdjacent(temp, temp_len))
					;//strcpy(info->adjacent, temp);
				else if(isStackDirection(temp, temp_len))
					strcpy(info->stackDirection, temp);
				else
					printf("\n!!!!!---ERROR AFTER EDGE DESCRIPTION---!!!!!\n");
			}
			else if(state == CROSSED_PAIRING)
			{
				if(isParallelInfo(temp, temp_len))
					if(strcmp(temp, "parallel") == 0)
						info->isParallel = 1;
					else
						info->isParallel = 0;
				else if(isGlycosidicOrientation(temp, temp_len))
				{
					if(strcmp(temp, "cis") == 0)
						info->glycosidicOrientation = 'c';
					else
						info->glycosidicOrientation = 't';
					state = CROSSED_GLYCOSIDIC;
				}
				else
					printf("\n!!!!!---ERROR AFTER PAIRING---!!!!!\n\n");
			}
			else if(state == CROSSED_GLYCOSIDIC)
			{
				if(MC_SymNo_Cnt > 0)
					strcat(info->MC_SymNo, " ");
				strcat(info->MC_SymNo, temp);
				MC_SymNo_Cnt = 1;
			}
			temp_len = 0;
		}
		if(!isWhitespace(line[i]))
			temp[temp_len++] = line[i];
	}
}

int isPairIds(char line[], int len)
{
	if(!hasApos(line))
		return 1;
	int i, apo_count = 0, has_hiphen = 1;
	for(i = 0; i < len; i++)
		if(line[i] == '\'')
			apo_count++;
		else if(line[i] == '-')
			has_hiphen = 1;
	if(apo_count == 4 && has_hiphen == 1)
		return 1;
	return 0;
}

int isResidue(char line[], int len)
{
	if(line[1] == '-')
		return 1;
	return 0;
}

int isEdges(char line[], int len)
{
	int i;
	for(i = 0; i < len; i++)
		if(line[i] == '/')
			return 1;
	return 0;
}

int isAlphanumeric(char line[], int len)
{
	int i;
	for(i = 0; i < len; i++)
		if((line[i] >= 'A' && line[i] <= 'Z') || (line[i] >= 'A' && line[i] <= 'Z') || (line[i] >= '0' && line[i] <= '9'))
			return 1;
	return 0;
}

int isAdjacent(char line[], int len)
{
	char *pos = strstr(line, "adjacent");
	if(pos)
		return 1;
	return 0;
}

int isStackDirection(char line[], int len)
{
	char *arr[4] = {"inward", "outward", "upward", "downward"};
	int i;
	for(i = 0; i < 4; i++)
		if(strcmp(arr[i], line) == 0)
			return 1;
	return 0;
}

int isParallelInfo(char line[], int len)
{
	char *arr[2] = {"parallel", "antiparallel"};
	int i;
	for(i = 0; i < 2; i++)
		if(strcmp(arr[i], line) == 0)
			return 1;
	return 0;
}

int hasApos(char arr[])
{
	int i, len = strlen(arr);
	for(i = 0; i < len; i++)
		if(arr[i] == '\'')
			return 1;
	return 0;
}

int isGlycosidicOrientation(char line[], int len)
{
	char *arr[2] = {"cis", "trans"};
	int i;
	for(i = 0; i < 2; i++)
		if(strcmp(arr[i], line) == 0)
			return 1;
	return 0;
}

int isWhitespace(char c)
{
    if(c == ' ' || c == '\n' || c == '\t' || c == '\r')
        return 1;
    return 0;
}
