/*
 * ReadInputData.c
 *
 *  Created on: Nov 7, 2013
 *  Author: Shahidul
 */

#include "./inc/ReadPDBData.h"
#include "./inc/UtilityFunctions.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#define TARGET_ATOM_FIELD_COUNT 11

void readmmCIFFormat(FILE *fpt, PDB_RefineInfo *aPDB_RefineInfo, char targetChain[]);

int parseAtomInfo_PDBFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo, char targetChain[]);
void parseRefineInfo_PDBFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo);

void memStrCpy(char dest[], char source[], int len);
int isRNAInfo(char c);
int compareAtoms(const void *a, const void *b);

void calculateDPI(PDB_RefineInfo *aPDB_RefineInfo);
void printResSeq(int seqPos, char iCode, int len);
void printResSeqFile(int seqPos, char iCode, int len, FILE*fpt);
void getWhiteSpaceFreeString(char line[], char whiteSpaceFreeLine[]);

extern atomInfo atomList[MAX_ATOM_COUNT];
extern motifIndices candidateMotifs[MAX_MOTIF_FREQ];
extern angleDefinition angleDef[TARGET_ANGLES_COUNT];
extern int atomCount;
extern int candidateCount;
extern int angleDefCount;
//extern int removeHelix;

typedef struct{
	int gotCompleteness;
	int gotResolution;
	int gotFreeRValue;
	int gotNumberOfAtoms;
	int gotNunmberOfObservations;
}refineInfoFlag;

int readPDBFile(char pathInput[], char PDB_name[], PDB_RefineInfo *aPDB_RefineInfo, char targetChain[])
{
	atomCount = 0;
	memset(atomList, 0, sizeof(atomList));

	aPDB_RefineInfo->completeness = 0.0;
	aPDB_RefineInfo->resolution = 0.0;
	aPDB_RefineInfo->freeRValue = 0.0;
	aPDB_RefineInfo->numberOfAtoms = 0;
	aPDB_RefineInfo->nunmberOfObservations = 0;

	char line[LINE_SIZE];
	char inputFile[MAX_FILE_SZ] = "";
	//Attempt to open .pdb file
	strcpy(inputFile, pathInput);
	strcat(inputFile, PDB_name);
	strcat(inputFile, ".pdb");
	FILE *fpt = fopen(inputFile, "r");
	if(!fpt)
	{
		//Attempt to open mmCIF (.cif) format file
		strcpy(inputFile, pathInput);
		strcat(inputFile, PDB_name);
		strcat(inputFile, ".cif");
		fpt = fopen(inputFile, "r");
		if(!fpt)
		{
			printf("Neither .pdb or .cif file for PDB %s was found in the %s folder\n", PDB_name, pathInput);
			return 0;
		}
		//Read from mmCIF
		readmmCIFFormat(fpt, aPDB_RefineInfo, targetChain);
	}
	else
	{
		int startedAtomBlock = 0;

		while(fscanf(fpt, " %[^\n]", line) == 1)
			if(strlen(line) > 0)
			{
				if(!startedAtomBlock)
					parseRefineInfo_PDBFormat(line, aPDB_RefineInfo);
				if(parseAtomInfo_PDBFormat(line, aPDB_RefineInfo, targetChain) == 1)
					startedAtomBlock = 1;
			}
	}

	calculateDPI(aPDB_RefineInfo);

	qsort(atomList, atomCount, sizeof(atomInfo), compareAtoms);
	if(PRINT_ALIGN_BP_INFO)
		printf("Atom Count = %d\n", atomCount);
	//	printf("%0.4lf, %0.4lf, %0.4lf, %d (%d)\n", aPDB_RefineInfo->resolution, aPDB_RefineInfo->freeRValue, aPDB_RefineInfo->completeness, aPDB_RefineInfo->nunmberOfObservations, aPDB_RefineInfo->numberOfAtoms);
	//	getchar();

    if(fpt)
        fclose(fpt);
	return 1;
}

//Functions related to reading mmCIF format file

void getResolutionInfo_mmCIFFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo)
{
    char *resolutionField[2] = {"_refine.ls_d_res_high", "_refine_hist.d_res_high"};//, "_reflns_shell.d_res_high", "_refine_ls_shell.d_res_high"
    char temp1[LINE_SIZE], temp2[LINE_SIZE];
    double resolution = 100.0; //Worst possible reolutions initialized
    int i;

    for(i = 0; i < 2; i++)
        if(strncmp(resolutionField[i], line, strlen(resolutionField[i])) == 0)
        {
            sscanf(line, "%s %s", temp1, temp2);
            if(strcmp(temp1, resolutionField[i]) == 0 && isdigit(temp2[0]))
                if(sscanf(temp2, "%lf", &resolution) == 1)
                    if(aPDB_RefineInfo->resolution < ERR || resolution < aPDB_RefineInfo->resolution)
                        aPDB_RefineInfo->resolution = resolution;
        }
}

void getFreeRValue_mmCIFFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo)
{
    char *freeRValueField[2] = {"_refine.ls_R_factor_R_free", "_refine_ls_shell.R_factor_R_free"};
    char temp1[LINE_SIZE], temp2[LINE_SIZE];
    double freeRValue = 1.0; //Worst Free R Value initialized
    int i;

    for(i = 0; i < 2; i++)
        if(strncmp(freeRValueField[i], line, strlen(freeRValueField[i])) == 0)
        {
            sscanf(line, "%s %s", temp1, temp2);
            if(strcmp(temp1, freeRValueField[i]) == 0 && isdigit(temp2[0]))
                if(sscanf(temp2, "%lf", &freeRValue) == 1)
                    if(aPDB_RefineInfo->freeRValue < ERR || freeRValue < aPDB_RefineInfo->freeRValue)
                        aPDB_RefineInfo->freeRValue = freeRValue;
        }
}

void getCompleteness_mmCIFFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo)
{
    char *completenessField[2] = {"_reflns.percent_possible_obs", "_refine.ls_percent_reflns_obs"};
    char temp1[LINE_SIZE], temp2[LINE_SIZE];
    double completeness = 0.0; //Worst completeness initialized
    int i;

    for(i = 0; i < 2; i++)
        if(strncmp(completenessField[i], line, strlen(completenessField[i])) == 0)
        {
            sscanf(line, "%s %s", temp1, temp2);
            if(strcmp(temp1, completenessField[i]) == 0 && isdigit(temp2[0]))
                if(sscanf(temp2, "%lf", &completeness) == 1)
                {
                    completeness /= 100.0;
                    if(aPDB_RefineInfo->completeness < ERR || completeness > aPDB_RefineInfo->completeness)
                        aPDB_RefineInfo->completeness = completeness;
                }
        }
}

void getNoOfReflection_mmCIFFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo)
{
    char *observationField[2] = {"_reflns.number_obs", "_refine.ls_number_reflns_obs"};
    char temp1[LINE_SIZE], temp2[LINE_SIZE];
    int nunmberOfObservations = 0; //Worst observation initialized
    int i;

    for(i = 0; i < 2; i++)
        if(strncmp(observationField[i], line, strlen(observationField[i])) == 0)
        {
            sscanf(line, "%s %s", temp1, temp2);
            if(strcmp(temp1, observationField[i]) == 0 && isdigit(temp2[0]))
                if(sscanf(temp2, "%d", &nunmberOfObservations) == 1)
                    if(aPDB_RefineInfo->nunmberOfObservations < ERR || nunmberOfObservations > aPDB_RefineInfo->nunmberOfObservations)
                        aPDB_RefineInfo->nunmberOfObservations = nunmberOfObservations;
        }
}

void findTargetIndex(int fieldIndex, char field[], char targetFields[][PART_SIZE], int targetIndices[])
{
	int i, len = strlen(field);
	while(field[len-1] == ' ')
        len--;
    field[len] = 0;


	for(i = 0; i < TARGET_ATOM_FIELD_COUNT; i++)
	{
		if(strcmp(targetFields[i], field) == 0)
		{
			targetIndices[i] = fieldIndex;
			break;
		}
	}
}

int isNewBlock(char line[])
{
    if(line[0] == '#' || line[0] == '_' || strncmp(line, "loop_", 5) == 0)
        return 1;
    return 0;
}


//2.5000, 0.2830, 0.9980, 487131 (52245)
#define REFINE_INFO_COUNT 4
void parseRefineInfo_mmCIFFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo)
{
	
	char targetFieldName[REFINE_INFO_COUNT][PART_SIZE] = {"resolution",    "Number of obs",        "Completeness",          "free R value"};
	char targetFields[REFINE_INFO_COUNT][PART_SIZE] =    {"ls_d_res_high", "ls_number_reflns_obs", "ls_percent_reflns_obs", "ls_R_factor_R_free"};
	int targetIndices[REFINE_INFO_COUNT] = {0};
	
	char fieldsList[REFINE_INFO_COUNT];
	//char line[LINE_SIZE * 10];
	int i, isLoop = 0, fieldCount = 0, startedRefine = 0, startedRefineSite =0;
	
	
	char s[3] = {" "};
	char **tokenMain = NULL;
	int mtCount;
	
	/***Read when the refine info is in the loop***/
	
	//Read mmCIF format file
	/*while(fscanf(fpt, " %[^\n]", line) == 1)
	{
		parseRefineInfo_mmCIFFormat(line, aPDB_RefineInfo);
		//Detect the end of atom info block
		if(startedAtom == 1)
		{
			if(isNewBlock(line))
			{
				for(i = 0; i < fieldCount; i++)
					if(tokenMain[i])
						free(tokenMain[i]);
				if(tokenMain)
					free(tokenMain);
				tokenMain = NULL;
				startedAtom = 0;
				startedAtomSite = 0;
				isLoop = 0;
			}
		}
		//First make sure a loop_ block started
		if(strncmp(line, "loop_", 5) == 0)
		{
			isLoop = 1;
			fieldCount = 0;
			startedAtom = 0;
		}
		else if(isLoop == 1)
		{
			//Make sure if atom site is started
			if(strncmp(line, "_atom_site.", 11) == 0)
			{
				findTargetIndex(fieldCount, &line[11], targetFields, targetIndices);
				fieldCount++;
				startedAtomSite = 1;
			}
			// Find atom info
			else if(startedAtomSite)
			{
				//Check if atom entry started
				if(strncmp(line, "ATOM", 4) == 0)
				{
					//To allocate memory for the tokens at the beginning of ATOM list
					if(startedAtom == 0)
					{
						tokenMain = malloc(sizeof(char*) * (fieldCount + 1));
						for(i = 0; i < fieldCount; i++)
							tokenMain[i] = malloc(sizeof(char) * LINE_SIZE);
						startedAtom = 1;
					}
	
					//Pick target values
					myTokenizeDynamic(line, s, tokenMain, &mtCount, 1);
					setAtomInfoCIFForma(targetIndices, tokenMain, targetChain, aPDB_RefineInfo);                    
				}
			}
		}
	}*/
	
	
	/***Read when the refine info is NOT in the loop***/ 
	
	//Get Resolution Info
	getResolutionInfo_mmCIFFormat(line, aPDB_RefineInfo);

	//Get Free R Value Info
	getFreeRValue_mmCIFFormat(line, aPDB_RefineInfo);

	//Get Completeness Info
	getCompleteness_mmCIFFormat(line, aPDB_RefineInfo);

	//Get Number of Reflection (/Observation)
	getNoOfReflection_mmCIFFormat(line, aPDB_RefineInfo);
}

void setAtomInfoCIFForma(int targetIndices[], char **tokenMain, char targetChain[], PDB_RefineInfo *aPDB_RefineInfo)
{
    //Use the value in "auth_asym_id" field if there is a valid chain id
    if(isalnum(tokenMain[targetIndices[2]][0]))
        strcpy(atomList[atomCount].chainId, tokenMain[targetIndices[2]]);
    //Otherwise use the value in "label_asym_id" field
    else
        strcpy(atomList[atomCount].chainId, tokenMain[targetIndices[1]]);

    //If not RNA info return
    if(!(strlen(tokenMain[targetIndices[0]]) && isRNAInfo(tokenMain[targetIndices[0]][0])))
        return;
    //Copy nucleic acid info
    atomList[atomCount].residueName = tokenMain[targetIndices[0]][0];

    //Copy atom name
    int startIndex = 0, len = strlen(tokenMain[targetIndices[3]]);
    while(tokenMain[targetIndices[3]][startIndex] == '\"')
        startIndex++;
    while(tokenMain[targetIndices[3]][len - 1] == '\"')
        len--;
    tokenMain[targetIndices[3]][len] = 0;
    strcpy(atomList[atomCount].atomName, &tokenMain[targetIndices[3]][startIndex]);

    //Copy nucleic acid sequence number and iCode
    sscanf(tokenMain[targetIndices[4]], " %d", &atomList[atomCount].resSeqNo);
    if(isalnum(tokenMain[targetIndices[5]][0]))
        atomList[atomCount].iCode = tokenMain[targetIndices[5]][0];
    else
        atomList[atomCount].iCode = NO_ICODE;

    sscanf(tokenMain[targetIndices[6]], " %lf", &atomList[atomCount].coOrd.x);
	sscanf(tokenMain[targetIndices[7]], " %lf", &atomList[atomCount].coOrd.y);
    sscanf(tokenMain[targetIndices[8]], " %lf", &atomList[atomCount].coOrd.z);
	sscanf(tokenMain[targetIndices[9]], " %lf", &atomList[atomCount].occupancy);
	sscanf(tokenMain[targetIndices[10]], " %lf", &atomList[atomCount].tempFactor);

    //printf("%s - %c - %s %d.%c\n", atomList[atomCount].chainId, atomList[atomCount].residueName, atomList[atomCount].atomName, atomList[atomCount].resSeqNo, atomList[atomCount].iCode);
    //printf("%0.2lf, %0.2lf, %0.2lf, %0.2lf, %0.2lf\n", atomList[atomCount].coOrd.x, atomList[atomCount].coOrd.y, atomList[atomCount].coOrd.z, atomList[atomCount].occupancy, atomList[atomCount].tempFactor);
    //getchar();

	//If chain is the target chain add to the atom list
	if(strcmp(atomList[atomCount].chainId, targetChain) == 0)
		atomCount++;
	
	//Increase atom count for "ATOM" entry
	aPDB_RefineInfo->numberOfAtoms++;
}

void readmmCIFFormat(FILE *fpt, PDB_RefineInfo *aPDB_RefineInfo, char targetChain[])
{
	char targetFieldName[TARGET_ATOM_FIELD_COUNT][PART_SIZE] = {"residueName",   "chainId1",      "chainId2",     "atomName",      "resSeqNo",     "iCode",
													 	 	 	"X",       "Y",       "Z",       "Occupancy",     "TempFacor/B Value"};
	char targetFields[TARGET_ATOM_FIELD_COUNT][PART_SIZE] =    {"label_comp_id", "label_asym_id", "auth_asym_id", "label_atom_id", "label_seq_id", "pdbx_PDB_ins_code",
													 	 	 	"Cartn_x", "Cartn_y", "Cartn_z", "occupancy", "B_iso_or_equiv"};
	int targetIndices[TARGET_ATOM_FIELD_COUNT] = {0};

	char fieldsList[TARGET_ATOM_FIELD_COUNT];
	char line[LINE_SIZE * 10];
	int i, isLoop = 0, fieldCount = 0, startedAtom = 0, startedAtomSite =0;


	char s[3] = {" "};
	char **tokenMain = NULL;
	int mtCount;

	//Read mmCIF format file
	while(fscanf(fpt, " %[^\n]", line) == 1)
    {
        parseRefineInfo_mmCIFFormat(line, aPDB_RefineInfo);
        //Detect the end of atom info block
        if(startedAtom == 1)
        {
            if(isNewBlock(line))
            {
                for(i = 0; i < fieldCount; i++)
                    if(tokenMain[i])
                        free(tokenMain[i]);
                if(tokenMain)
                    free(tokenMain);
                tokenMain = NULL;
                startedAtom = 0;
                startedAtomSite = 0;
                isLoop = 0;
            }
        }
        //First make sure a loop_ block started
        if(strncmp(line, "loop_", 5) == 0)
        {
            isLoop = 1;
            fieldCount = 0;
            startedAtom = 0;
        }
        else if(isLoop == 1)
        {
            //Make sure if atom site is started
            if(strncmp(line, "_atom_site.", 11) == 0)
            {
                findTargetIndex(fieldCount, &line[11], targetFields, targetIndices);
                fieldCount++;
                startedAtomSite = 1;
            }
            // Find atom info
            else if(startedAtomSite)
            {
                //Check if atom entry started
                if(strncmp(line, "ATOM", 4) == 0)
                {
                    //To allocate memory for the tokens at the beginning of ATOM list
                    if(startedAtom == 0)
                    {
                        tokenMain = malloc(sizeof(char*) * (fieldCount + 1));
                        for(i = 0; i < fieldCount; i++)
                            tokenMain[i] = malloc(sizeof(char) * LINE_SIZE);
                        startedAtom = 1;
                    }

                    //Pick target values
                    myTokenizeDynamic(line, s, tokenMain, &mtCount, 1);

                    //Test Print
                    /*for(i = 0; i < mtCount; i++)
                        printf("%2d. %s\n", i, tokenMain[i]);
                    for(i = 0; i < TARGET_ATOM_FIELD_COUNT; i++)
                        printf("%d ", targetIndices[i]);
                    printf("\n");
                    printf("-- (%d) --\n", aPDB_RefineInfo->numberOfAtoms);
                    getchar();*/

                    setAtomInfoCIFForma(targetIndices, tokenMain, targetChain, aPDB_RefineInfo);
                }
                //Increase atom count if "HETATM" entry
                else if(strncmp(line, "HETATM", 4) == 0)
                    aPDB_RefineInfo->numberOfAtoms++;

            }
        }
    }
    //printf("Free R Value = %0.3lf\n", aPDB_RefineInfo->freeRValue);
    //printf("Completeness = %0.3lf\n", aPDB_RefineInfo->completeness);
    //printf("Obs = %d\n", aPDB_RefineInfo->nunmberOfObservations);
	//getchar();
    //printf("Test Atom Count = %d\n", atomCount);
    //getchar();

}


//Functions related to reading PDB format file

void getFreeRValue_PDBFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo)
{
	char whiteSpaceFreeLine[LINE_SIZE];

	if(strncmp(line, "REMARK   3", 10) == 0)
	{
		getWhiteSpaceFreeString(line, whiteSpaceFreeLine);
		if(strncmp(&whiteSpaceFreeLine[7], "FREERVALUE:", 11) == 0 && strncmp(&whiteSpaceFreeLine[18], "NULL", 4) != 0)
			sscanf(&whiteSpaceFreeLine[18], "%lf", &aPDB_RefineInfo->freeRValue);
	}

}

void getCompleteness_PDBFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo)
{
	if(strncmp(line, "REMARK", 6) == 0)
	{
		if(strstr(line, "COMPLETENESS") != NULL)
		{
			char *ptr = strstr(line, ":");
			if(strncmp(ptr+2, "NULL", 4) != 0)
			{
				double completeness;
				sscanf(ptr+2, "%lf", &completeness);
				if(completeness / 100.0 > aPDB_RefineInfo->completeness)
					aPDB_RefineInfo->completeness = completeness / 100.0;
			}
		}
	}
}

void getNoOfReflection_PDBFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo)
{
	if(strncmp(line, "REMARK", 6) == 0)
	{
		if(strstr(line, "NUMBER OF REFLECTIONS") != NULL)
		{
			char *ptr = strstr(line, ":");
			if(strncmp(ptr+2, "NULL", 4) != 0)
				sscanf(ptr+2, "%d", &aPDB_RefineInfo->nunmberOfObservations);
		}
	}
}

void parseRefineInfo_PDBFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo)
{
	//Get Resolution Info
	if(strncmp(line, "REMARK   2", 10) == 0 && strncmp(&line[11], "RESOLUTION.", 11) == 0 && strncmp(&line[23], "NULL", 4) != 0)
		sscanf(&line[23], "%lf", &aPDB_RefineInfo->resolution);

	//Get Free R Value Info
	getFreeRValue_PDBFormat(line, aPDB_RefineInfo);

	//Get Completeness Info
	getCompleteness_PDBFormat(line, aPDB_RefineInfo);

	//Get Number of Reflection (/Observation)
	getNoOfReflection_PDBFormat(line, aPDB_RefineInfo);
}

int parseAtomInfo_PDBFormat(char line[], PDB_RefineInfo *aPDB_RefineInfo, char targetChain[])
{
	char token[PART_SIZE];
	if(strncmp(line, "ATOM", 4) == 0)
	{
		if(line[18] == ' ' && isRNAInfo(line[19]))// To ignore protein and other atom info
		{
			atomList[atomCount].residueName = line[19];
			atomList[atomCount].chainId[0] = line[21];
			atomList[atomCount].chainId[1] = 0;
			memStrCpy(token, &line[12], 4);
			sscanf(token, " %s", atomList[atomCount].atomName);
			memStrCpy(token, &line[22], 4);
			sscanf(token, " %d", &atomList[atomCount].resSeqNo);
			atomList[atomCount].iCode = line[26];
			memStrCpy(token, &line[30], 8);
			sscanf(token, " %lf", &atomList[atomCount].coOrd.x);
			memStrCpy(token, &line[38], 8);
			sscanf(token, " %lf", &atomList[atomCount].coOrd.y);
			memStrCpy(token, &line[46], 8);
			sscanf(token, " %lf", &atomList[atomCount].coOrd.z);
			memStrCpy(token, &line[54], 6);
			sscanf(token, " %lf", &atomList[atomCount].occupancy);
			memStrCpy(token, &line[60], 6);
			sscanf(token, " %lf", &atomList[atomCount].tempFactor);
            if(strcmp(atomList[atomCount].chainId, targetChain) == 0)
                atomCount++;
		}
		aPDB_RefineInfo->numberOfAtoms++;
		return 1;
	}
	else if(strncmp(line, "HETATM", 4) == 0)
		aPDB_RefineInfo->numberOfAtoms++;
	return 0;
}


//Supporting fuctions, independent of data format
void calculateDPI(PDB_RefineInfo *aPDB_RefineInfo)
{
	if(aPDB_RefineInfo->completeness > ERR &&
	aPDB_RefineInfo->resolution > ERR &&
	aPDB_RefineInfo->freeRValue > ERR &&
	aPDB_RefineInfo->numberOfAtoms > 0 &&
	aPDB_RefineInfo->nunmberOfObservations > 0)
	{
		double n_a_by_n_o = ((double)aPDB_RefineInfo->numberOfAtoms) / aPDB_RefineInfo->nunmberOfObservations;
		//aPDB_RefineInfo->dpi = 0.65 * sqrt((n_a_by_n_o)*(aPDB_RefineInfo->freeRValue*aPDB_RefineInfo->freeRValue)*(aPDB_RefineInfo->resolution*aPDB_RefineInfo->resolution)*pow(aPDB_RefineInfo->completeness, -2/3));
		aPDB_RefineInfo->dpi = sqrt((n_a_by_n_o)) * aPDB_RefineInfo->freeRValue*aPDB_RefineInfo->resolution*pow(aPDB_RefineInfo->completeness, -(1.0/3.0));
	}
	else
		aPDB_RefineInfo->dpi = 0.0;
}

int compareAtoms(const void *a, const void *b)
{
	atomInfo *x = (atomInfo*)a;
	atomInfo *y = (atomInfo*)b;
	if(strcmp(x->chainId, y->chainId) == 0)
		if(x->resSeqNo == y->resSeqNo)
			if(x -> iCode == y -> iCode)
				return strcmp(x -> atomName, y -> atomName);
			else
				return x -> iCode - y -> iCode;
		else
			return x->resSeqNo - y->resSeqNo;
	else
		return strcmp(x->chainId, y->chainId);
}

void printResSeq(int seqPos, char iCode, int len)
{
	if(iCode == NO_ICODE)
	{
		if(len < 4)
			len = 4;
	}
	else
		if(len < 6)
			len = 6;

	char *temp = (char *)malloc((len + 1) *  sizeof(char));

	if(iCode == NO_ICODE)
		sprintf(temp, "%d", seqPos);
	else
		sprintf(temp, "%d.%c", seqPos, iCode);
	printf("%-*s", len, temp);
}

void printResSeqFile(int seqPos, char iCode, int len, FILE*fpt)
{
	if(iCode == NO_ICODE)
	{
		if(len < 4)
			len = 4;
	}
	else
		if(len < 6)
			len = 6;

	char *temp = (char *)malloc((len + 1) *  sizeof(char));

	if(iCode == NO_ICODE)
		sprintf(temp, "%d", seqPos);
	else
		sprintf(temp, "%d.%c", seqPos, iCode);
	fprintf(fpt, "%-*s", len, temp);
}

void getWhiteSpaceFreeString(char line[], char whiteSpaceFreeLine[])
{
	int i, j, len = strlen(line);
	for(i = 0, j = 0; i < len; i++)
		if(line[i] != ' ')
			whiteSpaceFreeLine[j++] = line[i];
	whiteSpaceFreeLine[j] = 0;
}

void memStrCpy(char dest[], char source[], int len)
{
	int i;
	for(i = 0; i < len; i++)
		dest[i] = source[i];
	dest[len] = 0;
}

int isRNAInfo(char c)
{
	if(c == 'A' || c == 'C' || c == 'G' || c == 'U')
		return 1;
	return 0;
}
