/*
 * ReadRNAViewData.c
 *
 *  Created on: Feb 4, 2014
 *      Author: Shahidul
 */

#include "./inc/ReadRNAViewData.h"
#include "./inc/UtilityFunctions.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define BASE_PAIR_START "BEGIN_base-pair"
#define BASE_PAIR_END "END_base-pair"

enum
{
	PAIR_START,
	PAIR_END,
	OTHERS
}contextRNAView = OTHERS;

int isTitleRNAView(char line[], int len);
void setContextRNAView(char line[], int len);
int parsePairLineRNAView(char line[], int len, basePairInfo *info);

int hasBasePair(char line[]);
extern int isGlycosidicOrientation(char line[], int len);
extern int isAlphanumeric(char line[], int len);
extern int isWhitespace(char c);
extern int compBasePairId(const void *a, const void *b);
int filterEdge(edges aEdge, char MC_SymNo[]);
int matchChainRV(char line[], char targetChain[]);

int readRNAViewOutput(char inputFile[], basePairInfo basePairRNAView[MAX_BASE_PAIR], int *basePairCountRNAView, char targetChain[], char stackOutput[])
{
	*basePairCountRNAView = 0;
	FILE *fpt = fopen(inputFile, "r");
	if(!fpt)
	{
		printf("%s File Not Found\n", inputFile);
		return 0;
	}

	FILE *stackOut = fopen(stackOutput, "w");
	if(!stackOut)
	{
		printf("%s File Not Found\n", stackOutput);
		return 0;
	}

	basePairInfo temp2;
	int i, len;
	char line[LINE_SIZE];

	while(fscanf(fpt, " %[^\n]", line) == 1)
	{
	    //printf("--- %s\n", line);
		len = strlen(line);
		if(isTitleRNAView(line, len))
			setContextRNAView(line, len);
		else
		{
			if(contextRNAView == PAIR_START)
			{
				if(parsePairLineRNAView(line, len, &temp2))
				{
					if(ONLY_TARGET_CHAIN && (strcmp(temp2.chainResidue[0], targetChain) != 0 || strcmp(temp2.chainResidue[1], targetChain) != 0))
						continue;
					if(!FILTER_CIS_TRANS_UNSPECIFIED_BONDS || temp2.glycosidicOrientation == 'c' || temp2.glycosidicOrientation == 't')
					{
						if(!FILTER_UNWANTED_BONDS_RNA_VIEW || !filterEdge(temp2.aEdges[0], temp2.MC_SymNo))
						{
							if(!FILTER_NON_WHS_BONDS || isWHS_BP(temp2))
								basePairRNAView[(*basePairCountRNAView)++] = temp2;
						}
					}
				}
				else
				{
					if(matchChainRV(line, targetChain))
                    {
                        //printf("%s\n", line);
                        //printf("%d\n", stackOut);
                        fprintf(stackOut, "%s\n", line);
                    }
				}
			}
			else if(contextRNAView == PAIR_END)
			{

			}
			else
			{
			}
		}
	}

	qsort(basePairRNAView, *basePairCountRNAView, sizeof(basePairInfo), compBasePairId);
	if(TEST_PRINT_COUNT > 0)
		printf("RNA-View Pair Sample\n");
	for(i = 0; i < TEST_PRINT_COUNT; i++)
	{
		printPairInfoOnScreen(basePairRNAView[i]);
		//char ch = getchar();
		//if(ch == 'X')
			//break;
	}
	if(stackOut)
		fclose(stackOut);
    if(fpt)
        fclose(fpt);
    return 1;
}

int matchChainRV(char line[], char targetChain[])
{
	char tokens[3][LINE_SIZE], s1[] = ":";
	int tokensCount = 0;
	myTokenize(line, s1, tokens, &tokensCount, 1);
	char chains[2][MAX_CHAIN_NAME_LENGTH + 1];
	int i, j, len;
	for(i = 0; i < tokensCount && i < 2; i++)
	{
		len = strlen(tokens[i]);
		for(j = len - 1; j >=0 && tokens[i][j] != ' '; j--)
			;
		strcpy(chains[i], &tokens[i][j + 1]);
		if(strcmp(chains[i], targetChain) == 0)
			return 0;
	}
	return 1;
}

int filterEdge(edges aEdge, char MC_SymNo[])
{
	int i, len = strlen(MC_SymNo);
	for(i = 0; i < len; i++)
		if(MC_SymNo[i] == '!')
			return 1;
	//if(EXTRA_FILTER && strcmp(MC_SymNo, "n/a") == 0)
		//return 1;
	if(aEdge.edge1[0] == '.' || aEdge.edge2[0] == '.' || aEdge.edge1[0] == '?' || aEdge.edge2[0] == '?')
		return 1;
	return 0;
}

int isTitleRNAView(char line[], int len)
{
	if(strcmp(line, BASE_PAIR_START) == 0 || strcmp(line, BASE_PAIR_END) == 0)
		return 1;
	return 0;
}

void setContextRNAView(char line[], int len)
{
	char texts[TITLE_COUNT][LINE_SIZE] = {BASE_PAIR_START, BASE_PAIR_END};
	int i, j, lenText, flag[TITLE_COUNT] = {1, 1};

	for(i = 0; i < TITLE_COUNT; i++)
	{
		lenText = strlen(texts[i]);
		for(j = 0; j < lenText; j++)
			if(j >= len || line[j] != texts[i][j])
			{
				flag[i] = 0;
				break;
			}
		if(flag[i]) break;
	}
	if(flag[0])
		contextRNAView = PAIR_START;
	else if(flag[1])
		contextRNAView = PAIR_END;
	else
		contextRNAView = OTHERS;
	return;
}

void generalizeEdgeAnnotation(char *aEdge)
{
	if(*aEdge == '-' || *aEdge == '+')
		*aEdge = 'W';
}

int parsePairLineRNAView(char line[], int len, basePairInfo *info)
{
	char temp[PART_SIZE];
	char part1[PART_SIZE], part2[PART_SIZE], part3[PART_SIZE];
	//strcpy(info->adjacent, "");
	strcpy(info->stackDirection, "");
	strcpy(info->MC_SymNo, "");
	info->isParallel = -1;
	info->edgeCount = 0;
	info->glycosidicOrientation = 0;

	sscanf(line, " %[^\n\r:]: %[^\n\r:] : %[^\n\r:]", part1, part2, part3);
	sscanf(part1, " %[^\n,], %[^\n\r: ]", temp, &info->chainResidue[0]);
	sscanf(part2, " %d %c - %c %d %[^\n\r: ]", &info->residueID[0], &info->residue[0], &info->residue[1], &info->residueID[1], &info->chainResidue[1]);
	//info->chainResidue[0] = info->chainResidue[1];
	info->iCode[0] = info->iCode[1] = NO_ICODE;

	if(hasBasePair(part3))
	{
		if(strstr(part3, "syn"))
			sscanf(part3, " %[^\n\r\t /]/%s %s syn %s", info->aEdges[0].edge1, info->aEdges[0].edge2, temp, info->MC_SymNo);
		else
			sscanf(part3, " %[^\n\r\t /]/%s %s %s", info->aEdges[0].edge1, info->aEdges[0].edge2, temp, info->MC_SymNo);
		info->aEdges[0].edge1[0] = toupper(info->aEdges[0].edge1[0]);
		info->aEdges[0].edge2[0] = toupper(info->aEdges[0].edge2[0]);

		generalizeEdgeAnnotation(&(info->aEdges[0].edge1[0]));
		generalizeEdgeAnnotation(&(info->aEdges[0].edge2[0]));

		info->edgeCount = 1;

		if(strcmp(temp, "cis") == 0)
			info->glycosidicOrientation = 'c';
		else
			info->glycosidicOrientation = 't';
		return 1;
	}
	return 0;
}

int hasBasePair(char line[])
{
	int i, len;
	len = strlen(line);
	for(i = 0; i < len; i++)
		if(line[i] == '/')
			return 1;
	return 0;
}
