/*
 * UtilityFunctions.c
 *
 *  Created on: May 27, 2014
 *  Author: Shahidul
 */

#include "./inc/UtilityFunctions.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

int compEdgeAtomPair(const void*A, const void*B);
int printMotifs;


//Update With iCode
void findHelices(basePairInfo basePairMapping[], int bpCount, int minHelixLen, helix helixList[MAX_HELIX_COUNT], int *helixCount, char helixOutFile[])
{
	*helixCount = 0;
	FILE *fpt = fopen(helixOutFile, "w");
	int i, j, k, countHelixLen = 0, helixEndFlag = 0, startHelix;
	basePairInfo prevBP = basePairMapping[0];
	for(i = 0; i < bpCount; i++)
	{
		if(isCanonicalBP(basePairMapping[i]))
		{
			if(countHelixLen > 0 && basePairMapping[i].residueID[0] == prevBP.residueID[0] + 1 && basePairMapping[i].residueID[1] == prevBP.residueID[1] - 1)
			{
				countHelixLen++;
				helixList[*helixCount].basePairIndex[0] = startHelix;
				helixList[*helixCount].basePairIndex[1]= startHelix + countHelixLen - 1;
			}
			else
			{
				if(countHelixLen >= minHelixLen)
					helixEndFlag = 1;
				countHelixLen = 1;
				startHelix = i;
			}
			prevBP = basePairMapping[i];
		}
		if(i == bpCount - 1 && countHelixLen >= minHelixLen)
			helixEndFlag = 1;
		if(helixEndFlag)
		{
			fprintf(fpt, "Helix Length = %d\n", helixList[*helixCount].basePairIndex[1] - helixList[*helixCount].basePairIndex[0] + 1);
			for(j = helixList[*helixCount].basePairIndex[0], k = helixList[*helixCount].basePairIndex[0]; j <= helixList[*helixCount].basePairIndex[1];)
			{
				if(!isCanonicalBP(basePairMapping[k]))
				{
					k++;
					continue;
				}
				fprintf(fpt, "[%4d] ", k);
				printPairMCAnnotateFormat(fpt, basePairMapping[k], 0);
				j++;
				k++;
			}
			(*helixCount)++;
			helixEndFlag = 0;
		}
	}
	fprintf(fpt, "\n");
	fprintf(fpt, "Total BP = %d\n", bpCount);
	fprintf(fpt, "Total %d length helix = %d\n", minHelixLen, *helixCount);
	if(fpt)
		fclose(fpt);
}

//Given the pair of interacting atoms in in the base-pair of one PDB, find correspoding atoms in the base-pair of other PDB
void getCorrespondingDistance(edgeAtomCoordinates edgeAtoms[], int startIndex, int n, edgeAtomPair aEdgeAtomPairRef[], edgeAtomPair aEdgeAtomPair[])
{
	int i, j, flag = 0, nextIndex = (startIndex + 1) % 2;
	coOrdinate3d coordinate[2];
	edgeAtomName *edgeAtomNames[2];
	edgeAtomNames[startIndex] = edgeAtoms[startIndex].edgeAtomNamesRef;
	edgeAtomNames[nextIndex] = edgeAtoms[nextIndex].edgeAtomNamesRef;
	for(i = 0; i < n; i++)
	{
		aEdgeAtomPair[i].distance = INF_DOUBLE;
		//If the donor atom is in the 1st base of the pair
		if(aEdgeAtomPairRef[i].donorIndex == 0)
		{
			flag = 0;
			//Find the corresponding donor atom coordinate in actual base-pair, given atoms in the proposed base-pair
			for(j = 0; j < edgeAtomNames[startIndex]->donorCount; j++)
			{
				if(strcmp(aEdgeAtomPairRef[i].donorAtomName, edgeAtomNames[startIndex]->donorAtomNames[j]) == 0)
				{
					strcpy(aEdgeAtomPair[i].donorAtomName, aEdgeAtomPairRef[i].donorAtomName);
					aEdgeAtomPair[i].donorIndex = startIndex;
					coordinate[startIndex] = edgeAtoms[startIndex].donorAtomsInfo[j]->coOrd;
					flag = 1;
					break;
				}
			}
			if(flag)
			{
				flag = 0;
				//Find the corresponding acceptor atom coordinate in actual base-pair, given atoms in the proposed base-pair
				for(j = 0; j < edgeAtomNames[nextIndex]->acceptorCount; j++)
				{
					if(strcmp(aEdgeAtomPairRef[i].acceptorAtomName, edgeAtomNames[nextIndex]->acceptorAtomNames[j]) == 0)
					{
						strcpy(aEdgeAtomPair[i].acceptorAtomName, aEdgeAtomPairRef[i].acceptorAtomName);
						aEdgeAtomPair[i].acceptorIndex = nextIndex;
						coordinate[nextIndex] = edgeAtoms[nextIndex].acceptorAtomsInfo[j]->coOrd;
						flag = 1;
						break;
					}
				}
			}
		}
		//If the donor atom is in the 2nd base of the pair
		else
		{
			flag = 0;
			for(j = 0; j < edgeAtomNames[nextIndex]->donorCount; j++)
			{
				if(strcmp(aEdgeAtomPairRef[i].donorAtomName, edgeAtomNames[nextIndex]->donorAtomNames[j]) == 0)
				{
					strcpy(aEdgeAtomPair[i].donorAtomName, aEdgeAtomPairRef[i].donorAtomName);
					aEdgeAtomPair[i].donorIndex = nextIndex;
					coordinate[nextIndex] = edgeAtoms[nextIndex].donorAtomsInfo[j]->coOrd;
					flag = 1;
					break;
				}
			}
			if(flag)
			{
				flag = 0;
				for(j = 0; j < edgeAtomNames[startIndex]->acceptorCount; j++)
				{
					if(strcmp(aEdgeAtomPairRef[i].acceptorAtomName, edgeAtomNames[startIndex]->acceptorAtomNames[j]) == 0)
					{
						strcpy(aEdgeAtomPair[i].acceptorAtomName, aEdgeAtomPairRef[i].acceptorAtomName);
						aEdgeAtomPair[i].acceptorIndex = startIndex;
						//Error
						coordinate[startIndex] = edgeAtoms[startIndex].acceptorAtomsInfo[j]->coOrd;
						flag = 1;
						break;
					}
				}
			}
		}
		if(flag)
			aEdgeAtomPair[i].distance = distance3D(coordinate[startIndex], coordinate[nextIndex]);
	}
}

int hasSameAtom(edgeAtomPair pairA, edgeAtomPair pairB)
{
	if(pairA.donorIndex == pairB.donorIndex && strcmp(pairA.donorAtomName, pairB.donorAtomName) == 0)
		return 1;
	if(pairA.acceptorIndex == pairB.acceptorIndex && strcmp(pairA.acceptorAtomName, pairB.acceptorAtomName) == 0)
		return 1;
	return 0;
}

// Find the closest pair of donor-acceptor atoms (potentially interacting) for a given base pair
void closestNInteractions(edgeAtomCoordinates edgeAtoms[], int *n, edgeAtomPair aEdgeAtomPair[])
{
	int i, j, k, count = 0;
	edgeAtomPair pairs[32];
	edgeAtomName *edgeAtomNames[2];
	edgeAtomNames[0] = edgeAtoms[0].edgeAtomNamesRef;
	edgeAtomNames[1] = edgeAtoms[1].edgeAtomNamesRef;

	//"donorIndex" will store if the atom of the 1st or 2nd base is donating electron
	//"acceptorIndex" will store if the atom of the 1st or 2nd base is accepting  electron

	//Get distance between all donor from 1st base and all acceptor from the 2nd base
	for(i = 0; i < edgeAtomNames[0]->donorCount; i++)
		for(j = 0; j < edgeAtomNames[1]->acceptorCount; j++)
		{
			strcpy(pairs[count].donorAtomName, edgeAtomNames[0]->donorAtomNames[i]);
			pairs[count].donorIndex = 0;
			strcpy(pairs[count].acceptorAtomName, edgeAtomNames[1]->acceptorAtomNames[j]);
			pairs[count].acceptorIndex = 1;
			pairs[count].distance = distance3D(edgeAtoms[0].donorAtomsInfo[i]->coOrd, edgeAtoms[1].acceptorAtomsInfo[j]->coOrd);
			count++;
		}

	//Get distance between all donor from 2nd base and all acceptor from the 1st base
	for(i = 0; i < edgeAtomNames[1]->donorCount; i++)
		for(j = 0; j < edgeAtomNames[0]->acceptorCount; j++)
		{
			strcpy(pairs[count].donorAtomName, edgeAtomNames[1]->donorAtomNames[i]);
			pairs[count].donorIndex = 1;
			strcpy(pairs[count].acceptorAtomName, edgeAtomNames[0]->acceptorAtomNames[j]);
			pairs[count].acceptorIndex = 0;
			pairs[count].distance = distance3D(edgeAtoms[1].donorAtomsInfo[i]->coOrd, edgeAtoms[0].acceptorAtomsInfo[j]->coOrd);
			count++;
		}

	qsort(pairs, count, sizeof(edgeAtomPair), compEdgeAtomPair);

	int flag;
	//Select atom pair with the smallest distance
	for(i = 0, j = 0; i < count && j < *n; i++)
	{
		flag = 1;
		//Make sure same atom is not participating in different chosen interactions
		for(k = 0; k < j; k++)
			if(hasSameAtom(pairs[i], aEdgeAtomPair[k]))
			{
				flag = 0;
				break;
			}
		if(flag)
			aEdgeAtomPair[j++] = pairs[i];
	}
	*n = j;
	/*for(j = 0; j < *n; j++)
		printf("%s-%s\n", aEdgeAtomPair[j].donorAtomName, aEdgeAtomPair[j].acceptorAtomName);
	getchar();*/
}

int compEdgeAtomPair(const void*A, const void*B)
{
	edgeAtomPair *x = (edgeAtomPair*)A;
	edgeAtomPair *y = (edgeAtomPair*)B;
	if(fabs(x->distance - y->distance) < ERR)
		return 0;
	else if(x->distance > y->distance)
		return 1;
	else
		return -1;
}

int compareEdges(basePairInfo x, basePairInfo y)
{
	if(
			((x.aEdges[0].edge1[0] == '+' || x.aEdges[0].edge1[0] == '-' || x.aEdges[0].edge1[0] == 'W') &&
			(y.aEdges[0].edge1[0] == '+' || y.aEdges[0].edge1[0] == '-' || y.aEdges[0].edge1[0] == 'W')) ||
			x.aEdges[0].edge1[0] == y.aEdges[0].edge1[0]
	  )
	{
		if(
			((x.aEdges[0].edge2[0] == '+' || x.aEdges[0].edge2[0] == '-' || x.aEdges[0].edge2[0] == 'W') &&
			(y.aEdges[0].edge2[0] == '+' || y.aEdges[0].edge2[0] == '-' || y.aEdges[0].edge2[0] == 'W')) ||
			x.aEdges[0].edge2[0] == y.aEdges[0].edge2[0]
		  )
		{
			if(x.glycosidicOrientation == y.glycosidicOrientation)
				return 0;
			else
				return x.glycosidicOrientation - y.glycosidicOrientation;
		}
		else
			return x.aEdges[0].edge2[0] - y.aEdges[0].edge2[0];
	}
	else
		return x.aEdges[0].edge1[0] - y.aEdges[0].edge1[0];
}

int compBasePairAndEdge(const void *a, const void *b)
{
	basePairInfo *x = (basePairInfo *)a;
	basePairInfo *y = (basePairInfo *)b;
	if(x->residue[0] == y->residue[0])
	{
		if(x->residue[1] == y->residue[1])
			return compareEdges(*x, *y);
		return x->residue[1] - y->residue[1];
	}
	return x->residue[0] - y->residue[0];
}

int compBasePairId(const void *a, const void *b)
{
	basePairInfo *x = (basePairInfo *)a;
	basePairInfo *y = (basePairInfo *)b;
	if(strcmp(x->chainResidue[0], y->chainResidue[0]) == 0)
	{
		if(strcmp(x->chainResidue[1], y->chainResidue[1]) == 0)
		{
			if(x->residueID[0] == y->residueID[0])
				if(x->iCode[0] == y->iCode[0])
					if(x->residueID[1] == y->residueID[1])
						return x->iCode[1] - y->iCode[1];
					else
						return x->residueID[1] - y->residueID[1];
				else
					return x->iCode[0] - y->iCode[0];
			else
				return x->residueID[0] - y->residueID[0];
		}
		return strcmp(x->chainResidue[1], y->chainResidue[1]);
	}
	return strcmp(x->chainResidue[0], y->chainResidue[0]);
}

//int printLimit = 10, printed = 0;

edgeAtomName getEdgeAtomName(char edge, char residue)
{
	/* X: Z (Acceptor - Donor)
	 * W: A (N1 - N6, C2),         		G (O6 - N1, N2),     C (O2, N3 - N4),     U (O2, O4 - N3)
	   H: A (N7 - N6, C8),          	G (O6, N7 - C8),     C ( - N4, C5, C6),   U (O4 - C5)
	   S: A (N3 - C2, O2', N1):O3’??,   G (N3 - O2’, N2),    C (O2 - O2’),        U (O2 - O2’)*/
	edgeAtomName aEdgeAtomName;
	aEdgeAtomName.edge = edge;
	aEdgeAtomName.residue = residue;
	//W: A (N1 - N6, C2),         		G (O6 - N1, N2),     C (O2, N3 - N4),     U (O2, O4 - N3)
	if(toupper(edge) == 'W')
	{
		if(toupper(residue) == 'A')
		{
			aEdgeAtomName.acceptorCount = 1;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "N1");
			aEdgeAtomName.donorCount = 2;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N6");
			strcpy(aEdgeAtomName.donorAtomNames[1], "C2");
		}
		else if(toupper(residue) == 'G')
		{
			aEdgeAtomName.acceptorCount = 1;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "O6");
			aEdgeAtomName.donorCount = 2;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N1");
			strcpy(aEdgeAtomName.donorAtomNames[1], "N2");
		}
		else if(toupper(residue) == 'C')
		{
			aEdgeAtomName.acceptorCount = 2;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "O2");
			strcpy(aEdgeAtomName.acceptorAtomNames[1], "N3");
			aEdgeAtomName.donorCount = 1;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N4");
		}
		else if(toupper(residue) == 'U')
		{
			aEdgeAtomName.acceptorCount = 2;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "O2");
			strcpy(aEdgeAtomName.acceptorAtomNames[1], "O4");
			aEdgeAtomName.donorCount = 1;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N3");
		}
		else
		{
			aEdgeAtomName.acceptorCount = 0;
			aEdgeAtomName.donorCount = 0;
			printf("!!!Error-\'%c\' as residue with edge %c!!!\n", residue, edge);
		}
	}
	//H: A (N7 - N6, C8),          	G (O6, N7 - C8),     C ( - N4, C5, C6),   U (O4 - C5)
	else if(toupper(edge) == 'H')
	{
		if(toupper(residue) == 'A')
		{
			aEdgeAtomName.acceptorCount = 1;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "N7");
			aEdgeAtomName.donorCount = 2;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N6");
			strcpy(aEdgeAtomName.donorAtomNames[1], "C8");
		}
		else if(toupper(residue) == 'G')
		{
			aEdgeAtomName.acceptorCount = 2;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "O6");
			strcpy(aEdgeAtomName.acceptorAtomNames[1], "N7");
			aEdgeAtomName.donorCount = 1;
			strcpy(aEdgeAtomName.donorAtomNames[0], "C8");
		}
		else if(toupper(residue) == 'C')
		{
			aEdgeAtomName.acceptorCount = 0;
			aEdgeAtomName.donorCount = 3;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N4");
			strcpy(aEdgeAtomName.donorAtomNames[1], "C5");
			strcpy(aEdgeAtomName.donorAtomNames[2], "C6");
		}
		else if(toupper(residue) == 'U')
		{
			aEdgeAtomName.acceptorCount = 1;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "O4");
			aEdgeAtomName.donorCount = 1;
			strcpy(aEdgeAtomName.donorAtomNames[0], "C5");
		}
		else
		{
			aEdgeAtomName.acceptorCount = 0;
			aEdgeAtomName.donorCount = 0;
			printf("!!!Error-\'%c\' as residue with edge %c!!!\n", residue, edge);
		}
	}
	//S: A (N3 - C2, O2', N1):O3’??,   G (N3 - O2’, N2),    C (O2 - O2’),        U (O2 - O2’) ???---(O3')---???
	else if(toupper(edge) == 'S')
	{
		if(toupper(residue) == 'A')
		{
			aEdgeAtomName.acceptorCount = 2;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "N3");
			strcpy(aEdgeAtomName.acceptorAtomNames[1], "N1");
			aEdgeAtomName.donorCount = 2;
			strcpy(aEdgeAtomName.donorAtomNames[0], "C2");
			strcpy(aEdgeAtomName.donorAtomNames[1], "O2\'");

		}
		else if(toupper(residue) == 'G')
		{
			aEdgeAtomName.acceptorCount = 1;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "N3");
			aEdgeAtomName.donorCount = 2;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N2");
			strcpy(aEdgeAtomName.donorAtomNames[1], "O2\'");
		}
		else if(toupper(residue) == 'C')
		{
			aEdgeAtomName.acceptorCount = 1;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "O2");
			aEdgeAtomName.donorCount = 1;
			strcpy(aEdgeAtomName.donorAtomNames[0], "O2\'");
		}
		else if(toupper(residue) == 'U')
		{
			aEdgeAtomName.acceptorCount = 1;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "O2");
			aEdgeAtomName.donorCount = 1;
			strcpy(aEdgeAtomName.donorAtomNames[0], "O2\'");
		}
		else
		{
			aEdgeAtomName.acceptorCount = 0;
			aEdgeAtomName.donorCount = 0;
			printf("!!!Error-\'%c\' as residue with edge %c!!!\n", residue, edge);
		}

	}
	else
	{
		/*if(printed < printLimit)
			printf("!!!---%c(%c)---!!!\n", edge, residue);
		printed++;
		if(printed % (printLimit * 10) == 0)
			printf("!!!---%d---!!!\n", printed);*/
		if(toupper(residue) == 'A')
		{
			aEdgeAtomName.acceptorCount = 3;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "N1");
			strcpy(aEdgeAtomName.acceptorAtomNames[1], "N3");
			strcpy(aEdgeAtomName.acceptorAtomNames[2], "N7");
			aEdgeAtomName.donorCount = 4;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N6");
			strcpy(aEdgeAtomName.donorAtomNames[1], "C2");
			strcpy(aEdgeAtomName.donorAtomNames[2], "C8");
			strcpy(aEdgeAtomName.donorAtomNames[3], "O2\'");
		}
		else if(toupper(residue) == 'G')
		{
			aEdgeAtomName.acceptorCount = 3;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "O6");
			strcpy(aEdgeAtomName.acceptorAtomNames[1], "N7");
			strcpy(aEdgeAtomName.acceptorAtomNames[2], "N3");
			aEdgeAtomName.donorCount = 4;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N1");
			strcpy(aEdgeAtomName.donorAtomNames[1], "N2");
			strcpy(aEdgeAtomName.donorAtomNames[2], "C8");
			strcpy(aEdgeAtomName.donorAtomNames[3], "O2\'");
		}
		else if(toupper(residue) == 'C')
		{
			aEdgeAtomName.acceptorCount = 2;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "O2");
			strcpy(aEdgeAtomName.acceptorAtomNames[1], "N3");
			aEdgeAtomName.donorCount = 4;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N4");
			strcpy(aEdgeAtomName.donorAtomNames[1], "C5");
			strcpy(aEdgeAtomName.donorAtomNames[2], "C6");
			strcpy(aEdgeAtomName.donorAtomNames[3], "O2\'");
		}
		else if(toupper(residue) == 'U')
		{
			aEdgeAtomName.acceptorCount = 2;
			strcpy(aEdgeAtomName.acceptorAtomNames[0], "O2");
			strcpy(aEdgeAtomName.acceptorAtomNames[1], "O4");
			aEdgeAtomName.donorCount = 3;
			strcpy(aEdgeAtomName.donorAtomNames[0], "N3");
			strcpy(aEdgeAtomName.donorAtomNames[1], "C5");
			strcpy(aEdgeAtomName.donorAtomNames[2], "O2\'");
		}
		else
		{
			aEdgeAtomName.acceptorCount = 0;
			aEdgeAtomName.donorCount = 0;
			printf("!!!Error-\'%c\' as residue with edge %c!!!\n", residue, edge);
		}
	}
	return aEdgeAtomName;
}

double distance3D(coOrdinate3d coordA, coOrdinate3d coordB)
{
	double dist = (coordA.x - coordB.x) * (coordA.x - coordB.x) + (coordA.y - coordB.y) * (coordA.y - coordB.y) + (coordA.z - coordB.z) * (coordA.z - coordB.z);
	return sqrt(dist);
}

int isPurine(char c)
{
	if(c == 'A' || c == 'G')
		return 1;
	return 0;
}

int isPyrimidine(char c)
{
	if(c == 'C' || c == 'U')
		return 1;
	return 0;
}

int is_whitespace(char c)
{
    if(c == ' ' || c == '\n' || c == '\t' || c == '\r')
        return 1;
    return 0;
}

int isWobbleBP(char residue1, char residue2)
{
	if(residue1 == 'G' && residue2 == 'U')
		return 1;
	//Add more combination
	return 0;
}

//Decision is based on order of nucleotide. If two orientation in necessary then, call twice.
int isCanoNucleoCombination(char residue1, char residue2)
{
	if(isWobbleBP(residue1, residue2))
		return 1;
	if((residue1 == 'A' && residue2 == 'U') || (residue1 == 'C' && residue2 == 'G'))
		return 1;
	return 0;
}

int isCanonicalBP(basePairInfo aInfo)
{
	char edge1 = aInfo.aEdges[0].edge1[0], edge2 = aInfo.aEdges[0].edge2[0];
	if((edge1 == 'W' && edge2 == 'W') || (edge1 == '+' && edge2 == '+') || (edge1 == '-' && edge2 == '-'))
	{
		if(isCanoNucleoCombination(aInfo.residue[0], aInfo.residue[1]) || isCanoNucleoCombination(aInfo.residue[1], aInfo.residue[0]))
			return 1;
	}
	return 0;
}

int isWHS_BP(basePairInfo aInfo)
{
	char edge1 = aInfo.aEdges[0].edge1[0], edge2 = aInfo.aEdges[0].edge2[0];
	if((edge1 == 'W' || edge1 == 'H' || edge1 == 'S' || edge1 == '+' || edge1 == '-') &&
		(edge2 == 'W' || edge2 == 'H' || edge2 == 'S' || edge2 == '+' || edge2 == '-'))
		return 1;
	return 0;
}

void printPairInfoOnScreen(basePairInfo info)
{
	printf(" \'%s\'%d", info.chainResidue[0], info.residueID[0]);
	if(info.iCode[0] != NO_ICODE)
		printf(".%c", info.iCode[0]);
	printf("-\'%s\'%d", info.chainResidue[1], info.residueID[1]);
	if(info.iCode[1] != NO_ICODE)
		printf(".%c", info.iCode[1]);
	printf(" %c-%c", info.residue[0], info.residue[1]);
	printf(" %c/%c", info.aEdges[0].edge1[0], info.aEdges[0].edge2[0]);
	if(info.glycosidicOrientation == 'c')
		printf(" cis");
	else if(info.glycosidicOrientation == 't')
		printf(" trans");
	printf(" %s\n", info.MC_SymNo);
}

void printPairMCAnnotateOnScreen(basePairInfo info)
{
	printf("\'%s\'%d", info.chainResidue[0], info.residueID[0]);
	if(info.iCode[0] != NO_ICODE)
		printf(".%c", info.iCode[0]);
	printf("-\'%s\'%d", info.chainResidue[1], info.residueID[1]);
	if(info.iCode[1] != NO_ICODE)
		printf(".%c", info.iCode[1]);
	printf(" : %c-%c", info.residue[0], info.residue[1]);

	int i;
	for(i = 0; i < info.edgeCount; i++)
		printf(" %s/%s", info.aEdges[i].edge1, info.aEdges[i].edge2);

	//if(strlen(info.adjacent) > 0)
		//printf(" %s", info.adjacent);
	if(strlen(info.stackDirection) > 0)
		printf(" %s", info.stackDirection);
	printf(" pairing");
	if(info.isParallel == 1)
		printf(" parallel");

	else if(info.isParallel == 1)
		printf("antiparallel");
	if(info.glycosidicOrientation == 'c')
		printf(" cis");
	else if(info.glycosidicOrientation == 't')
		printf(" trans");

	if(strlen(info.MC_SymNo) > 0)
		printf(" %s", info.MC_SymNo);
	printf("\n");
}

void printPairInfo(FILE* fpt, basePairInfo info, int skipNewLine)
{
	fprintf(fpt, " \'%s\'%d", info.chainResidue[0], info.residueID[0]);
	if(info.iCode[0] != NO_ICODE)
		fprintf(fpt, ".%c", info.iCode[0]);
	fprintf(fpt, "-\'%s\'%d", info.chainResidue[1], info.residueID[1]);
	if(info.iCode[1] != NO_ICODE)
		fprintf(fpt, ".%c", info.iCode[1]);
	fprintf(fpt, " %c-%c", info.residue[0], info.residue[1]);
	fprintf(fpt, " %c/%c", info.aEdges[0].edge1[0], info.aEdges[0].edge2[0]);
	if(info.glycosidicOrientation == 'c')
		fprintf(fpt," cis");
	else if(info.glycosidicOrientation == 't')
		fprintf(fpt," trans");
	fprintf(fpt," %s", info.MC_SymNo);
	
	if(!skipNewLine)
		fprintf(fpt,"\n");
}

void printPairRCAFormat(FILE* fpt, basePairInfo info, int skipNewLine)
{
	if(isalpha(info.chainResidue[0][0]))
		fprintf(fpt, "%s%d", info.chainResidue[0], info.residueID[0]);
	else
		fprintf(fpt, "\'%s\'%d", info.chainResidue[0], info.residueID[0]);
	if(info.iCode[0] != NO_ICODE)
		fprintf(fpt, ".%c", info.iCode[0]);
	if(isalpha(info.chainResidue[0][0]))
		fprintf(fpt, "-%s%d", info.chainResidue[1], info.residueID[1]);
	else
		fprintf(fpt, "-\'%s\'%d", info.chainResidue[1], info.residueID[1]);
	if(info.iCode[1] != NO_ICODE)
		fprintf(fpt, ".%c", info.iCode[1]);
	fprintf(fpt, " : %c-%c", info.residue[0], info.residue[1]);

	int i;
	for(i = 0; i < info.edgeCount; i++)
		fprintf(fpt, " %s/%s", info.aEdges[i].edge1, info.aEdges[i].edge2);

	if(info.glycosidicOrientation == 'c')
		fprintf(fpt," cis");
	else if(info.glycosidicOrientation == 't')
		fprintf(fpt," trans");
	
	if(!skipNewLine)
		fprintf(fpt,"\n");
}

void printPairMCAnnotateFormat(FILE* fpt, basePairInfo info, int skipNewLine)
{
	if(isalpha(info.chainResidue[0][0]))
		fprintf(fpt, "%s%d", info.chainResidue[0], info.residueID[0]);
	else
		fprintf(fpt, "\'%s\'%d", info.chainResidue[0], info.residueID[0]);
	if(info.iCode[0] != NO_ICODE)
		fprintf(fpt, ".%c", info.iCode[0]);
	if(isalpha(info.chainResidue[0][0]))
		fprintf(fpt, "-%s%d", info.chainResidue[1], info.residueID[1]);
	else
		fprintf(fpt, "-\'%s\'%d", info.chainResidue[1], info.residueID[1]);
	if(info.iCode[1] != NO_ICODE)
		fprintf(fpt, ".%c", info.iCode[1]);
	fprintf(fpt, " : %c-%c", info.residue[0], info.residue[1]);

	int i;
	for(i = 0; i < info.edgeCount; i++)
		fprintf(fpt, " %s/%s", info.aEdges[i].edge1, info.aEdges[i].edge2);

	//if(strlen(info.adjacent) > 0)
		//fprintf(fpt, " %s", info.adjacent);
	if(strlen(info.stackDirection) > 0)
		fprintf(fpt, " %s", info.stackDirection);
	fprintf(fpt, " pairing");
	if(info.isParallel == 1)
		fprintf(fpt," parallel");
	else if(info.isParallel == 0)
		fprintf(fpt," antiparallel");

	if(info.glycosidicOrientation == 'c')
		fprintf(fpt," cis");
	else if(info.glycosidicOrientation == 't')
		fprintf(fpt," trans");

	if(strlen(info.MC_SymNo) > 0)
		fprintf(fpt," %s", info.MC_SymNo);
	
	if(!skipNewLine)
		fprintf(fpt,"\n");

}

void convertToRNAViewEdgeFormat(basePairInfo info, char *edge1, char *edge2)
{
	if(*edge1 == 'W' && *edge2 == 'W')
	{
		if((info.residue[0] == 'C' && info.residue[1] == 'G') || (info.residue[0] == 'G' && info.residue[1] == 'C'))
			*edge1 = *edge2 = '+';
		if((info.residue[0] == 'A' && info.residue[1] == 'U') || (info.residue[0] == 'U' && info.residue[1] == 'A'))
			*edge1 = *edge2 = '-';
	}
}

void printPairRNAViewFormat(FILE* fpt, basePairInfo info, int skipNewLine)
{
	//Incomplete
	char temp[10];
	sprintf(temp, "%d_%d", info.residueID[0], info.residueID[1]);
	fprintf(fpt, "%9s, ", temp);
	fprintf(fpt, "%s: %5d", info.chainResidue[0], info.residueID[0]);
	if(info.iCode[0] != NO_ICODE)
		fprintf(fpt, ".%c", info.iCode[0]);
	fprintf(fpt, " %c-%c", info.residue[0], info.residue[1]);
	fprintf(fpt, " %5d", info.residueID[1]);
	if(info.iCode[1] != NO_ICODE)
		fprintf(fpt, ".%c", info.iCode[1]);
	fprintf(fpt, " %s:", info.chainResidue[0]);


	char edge1, edge2;
	edge1 = info.aEdges[0].edge1[0];
	edge2 = info.aEdges[0].edge2[0];
	fprintf(fpt, " %c/%c", edge1, edge2);

	convertToRNAViewEdgeFormat(info, &edge1, &edge2);

	if(info.glycosidicOrientation == 'c')
		fprintf(fpt," %-11s", "cis");
	else if(info.glycosidicOrientation == 't')
		fprintf(fpt," %-11s", "tran");
	fprintf(fpt," %s", info.MC_SymNo);
	
	if(!skipNewLine)
		fprintf(fpt,"\n");
}

void printPairRNAViewStack(FILE* fpt, basePairInfo info)
{
	//Incomplete
	char temp[10];
	sprintf(temp, "%d_%d", info.residueID[0], info.residueID[1]);
	fprintf(fpt, "%9s, ", temp);
	fprintf(fpt, "%s: %5d", info.chainResidue[0], info.residueID[0]);
	if(info.iCode[0] != NO_ICODE)
		fprintf(fpt, ".%c", info.iCode[0]);
	fprintf(fpt, " %c-%c", info.residue[0], info.residue[1]);
	fprintf(fpt, " %5d", info.residueID[1]);
	if(info.iCode[1] != NO_ICODE)
		fprintf(fpt, ".%c", info.iCode[1]);
	fprintf(fpt, " %s:", info.chainResidue[0]);
	fprintf(fpt, " stacked\n");
}

void printPairSimpleFormat(FILE* fpt, basePairInfo info, int skipNewLine)
{
	//if(!isCanonicalBP(info) && !isWHS_BP(info))
		//return;
	fprintf(fpt, "%s:%d-", info.chainResidue[0], info.residueID[0]);
	fprintf(fpt, "%s:%d", info.chainResidue[1], info.residueID[1]);
	
	if(!skipNewLine)
		fprintf(fpt,"\n");
}

int isDefaultDelim(char ch)
{
	if(ch == '\0' || ch == '\n' || ch == '\r')
		return 1;
	return 0;
}

int isDelim(char ch, char delim[], int delimLen)
{
	int i;
	//Default delimiter
	if(ch == '\0' || ch == '\n' || ch == '\r')
		return 1;
	for(i = 0; i < delimLen; i++)
		if(delim[i] == ch)
			return 1;
	return 0;
}

void myTokenize(char str[], char delim[], char token[][200], int *tokenCount, int ignoreExtraSpace)
{
	int i = 0, k = 0, spaceStarted = 0;
	int len = strlen(str);
	int delimLen = strlen(delim);
	*tokenCount = 0;
	for(i = 0; i <= len; i++)
	{
		if(ignoreExtraSpace)
		{
			if(spaceStarted && str[i] == ' ')
				continue;
			if(str[i] == ' ')
				spaceStarted = 1;
			else
				spaceStarted = 0;
		}
		if(isDelim(str[i], delim, delimLen))
		{
			token[*tokenCount][k] = '\0';
			(*tokenCount)++;
			k = 0;
			if(isDefaultDelim(str[i]))
				break;
		}
		else
			token[*tokenCount][k++] = str[i];

	}
}

void myTokenizeDynamic(char str[], char delim[], char **token, int *tokenCount, int ignoreExtraSpace)
{
	int i = 0, k = 0, spaceStarted = 0;
	int len = strlen(str);
	int delimLen = strlen(delim);
	*tokenCount = 0;
	for(i = 0; i <= len; i++)
	{
		if(ignoreExtraSpace)
		{
			if(spaceStarted && (str[i] == ' ' || str[i] == '\t'))
				continue;
			if(str[i] == ' ' || str[i] == '\t')
				spaceStarted = 1;
			else
				spaceStarted = 0;
		}
		if(isDelim(str[i], delim, delimLen))
		{
			if(k > 0)
			{
                token[*tokenCount][k] = '\0';
                (*tokenCount)++;
                k = 0;
            }
			if(isDefaultDelim(str[i]))
				break;
		}
		else
			token[*tokenCount][k++] = str[i];
	}
}


void generateMotifFileName(motifIndices aMotif, char fileName[MAX_FILE_SZ], char pathCW[])
{
	int i, len;
	strcpy(fileName, pathCW);
	strcat(fileName, "Motif_Segments/");
	strcat(fileName, aMotif.PDB);
	strcat(fileName, "_");

	len = strlen(fileName);
	strcpy(&fileName[len], aMotif.chainId);
	strcat(fileName, "_");

	strcat(fileName, aMotif.name);
	strcat(fileName, "_");

	char temp[20];
	for(i = 0; i < aMotif.numberOfSegments; i++)
	{
		sprintf(temp, "%d", aMotif.pairs[i].a);
		strcat(fileName, temp);
		strcat(fileName, "-");
		sprintf(temp, "%d", aMotif.pairs[i].b);
		strcat(fileName, temp);
		strcat(fileName, "_");
	}
	strcat(fileName, "MotifAnnotations.txt");
}

void createAllMotifFile(char pathCW[], motifIndices motifs[], int motifCount)
{
	int i;
	char fileName[MAX_FILE_SZ];
	for(i = 0; i < motifCount; i++)
	{
		generateMotifFileName(motifs[i], fileName, pathCW);
		FILE *fptMotifSegments = fopen(fileName, "w");
		if(!fptMotifSegments)
		{
			printf("%s File Not Created\n", fileName);
			printMotifs = 0;
		}
		else
			fclose(fptMotifSegments);
	}
}

void motifInitialization(motifIndices motifs[], int *motifCount, char pathCW[])
{
	int i;
	*motifCount = 15;
	motifIndices temp[MAX_MOTIF_FREQ] =
	{
		//All Count - 15
		{"2B9N", "A", "Kink-Turn-6", 3, {{25,28},{582,582},{512,515}}}
		,{"2B9N", "A", "Kink-Turn-1", 2, {{81,85},{97,105}}}
		,{"2B9N", "A", "Kink-Turn-3", 2, {{1234,1238},{1208,1215}}}
		,{"2B9N", "A", "Kink-Turn-12", 3, {{1001,1001},{977,977},{1154,1154}}}

		,{"2B9N", "A", "Sarcin-Ricin-3", 2, {{240,244},{254,257}}}
		,{"2B9N", "A", "Sarcin-Ricin-4", 2, {{203,207},{189,192}}}
		,{"2B9N", "A", "Sarcin-Ricin-5", 2, {{456,460},{469,472}}}
		,{"2B9N", "A", "Sarcin-Ricin-2", 2, {{2653,2657},{2664,2667}}}
		,{"2B9N", "A", "Sarcin-Ricin-1", 2, {{1266,1268},{2012,2013}}}

		,{"2B9N", "A", "C-Loop-1", 2, {{908,913},{863,869}}}
		,{"2B9N", "A", "C-Loop-2", 2, {{1330,1333},{1319,1324}}}
		,{"2B9N", "A", "C-Loop-3", 2, {{2724,2728},{2679,2685}}}


		,{"2B9N", "A", "Reverse-Kturn-1", 2, {{1425,1427},{1571,1573}}}
		,{"2B9N", "A", "Reverse-Kturn-4", 3, {{1124,1127},{2488,2488},{1028,1030}}}
		,{"2B9N", "A", "Reverse-Kturn-6", 2, {{239,243},{256,258}}}


		//NO FR3D Data Available
		,{"3J2L", "1", "Kink-Turn-2", 2, {{979,982},{1066,1072}}}
		,{"3J2L", "1", "Kink-Turn-7", 2, {{1640,1646},{1656,1662}}}
		,{"3J2L", "1", "Kink-Turn-1", 2, {{83,85},{97,102}}}
		,{"3J2L", "1", "Kink-Turn-6", 3, {{25,28}, {673, 673}, {549,552}}}
		,{"3J2L", "1", "Kink-Turn-9", 3, {{3005,3007},{1888,1889},{2995,2997}}}


	};
	for(i = 0; i < *motifCount; i++)
	{
		motifs[i]= temp[i];
		/*for(j = 0; j < motifs[i].numberOfSegments; j++)
			printf("%d-%d\n", motifs[i].pairs[j].a, motifs[i].pairs[j].b);
		getchar();*/
	}
	printMotifs = 1;
	createAllMotifFile(pathCW, motifs, *motifCount);
}

void extractMotifInfo(motifIndices motifs[], int motifCount, char PDB[], char method[], basePairInfo basePairMap[MAX_BASE_PAIR], int basePairCount, char pathCW[])
{
	if(!printMotifs)
		return;

	char fileName[MAX_FILE_SZ];
	int i, j, k, l, flag[2];
	int printOnScreen = 0;
	for(i = 0; i < motifCount; i++)
	{
		if(strcmp(PDB, motifs[i].PDB) == 0)
		{
			generateMotifFileName(motifs[i], fileName, pathCW);
			FILE *fpt = fopen(fileName, "a+");
			if(!fpt)
			{
				printf("%s File Not Created\n", fileName);
				return;
			}

			fprintf(fpt, "%s - %s\n", PDB, method);
			if(printOnScreen)
				printf("%s - %s\n", PDB, method);
			int motifBpCount = 0;
			for(j = 0; j < basePairCount; j++)
			{
				if(strcmp(basePairMap[j].chainResidue[0], motifs[i].chainId) == 0 && strcmp(basePairMap[j].chainResidue[1], motifs[i].chainId) == 0)
				{
					flag[0] = flag[1] = 0;
					for(k = 0; k < 2; k++)
					{
						for(l = 0; l < motifs[i].numberOfSegments; l++)
						{
							if(basePairMap[j].residueID[k] >= motifs[i].pairs[l].a && basePairMap[j].residueID[k] <= motifs[i].pairs[l].b)
							{
								//printf("%d %d - %d - %d\n", basePairMap[j].residueID[k], motifs[i].pairs[l].a, basePairMap[j].residueID[k], motifs[i].pairs[l].b);
								//getchar();
								flag[k] = 1;
								break;
							}

						}
					}

					if(flag[0] && flag[1])
					{
						if(method[0] == 'M')
							printPairMCAnnotateFormat(fpt, basePairMap[j], 0);
						else
							printPairRNAViewFormat(fpt, basePairMap[j], 0);
						if(printOnScreen)
						{
							printPairInfoOnScreen(basePairMap[j]);
							getchar();
						}
						motifBpCount++;
					}
				}
			}
			fprintf(fpt, "%s\t", motifs[i].name);
			for(j = 0; j < motifs[i].numberOfSegments; j++)
			{
				if(j > 0)
					fprintf(fpt, "_");
				fprintf(fpt, "%d-%d", motifs[i].pairs[j].a, motifs[i].pairs[j].b);
			}
			fprintf(fpt, " : %d\n\n", motifBpCount);
			if(fpt)
				fclose(fpt);
		}
	}
}

int compStr(const void*A, const void*B)
{
	char *x = (char*)A;
	char *y = (char*)B;
	return strcmp(x, y);
}

int mappedSeqComp(const void *a, const void *b)
{
	posMap *x = (posMap*)a;
	posMap *y = (posMap*)b;
	if(x->resPos == y->resPos)
		return x->iCode - y->iCode;
	return x->resPos - y->resPos;
}

annotationMethod getMethodId(char methodName[])
{
    if(strcmp(methodName, "MC-Annotate") == 0)
        return mcAnnotate;
    if(strcmp(methodName, "RNAView") == 0)
        return rnaView;
    if(strcmp(methodName, "FR3D") == 0)
        return fr3d;
    if(strcmp(methodName, "DSSR") == 0)
        return dssr;
    if(strcmp(methodName, "ClaRNA") == 0)
        return clarna;
    printf("Warning!!! Unknown Method in log file.\n");
    return mcAnnotate;
}

void methodPrintToFile(annotationMethod aMethod, FILE*fpt)
{
	if(testCodeLog)
	{
		if(aMethod == mcAnnotate)
			fprintf(fpt, "\nMC-Annotate\n\n");
		if(aMethod == rnaView)
			fprintf(fpt, "\nRNAView\n\n");
		if(aMethod == fr3d)
			fprintf(fpt, "\nFR3D\n\n");
		if(aMethod == dssr)
			fprintf(fpt, "\nDSSR\n\n");
		if(aMethod == clarna)
			fprintf(fpt, "\nClaRNA\n\n");
	}
}
