/*
 * WriteResultTables.c
 *
 *  Created on: July 19, 2015
 *      Author: Shahidul
 */

#include "./inc/ReadRNAViewData.h"
#include "./inc/UtilityFunctions.h"
#include "./inc/WriteResultTables.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

annotationMethod currentMethod;

void processRegularAnnotateData(FILE*fpt);
void processCompAnnotateData(FILE*fpt);
void processRegularInferabilityData(FILE*fpt);
void processCompAnnotateInferabilityData(FILE*fpt);
int writeLatexFormat(char path[]);
int writeInferableLatexFormat(char path[]);
void methodPrintToFile_NoNewLines(annotationMethod aMethod, FILE*fpt);

int processInferabilityLog(char path[], char logFile[])
{
	char fileName[MAX_FILE_SZ];
    strcpy(fileName, path);
    strcat(fileName, logFile);
    FILE *fpt = fopen(fileName, "r");
	if(!fpt)
	{
		printf("%s file not found\n", fileName);
		return 0;
	}

	char line[LINE_SIZE];

	while(fscanf(fpt, " %[^\n]", line) == 1)
    {
        if(strcmp(line, "Regular Annotate") == 0)
        	processRegularInferabilityData(fpt);
        else if(strcmp(line, "CompAnnotate") == 0)
        	processCompAnnotateInferabilityData(fpt);
    }

	writeInferableLatexFormat(path);
	if(fpt)
        fclose(fpt);
}

void writeInferTableHeader(FILE* fpt, int method, char ref[], char target[], int tableCount)
{
	static int count = 7;
	fprintf(fpt, "\\begin{table}[H]\n");
	fprintf(fpt, "\\renewcommand\\thetable{S%d}\n", count++);
	fprintf(fpt, "%%\\tableparts{%%\n");
	fprintf(fpt, "\\centering\n");
	fprintf(fpt, "\\caption{Acceptance ratios of different inferability relationship for \n");
	methodPrintToFile_NoNewLines((annotationMethod)method, fpt);
	fprintf(fpt, " annotation, using reference structure in PDB %s for target \n", ref);
	fprintf(fpt, "structure in PDB %s.}\n", target);

	fprintf(fpt, "\\label{table:BP_Inferable%d}%%\n", tableCount);
	fprintf(fpt, "%%}\n");
	fprintf(fpt, "{%%\n");
	fprintf(fpt, "\\footnotesize\n");
	fprintf(fpt, "\\centering\n");
	fprintf(fpt, "\\begin{tabular*}{0.99\\textwidth}{@{}llccc@{}}\n");
	fprintf(fpt, "\\cline{1-5}\n");
	fprintf(fpt, "Target (Ref.)      & Relationship 			& Total \\# of BP 	& Accepted \\# of BP 	& Accept \\%% \\\\\n");
	fprintf(fpt, "\\cline{1-5}\n");
}

void writeInferTableFooter(FILE* fpt)
{
	fprintf(fpt, "\\end{tabular*}\n");
	fprintf(fpt, "}\n");
	fprintf(fpt, "{\\footnotesize\n");
	fprintf(fpt, "}%%\n");
	fprintf(fpt, "\\end{table}\n\n\n");
}	

void writeInferTableLineNumbers(FILE* fpt, char inferRelation[], inferCount aCount)
{
	fprintf(fpt, " & %-25s", inferRelation);
	double percent = (100.0 * aCount.accept)/aCount.total;
	fprintf(fpt, " & %-3d & %-3d & %-0.2lf \\\\\n", aCount.total, aCount.accept, percent);
}

void writeInferTableLine(FILE* fpt, char bpType[], inferRelation aRelation)
{
	fprintf(fpt, "\\multirow{5}{*}{%s}\t", bpType);
	writeInferTableLineNumbers(fpt, "Common", aRelation.common);
	fprintf(fpt, "\t\t\t\t\t\t\t");
	writeInferTableLineNumbers(fpt, "Inferable --- Case 1", aRelation.inferRef);
	fprintf(fpt, "\t\t\t\t\t\t\t");
	writeInferTableLineNumbers(fpt, "Inferable --- Case 2", aRelation.inferTarget);
	fprintf(fpt, "\t\t\t\t\t\t\t");
	writeInferTableLineNumbers(fpt, "Non-Inferable --- Case 1", aRelation.nonInferRef);
	fprintf(fpt, "\t\t\t\t\t\t\t");
	writeInferTableLineNumbers(fpt, "Non-Inferable --- Case 2", aRelation.nonInferTarget);
	fprintf(fpt, "\\cline{1-5}\n");
}

int writeInferableLatexFormat(char path[])
{
	int i, j;
	char fileName[MAX_FILE_SZ];
	strcpy(fileName, path);
	strcat(fileName, "inferablityTablesInLatex.txt");
	FILE *fpt = fopen(fileName, "w");
	if(!fpt)
	{
		printf("%s file could not be created\n", fileName);
		return 0;
	}
	for(i = 0; i < 5; i++)
	{
		for(j = 0; j <= 0; j++)
		{
			writeInferTableHeader(fpt, i, allMethodInferStat[j].refPDB, allMethodInferStat[j].targetPDB, i*4+j+1);
			//Print Canonical
			writeInferTableLine(fpt, "Canonical", allMethodInferStat[j].cano[i]);
			//Print Non-Canonical
			writeInferTableLine(fpt, "Non-Canonical", allMethodInferStat[j].nonCano[i]);

			writeInferTableFooter(fpt);
		}
	}

	if(fpt)
		fclose(fpt);
}

void processRegularInferabilityData(FILE*fpt)
{
	int i, j, k;
	char method[LINE_SIZE], dummy[LINE_SIZE];
	double percent;
	for(i = 0; i < 5; i++)
	{
		fscanf(fpt, " %[^\n]", method);
		for(j = 0; j < 4; j++)
			for(k = 0; k < 18; k++)
				fscanf(fpt, " %[^\n]", dummy);
	}
}

void readRefTotal(FILE *fpt, inferRelation *aInfRelation)
{
	char dummy[LINE_SIZE];
	fscanf(fpt, " %[^\n=]=", dummy);
	fscanf(fpt, " %d", &aInfRelation->common.total);
	fscanf(fpt, " %[^\n=]=", dummy);
	fscanf(fpt, " %d", &aInfRelation->inferRef.total);
	fscanf(fpt, " %[^\n=]=", dummy);
	fscanf(fpt, " %d", &aInfRelation->nonInferRef.total);
}

void readTargetTotal(FILE *fpt, inferRelation *aInfRelation)
{
	char dummy[LINE_SIZE];
	fscanf(fpt, " %[^\n=]=", dummy);
	fscanf(fpt, " %[^\n=]=", dummy);
	fscanf(fpt, " %d", &aInfRelation->inferTarget.total);
	fscanf(fpt, " %[^\n=]=", dummy);
	fscanf(fpt, " %d", &aInfRelation->nonInferTarget.total);
}

void readAcceptData(FILE *fpt, int *cano, int *nonCano)
{
	char dummy[LINE_SIZE];
	fscanf(fpt, " %[^\n=]=", dummy);
	fscanf(fpt, " %d", cano);
	fscanf(fpt, " %[^\n=]=", dummy);
	fscanf(fpt, " %d", nonCano);
	fscanf(fpt, " %[^\n]", dummy);
}

void processCompAnnotateInferabilityData(FILE*fpt)
{
	int i, j, k, oth;
	char line[LINE_SIZE], method[LINE_SIZE];
	char dummy[LINE_SIZE];
	double percent;
	for(i = 0; i < 5; i++)
	{
		fscanf(fpt, " %[^\n]", method);
		for(j = 0; j < 4; j++)
		{
			//Line 1
			fscanf(fpt, " %[^ \n] vs %[^ \n]", allMethodInferStat[j].refPDB, allMethodInferStat[j].targetPDB);
			readRefTotal(fpt, &allMethodInferStat[j].cano[i]);
			//Line 2
			readRefTotal(fpt, &allMethodInferStat[j].nonCano[i]);
			fscanf(fpt, " %[^\n]", dummy);
			readTargetTotal(fpt, &allMethodInferStat[j].cano[i]);
			readTargetTotal(fpt, &allMethodInferStat[j].nonCano[i]);
			for(k = 0; k < 3; k++)
				fscanf(fpt, " %[^\n]", dummy);
			//Line 9
			readAcceptData(fpt, &(allMethodInferStat[j].cano[i].common.accept), &(allMethodInferStat[j].nonCano[i].common.accept));
			for(k = 0; k < 2; k++)
				fscanf(fpt, " %[^\n]", dummy);
			//Line 12
			readAcceptData(fpt, &(allMethodInferStat[j].cano[i].inferRef.accept), &(allMethodInferStat[j].nonCano[i].inferRef.accept));
			for(k = 0; k < 2; k++)
				fscanf(fpt, " %[^\n]", dummy);
			//Line 15
			readAcceptData(fpt, &(allMethodInferStat[j].cano[i].inferTarget.accept), &(allMethodInferStat[j].nonCano[i].inferTarget.accept));
			for(k = 0; k < 2; k++)
				fscanf(fpt, " %[^\n]", dummy);
			//Line 18
			readAcceptData(fpt, &(allMethodInferStat[j].cano[i].nonInferTarget.accept), &(allMethodInferStat[j].nonCano[i].nonInferTarget.accept));

			for(k = 0; k < 10; k++)
				fscanf(fpt, " %[^\n]", dummy);
		}
	}
}

int readLogWriteTables(char path[], char logFile[])
{
	char fileName[MAX_FILE_SZ];
    strcpy(fileName, path);
    strcat(fileName, logFile);
    FILE *fpt = fopen(fileName, "r");
	if(!fpt)
	{
		printf("%s file not found\n", fileName);
		return 0;
	}

	char line[LINE_SIZE];

	while(fscanf(fpt, " %[^\n]", line) == 1)
    {
        //printf("%s\n", line);
        //getchar();
        if(strcmp(line, "Regular Annotate") == 0)
        	processRegularAnnotateData(fpt);
        else if(strcmp(line, "CompAnnotate") == 0)
            processCompAnnotateData(fpt);
    }

	writeLatexFormat(path);
	if(fpt)
        fclose(fpt);
}

double calculateSensitivityPercent(int truePositive, int positive)
{
	return (100.0 * truePositive) / positive;
}

double calculatePrecisionPercent(int truePositive, int falsePositive)
{
	return (100.0 * truePositive) / (truePositive + falsePositive);
}

//statType -> Sensitivity, Precision
void writePrecSensTableHeaderCommon(FILE* fpt, int method, char statType[], int tableCount)
{
	static int count = 12;
	fprintf(fpt, "\\begin{table}[H]\n");
	fprintf(fpt, "\\renewcommand\\thetable{S%d}\n", count++);
	fprintf(fpt, "%%\\tableparts{%%\n");
	fprintf(fpt, "\\centering\n");

	if((annotationMethod)method != undefined)
	{
		fprintf(fpt, "\\caption{%s compared for ", statType);
		methodPrintToFile_NoNewLines((annotationMethod)method, fpt);
		fprintf(fpt, " annotation}\n");
	}
	else
	{
		fprintf(fpt, "\\caption{ The improvements in base-pairing annotation %s", statType);
		fprintf(fpt, " for five tools by using CompAnnotate}\n");
	}


	fprintf(fpt, "\\label{table:");
	methodPrintToFile_NoNewLines((annotationMethod)method, fpt);
	fprintf(fpt, "%d}%%\n", tableCount);
	fprintf(fpt, "%%}\n");
	fprintf(fpt, "{%%\n");
	fprintf(fpt, "\\small\n");
	fprintf(fpt, "\\centering\n");
	if(strcmp(statType, "Sensitivity") == 0)
		fprintf(fpt, "\\begin{tabular*}{0.99\\textwidth}{@{}llccccc@{}}\n");
	else
		fprintf(fpt, "\\begin{tabular*}{0.99\\textwidth}{@{}llccccc@{}}\n");
	fprintf(fpt, "\\cline{1-7}\n");

	fprintf(fpt, "\\multirow{3}{*}{Method} &\n");
	fprintf(fpt, "\\multirow{3}{*}{Base-pair Type} &\n");
	fprintf(fpt, "\\multirow{3}{*}{\\begin{tabular}[c]{@{}c@{}}\\# of BP \\\\ in Benchmark\n");
	fprintf(fpt, "\\end{tabular}} & \\multicolumn{2}{c}{Regular} &\n");
	fprintf(fpt, "\\multicolumn{2}{c}{CompAnnotate}\\\\\n");
	fprintf(fpt, "\\cline{4-5}\n");
	fprintf(fpt, "\\cline{6-7}\n");
}


void writeSensTableHeader(FILE* fpt, int method, int tableCount)
{
	writePrecSensTableHeaderCommon(fpt, method, "sensitivity", tableCount);
	fprintf(fpt, "& & & {\\begin{tabular}[c]{@{}c@{}}\\# of BP \\\\ Detected \\end{tabular}} & \\% & {\\begin{tabular}[c]{@{}c@{}}\\# of BP \\\\ Detected  \\end{tabular}} & \\% \\\\\n");
	fprintf(fpt, "\\cline{1-7}\n");
}

void writePrecTableHeader(FILE* fpt, int method, int tableCount)
{
	writePrecSensTableHeaderCommon(fpt, method, "precision", tableCount);
	fprintf(fpt, "& & & {\\begin{tabular}[c]{@{}c@{}}Match/\\\\\n");
	fprintf(fpt, "Conflict\\end{tabular}} & \\% & {\\begin{tabular}[c]{@{}c@{}}Match/\\\\\n");
	fprintf(fpt, "Conflict\\end{tabular}} & \\% \\\\\n");
	fprintf(fpt, "\\cline{1-7}\n");
}

void writePrecSensTableFooter(FILE* fpt)
{
	fprintf(fpt, "\\cline{1-7}\n");
	fprintf(fpt, "\\end{tabular*}\n");
	fprintf(fpt, "}\n");
	fprintf(fpt, "{\\footnotesize\n");
	fprintf(fpt, "}%%\n");
	fprintf(fpt, "\\end{table}\n\n\n");
}

void printPrecSensTableRow(FILE *fpt, char bpType[], dataType aData)
{
	fprintf(fpt, "& %-15s &", bpType);

	double regPercent, compAPercent;
	regPercent = calculateSensitivityPercent(aData.regular.match, aData.benchmark);
	compAPercent = calculateSensitivityPercent(aData.compAnnotate.match, aData.benchmark);

	fprintf(fpt, " %3d & %3d &", aData.benchmark, aData.regular.match);
	if(regPercent > compAPercent)
		fprintf(fpt, " \\textbf{%.2lf} &", regPercent);
	else
		fprintf(fpt, " %4.2lf &", regPercent);
	fprintf(fpt, " %3d &", aData.compAnnotate.match);
	if(regPercent > compAPercent)
		fprintf(fpt, " %4.2lf &", compAPercent);
	else
		fprintf(fpt, " \\textbf{%4.2lf} &", compAPercent);

	regPercent = calculatePrecisionPercent(aData.regular.match, aData.regular.misMatch);
	compAPercent = calculatePrecisionPercent(aData.compAnnotate.match, aData.compAnnotate.misMatch);

	fprintf(fpt, " %3d/%-3d &", aData.regular.match, aData.regular.misMatch);
	if(regPercent > compAPercent)
		fprintf(fpt, " \\textbf{%.2lf} &", regPercent);
	else
		fprintf(fpt, " %4.2lf &", regPercent);

	fprintf(fpt, " %3d/%-3d &", aData.compAnnotate.match, aData.compAnnotate.misMatch);
	if(regPercent > compAPercent)
		fprintf(fpt, " %4.2lf", compAPercent);
	else
		fprintf(fpt, " \\textbf{%4.2lf}", compAPercent);

	fprintf(fpt, "\\\\\n");
}

void printPrecTableRow(FILE *fpt, char bpType[], dataType aData)
{
	fprintf(fpt, "& %-15s &", bpType);

	double regPercent, compAPercent;
	regPercent = calculateSensitivityPercent(aData.regular.match, aData.benchmark);
	compAPercent = calculateSensitivityPercent(aData.compAnnotate.match, aData.benchmark);
	fprintf(fpt, " %3d &", aData.benchmark);

	regPercent = calculatePrecisionPercent(aData.regular.match, aData.regular.misMatch);
	compAPercent = calculatePrecisionPercent(aData.compAnnotate.match, aData.compAnnotate.misMatch);

	fprintf(fpt, " %3d/%-3d &", aData.regular.match, aData.regular.misMatch);
	if(regPercent > compAPercent)
		fprintf(fpt, " \\textbf{%.2lf} &", regPercent);
	else
		fprintf(fpt, " %4.2lf &", regPercent);

	fprintf(fpt, " %3d/%-3d &", aData.compAnnotate.match, aData.compAnnotate.misMatch);
	if(regPercent > compAPercent)
		fprintf(fpt, " %4.2lf", compAPercent);
	else
		fprintf(fpt, " \\textbf{%4.2lf}", compAPercent);

	fprintf(fpt, "\\\\\n");
}

void printSensTableRow(FILE *fpt, char bpType[], dataType aData)
{
	fprintf(fpt, "& %-15s &", bpType);

	double regPercent, compAPercent;
	regPercent = calculateSensitivityPercent(aData.regular.match, aData.benchmark);
	compAPercent = calculateSensitivityPercent(aData.compAnnotate.match, aData.benchmark);
	fprintf(fpt, " %3d &", aData.benchmark);

	fprintf(fpt, " %3d &", aData.regular.match);
	if(regPercent > compAPercent)
		fprintf(fpt, " \\textbf{%.2lf} &", regPercent);
	else
		fprintf(fpt, " %4.2lf &", regPercent);
	fprintf(fpt, " %3d &", aData.compAnnotate.match);
	if(regPercent > compAPercent)
		fprintf(fpt, " %4.2lf", compAPercent);
	else
		fprintf(fpt, " \\textbf{%4.2lf}", compAPercent);

	fprintf(fpt, "\\\\\n");
}

int writeTotalTables(FILE*fpt, char path[])
{
	int i, j;
	fprintf(fpt, "\n\n");
	for(i = 0; i < 5; i++)
	{
		totalStats.cano[i].benchmark = totalStats.nonCano[i].benchmark = 0;
		totalStats.cano[i].regular.match = totalStats.nonCano[i].regular.match = 0;
		totalStats.cano[i].regular.misMatch = totalStats.nonCano[i].regular.misMatch = 0;
		totalStats.cano[i].compAnnotate.match = totalStats.nonCano[i].compAnnotate.match = 0;
		totalStats.cano[i].compAnnotate.misMatch = totalStats.nonCano[i].compAnnotate.misMatch = 0;
		for(j = 0; j < 4; j++)
		{
			totalStats.cano[i].benchmark += allMethodStats[j].cano[i].benchmark;
			totalStats.nonCano[i].benchmark += allMethodStats[j].nonCano[i].benchmark;

			totalStats.cano[i].regular.match += allMethodStats[j].cano[i].regular.match;
			totalStats.nonCano[i].regular.match += allMethodStats[j].nonCano[i].regular.match;
			totalStats.cano[i].regular.misMatch += allMethodStats[j].cano[i].regular.misMatch;
			totalStats.nonCano[i].regular.misMatch += allMethodStats[j].nonCano[i].regular.misMatch;

			totalStats.cano[i].compAnnotate.match += allMethodStats[j].cano[i].compAnnotate.match;
			totalStats.nonCano[i].compAnnotate.match += allMethodStats[j].nonCano[i].compAnnotate.match;
			totalStats.cano[i].compAnnotate.misMatch += allMethodStats[j].cano[i].compAnnotate.misMatch;
			totalStats.nonCano[i].compAnnotate.misMatch += allMethodStats[j].nonCano[i].compAnnotate.misMatch;
		}
	}

	writeSensTableHeader(fpt, undefined, 1);
	for(i = 0; i < 5; i++)
	{
		//Print Canonical
		fprintf(fpt, "\\multirow{2}{*}{");
		methodPrintToFile_NoNewLines((annotationMethod)i, fpt);
		fprintf(fpt, "}\t");
		if(i > 1)
			fprintf(fpt, "\t");
		printSensTableRow(fpt, "Canonical", totalStats.cano[i]);

		//Print Non-Canonical
		//fprintf(fpt, "                               ");
		fprintf(fpt, "\t\t\t\t");

		printSensTableRow(fpt, "Non-Canonical", totalStats.nonCano[i]);
		if(i < 4)
			fprintf(fpt, "\\\\[1pt]\n");
	}
	writePrecSensTableFooter(fpt);

	writePrecTableHeader(fpt, undefined, 2);
	for(i = 0; i < 5; i++)
	{
		//Print Canonical
		fprintf(fpt, "\\multirow{2}{*}{");
		methodPrintToFile_NoNewLines((annotationMethod)i, fpt);
		fprintf(fpt, "}\t");
		if(i > 1)
			fprintf(fpt, "\t");
		printPrecTableRow(fpt, "Canonical", totalStats.cano[i]);

		//Print Non-Canonical
		//fprintf(fpt, "                               ");
		fprintf(fpt, "\t\t\t\t");

		printPrecTableRow(fpt, "Non-Canonical", totalStats.nonCano[i]);
		if(i < 4)
			fprintf(fpt, "\\\\[1pt]\n");
	}
	writePrecSensTableFooter(fpt);

	writeBarchartTableFormat(path);
}

int writeLatexFormat(char path[])
{
	int i, j;
	char fileName[MAX_FILE_SZ];
	strcpy(fileName, path);
	strcat(fileName, "tablesInLatex.txt");
	FILE *fpt = fopen(fileName, "w");
	if(!fpt)
	{
		printf("%s file could not be created\n", fileName);
		return 0;
	}

	for(i = 0; i < 5; i++)
	{
		writeSensTableHeader(fpt, i, 1);
		//methodPrintToFile((annotationMethod)i, fpt);
		for(j = 0; j < 4; j++)
		{
			//Print Canonical
			fprintf(fpt, "\\multirow{2}{*}{%s (%s)}\t", allMethodStats[j].targetPDB, allMethodStats[j].refPDB);
			printSensTableRow(fpt, "Canonical", allMethodStats[j].cano[i]);

			//Print Non-Canonical
			//fprintf(fpt, "                               ");
			fprintf(fpt, "\t\t\t\t");
			printSensTableRow(fpt, "Non-Canonical", allMethodStats[j].nonCano[i]);
			if(j < 3)
				fprintf(fpt, "\\\\[1pt]\n");
		}
		writePrecSensTableFooter(fpt);

		writePrecTableHeader(fpt, i, 2);
		//methodPrintToFile((annotationMethod)i, fpt);
		for(j = 0; j < 4; j++)
		{
			//Print Canonical
			fprintf(fpt, "\\multirow{2}{*}{%s (%s)}\t", allMethodStats[j].targetPDB, allMethodStats[j].refPDB);
			printPrecTableRow(fpt, "Canonical", allMethodStats[j].cano[i]);

			//Print Non-Canonical
			//fprintf(fpt, "                               ");
			fprintf(fpt, "\t\t\t\t");
			printPrecTableRow(fpt, "Non-Canonical", allMethodStats[j].nonCano[i]);
			if(j < 3)
				fprintf(fpt, "\\\\[1pt]\n");
		}
		writePrecSensTableFooter(fpt);
	}

	writeTotalTables(fpt, path);
	if(fpt)
		fclose(fpt);
}

void processRegularAnnotateData(FILE*fpt)
{
	int i, j;
	char method[LINE_SIZE], bpType[LINE_SIZE];
	double percent;
	for(i = 0; i < 5; i++)
	{
		fscanf(fpt, " %[^\n]", method);
		for(j = 0; j < 4; j++)
		{
			fscanf(fpt, " %[^ \n(] ( %[^ \n)] )", allMethodStats[j].targetPDB, allMethodStats[j].benchmarkPDB);
			fscanf(fpt, " %[^ \n:] : %d/%d %lf", bpType, &allMethodStats[j].cano[i].regular.match, &allMethodStats[j].cano[i].regular.misMatch, &percent);
			fscanf(fpt, " %[^ \n:] : %d/%d %lf", bpType, &allMethodStats[j].nonCano[i].regular.match, &allMethodStats[j].nonCano[i].regular.misMatch, &percent);
		}
	}
}

void processCompAnnotateData(FILE*fpt)
{
	int i, j, oth;
	char line[LINE_SIZE], method[LINE_SIZE], bpType[LINE_SIZE];
	char targetPDB[5], benchmarkPDB[5], dummy[LINE_SIZE];
	double percent;
	for(i = 0; i < 5; i++)
	{
		fscanf(fpt, " %[^\n]", method);
		for(j = 0; j < 4; j++)
		{
			fscanf(fpt, " %[^ \n(] ( %[^ \n)] )", allMethodStats[j].refPDB, targetPDB);
			fscanf(fpt, " %[^\n]", line);
			fscanf(fpt, " %[^\n]", line);
			fscanf(fpt, " %[^\n]", line);
			fscanf(fpt, " %[^ \n:] : %d, %[^ \n:] : %d, %[^ \n:] : %d", dummy, &allMethodStats[j].cano[i].benchmark, dummy, &allMethodStats[j].nonCano[i].benchmark, dummy, &oth);
			fscanf(fpt, " %[^ \n(] ( %[^ \n)] )", targetPDB, benchmarkPDB);
			if(!(strcmp(targetPDB, allMethodStats[j].targetPDB) == 0 && strcmp(benchmarkPDB, allMethodStats[j].benchmarkPDB) == 0))
			{
				printf("ERROR!! Regular and Comp-Annotate test data set ordeing is not proper.\n");
				printf("Press any key to continue.\n");
				getchar();
			}
			fscanf(fpt, " %[^ \n:] : %d/%d %lf", bpType, &allMethodStats[j].cano[i].compAnnotate.match, &allMethodStats[j].cano[i].compAnnotate.misMatch, &percent);
			fscanf(fpt, " %[^ \n:] : %d/%d %lf", bpType, &allMethodStats[j].nonCano[i].compAnnotate.match, &allMethodStats[j].nonCano[i].compAnnotate.misMatch, &percent);
		}
	}
}

void methodPrintToFile_NoNewLines(annotationMethod aMethod, FILE*fpt)
{
	if(testCodeLog)
	{
		if(aMethod == mcAnnotate)
			fprintf(fpt, "MC-Annotate");
		if(aMethod == rnaView)
			fprintf(fpt, "RNAView");
		if(aMethod == fr3d)
			fprintf(fpt, "FR3D");
		if(aMethod == dssr)
			fprintf(fpt, "DSSR");
		if(aMethod == clarna)
			fprintf(fpt, "ClaRNA");

		if(!(aMethod == mcAnnotate || aMethod == rnaView || aMethod == fr3d || aMethod == dssr || aMethod == clarna))
			fprintf(fpt, "TOT");
	}
}

int writeBarchartTableFormat(char path[])
{
	int i, j;
	char fileName[4][MAX_FILE_SZ];
	for(i = 0; i < 4; i++)
		strcpy(fileName[i], path);
	strcat(fileName[0], "PlotData_Average_Prec_Cano.csv");
	strcat(fileName[1], "PlotData_Average_Prec_NonCano.csv");
	strcat(fileName[2], "PlotData_Average_Sens_Cano.csv");
	strcat(fileName[3], "PlotData_Average_Sens_NonCano.csv");
	FILE *fpt[4];
	for(i = 0; i < 4; i++)
	{
		fpt[i] = fopen(fileName[i], "w");
		if(!fpt[i])
		{
			printf("%s file could not be created\n", fileName[i]);
			for(j = 0; j < i; j++)
				if(fpt[j])
					fclose(fpt[j]);
			return 0;
		}
	}

	for(j = 0; j < 4; j++)
		fprintf(fpt[j], "Method	Percent	Type\n");
	double regPrec, compAPrec;
	double regSens, compASens;

	for(i = 0; i < 5; i++)
	{
		for(j = 0; j < 4; j++)
			methodPrintToFile_NoNewLines((annotationMethod)i, fpt[j]);
		fprintf(fpt[0], "\t%0.2lf\t", calculatePrecisionPercent(totalStats.cano[i].regular.match, totalStats.cano[i].regular.misMatch));
		fprintf(fpt[1], "\t%0.2lf\t", calculatePrecisionPercent(totalStats.nonCano[i].regular.match, totalStats.nonCano[i].regular.misMatch));
		fprintf(fpt[2], "\t%0.2lf\t", calculateSensitivityPercent(totalStats.cano[i].regular.match, totalStats.cano[i].benchmark));
		fprintf(fpt[3], "\t%0.2lf\t", calculateSensitivityPercent(totalStats.nonCano[i].regular.match, totalStats.nonCano[i].benchmark));
		for(j = 0; j < 4; j++)
		{
			fprintf(fpt[j], "Regular\n");
			methodPrintToFile_NoNewLines((annotationMethod)i, fpt[j]);
		}
		fprintf(fpt[0], "\t%0.2lf\t", calculatePrecisionPercent(totalStats.cano[i].compAnnotate.match, totalStats.cano[i].compAnnotate.misMatch));
		fprintf(fpt[1], "\t%0.2lf\t", calculatePrecisionPercent(totalStats.nonCano[i].compAnnotate.match, totalStats.nonCano[i].compAnnotate.misMatch));
		fprintf(fpt[2], "\t%0.2lf\t", calculateSensitivityPercent(totalStats.cano[i].compAnnotate.match, totalStats.cano[i].benchmark));
		fprintf(fpt[3], "\t%0.2lf\t", calculateSensitivityPercent(totalStats.nonCano[i].compAnnotate.match, totalStats.nonCano[i].benchmark));
		for(j = 0; j < 4; j++)
			fprintf(fpt[j], "CompAnnotate\n");
	}

	for(i = 0; i < 4; i++)
		if(fpt[i])
			fclose(fpt[i]);
}
