/*
 * CommonDefinitions.h
 *
 *  Created on: Nov 11, 2013
 *      Author: Shahidul
 */

#ifndef COMMONDEFINITIONS_H_
#define COMMONDEFINITIONS_H_

#include <stdio.h>

/*Output Format
 * (Bypass MC-Annotate, RNAView format to generate
 * Simple format output for ARTS, SARA)*/
#define SIMPLE_OUTPUT_FORMAT 0
#define PRINT_BASE_MAPPING_ERRORS 0
int PRINT_ORIGINAL_SIMPLE;

//RNAView/FR3D
//#define IS_FR3D 1

//Align Parameter
#define MAX_ALIGN_PDB_COUNT 2
#define NO_ALIGN_INDEX -1

//PDB Data Max Limit
#define MAX_ATOM_LIST MAX_ALIGN_PDB_COUNT
#define MAX_ATOM_COUNT 100000//70000//120000
#define MAX_SEQ_LEN 7000//6000//7000
#define MAX_BASE_PAIR 10000//30000
#define MAX_HELIX_COUNT 5000//1000//5000
#define MAX_CHAIN_NAME_LENGTH 4

//Motif Related Max Limit
#define MAX_CHAIN_COUNT 1//20
#define MAX_MOTIF_FREQ 100
#define MAX_SEGMENT_NUMBER 10
#define MAX_MOTIF_SEGMENT_LENGTH 100
#define TOP_X_NUM 15

//File Processing Limits
#define MAX_PDB_FILES 2
#define MAX_FILE_SZ 300
#define LINE_SIZE 200
#define PART_SIZE 100
#define BP_FEATURE_NAME_SZ 20
#define PDB_ID_SZ 6
#define TARGET_ANGLES_COUNT 100

//File Parse Data Limit
#define TITLE_COUNT 6
#define ATOM_NAME_SZ 6
#define EDGE_NAME_SIZE 5
#define DONOR_ACCEPTOR_COUNT 4
#define EDGE_COUNT_LIMIT 3

//Geometric Claculation Constants
#define ANGLE_PRECISION 0//Update DIFFERENT_ANGLES also with it
#define DIFFERENT_ANGLES (180 * 1)//180*10^ANGLE_PRECISION
#define ERR 0.0000000001
#define INF_DOUBLE 1.0e9

//Forest Data Limits
#define MAX_NODES_IN_CONFUSION_FOREST 3000
#define MAX_ADJACENCY 40

//Data Filtering Options
#define ONLY_TARGET_CHAIN 1
//#define getAllBasePair 1
#define FILTER_NON_WHS_BONDS 0
#define FILTER_CIS_TRANS_UNSPECIFIED_BONDS 0
#define FILTER_ONEBOND 0
#define FILTER_UNWANTED_BONDS_RNA_VIEW FILTER_ONEBOND
#define FILTER_NON_CANNONICAL 0
#define TEST_PRINT_COUNT 0
#define PRINT_ALIGN_BP_INFO 0
#define PRINT_SMALL_FINAL_INFO 0

//String Constants

// Triangulation atoms
#define C_ALPHA "C1\'"
#define C2 "C2"
#define C5 "C5"

#define PHOSPHORUS "P"
#define NO_ICODE ' '
#define TARGET_ATOM C_ALPHA


#define myFabs(a) (a < 0.0 ? -(a):(a))
#define swap(a, b) a ^= b ^= a ^= b
#define min(a,b) (a < b ? a : b)
#define max(a, b) (a>b?a:b)


//Change
int COMPARE_EDGE;
FILE *testCodeLog, *testCodeLog2;
int testCodeFlag, testCodeFlag2;
int writeCompResult;
int fastaFileFound;
int COMP_EDGE_ONLY;
int doScoreOnlyAlign;

//3D Coordinate info
typedef struct
{
	double x, y, z;
}coOrdinate3d;

//ClustalW Structures
typedef struct
{
	char residue;
	int isMatch;
	int resPos;
	char iCode;
	int seq2MappedPos; // Position of the aligned second sequence residue, -1 if aligned to nothing
}posMap;

typedef struct
{
	char chainId[MAX_CHAIN_NAME_LENGTH + 1];
	char mappedChainId[MAX_CHAIN_NAME_LENGTH + 1]; //Update this info
	int resCount;
	posMap resMap[MAX_SEQ_LEN];
}chainMap;

typedef struct
{
	char seqId[PDB_ID_SZ];
	double resolution;
	int chainCount;
	chainMap aChainMap[MAX_CHAIN_COUNT];
}sequenceMap;

/*
typedef struct
{
	char PDBId[PDB_ID_SZ];
	double resolution;
	int chainCount;
	chainMap aChainMap[MAX_CHAIN_COUNT];
}clustalMap;*/

//Atom Info

typedef struct
{
	double completeness;
	double resolution;
	double freeRValue;
	int numberOfAtoms;
	int nunmberOfObservations;
	double dpi;
}PDB_RefineInfo;

typedef struct
{
	char atomName[ATOM_NAME_SZ];
	//char altLoc;
	char residueName;
	char chainId[MAX_CHAIN_NAME_LENGTH + 1];
	int resSeqNo;
	char iCode;
	coOrdinate3d coOrd;
	double occupancy;
	double tempFactor;
}atomInfo;

//Motif Info
typedef struct
{
	int a, b;
}pair;

typedef struct
{
	char PDB[5];
	char chainId[MAX_CHAIN_NAME_LENGTH + 1];
	char name[20];
	int numberOfSegments;
	pair pairs[MAX_SEGMENT_NUMBER];
}motifIndices;


//Static motifs for Extraction
motifIndices motifs[MAX_MOTIF_FREQ];
int motifCount;

//Geometric Analysis
typedef struct
{
	char pointA[ATOM_NAME_SZ];
	char pointO[ATOM_NAME_SZ];
	char pointB[ATOM_NAME_SZ];
	int freq[4][DIFFERENT_ANGLES + 1];
}angleDefinition;

typedef enum
{
	turnMaxLength,
	onlyLength
}strandPreference;

typedef enum
{
	mcAnnotate,
	rnaView,
	fr3d,
	dssr,
	clarna,
	undefined
}annotationMethod;

//Base and Stack Info
typedef struct
{
	char pair[PART_SIZE];
	char adjacency[PART_SIZE];
	char stackDirection[PART_SIZE];
	int isPair;
	char pairDetails[LINE_SIZE];
}stackAdjInfo;

typedef struct
{
	char edge1[EDGE_NAME_SIZE], edge2[EDGE_NAME_SIZE];
}edges;

typedef struct
{
	char chainResidue[MAX_CHAIN_NAME_LENGTH + 1];
	int residueId;
	char iCode;
	char residue;//A, U, G, C
}bases;

typedef struct
{
	bases aBase;
	edges aEdge;
	char glycosidicOrientation;
	int sourceCount;
	int level;
}node;

typedef struct
{
	bases aBase;
	int count;
	node adjacent[MAX_ADJACENCY];
	int backEdgeCount;
}adjacency;

typedef struct
{
	char chainResidue[2][MAX_CHAIN_NAME_LENGTH + 1];
	int residueID[2];
	char iCode[2];
	char residue[2];//A, U, G, C
	edges aEdges[EDGE_COUNT_LIMIT];//{W.-C., Hoogst., Sugar, C-H}2
	int edgeCount;
    //char adjacent[PART_SIZE];//null string if no adjacency
	char stackDirection[10];//null string if not stacked
	int isParallel;//-1 if no info
	char glycosidicOrientation;//cis/trans, '\0' if no info
	char MC_SymNo[BP_FEATURE_NAME_SZ];
	int baseCount;
	double confidenceWeight;
	int bpSrc;
}basePairInfo;

typedef struct
{
	basePairInfo actualBasePairs[MAX_BASE_PAIR];
	basePairInfo proposedBasePairs[MAX_BASE_PAIR];
	int bpCounts;
	int cannonicalCount;
	int nonCannonicalCount;
	int otherCount;
	char actualBPId[PDB_ID_SZ];
	char proposedBPId[PDB_ID_SZ];
}basePairMapping;

typedef struct
{
	int basePairIndex[2];
}helix;

typedef struct
{
	int pdb1IndRange[2];
	char pdb1iCodes[2];
	int pdb2IndRange[2];
	char pdb2iCodes[2];
}anchorMap;

typedef struct
{
	char chainId[MAX_CHAIN_NAME_LENGTH + 1];
	int residueId, helixIndexFromBoundary;
}residueInHelix;

//Base Pair Similarity Statistics

typedef struct
{
	double actualDist;
	double proposedDist;
}atomDist;

typedef struct
{
	double distDiff;
	atomDist aDist;
	coOrdinate3d coOrdDiff[2];
	coOrdinate3d coOrdsPropose[2];
	coOrdinate3d coOrdsActual[2];
	edges aEdgeActual;
	char glycosidicOrientation;
	char residueNamePropose[2];
	char residueNameActual[2];
	char chainIdPropose[2][MAX_CHAIN_NAME_LENGTH + 1];
	char chainIdActual[2][MAX_CHAIN_NAME_LENGTH + 1]; 
	int resSeqNoPropose[2];
	int iCodePropose[2];
	int resSeqNoActual[2];
	int iCodeActual[2];
}statisticsBP;

typedef struct
{
	char donorAtomNames[DONOR_ACCEPTOR_COUNT][EDGE_NAME_SIZE];
	char acceptorAtomNames[DONOR_ACCEPTOR_COUNT][EDGE_NAME_SIZE];
	int donorCount;
	int acceptorCount;
	char edge;
	char residue;
}edgeAtomName;

typedef struct
{
	atomInfo *donorAtomsInfo[DONOR_ACCEPTOR_COUNT];
	atomInfo *acceptorAtomsInfo[DONOR_ACCEPTOR_COUNT];
	edgeAtomName *edgeAtomNamesRef;
}edgeAtomCoordinates;

typedef struct
{
	double distance;
	char donorAtomName[EDGE_NAME_SIZE];
	int donorIndex;
	char acceptorAtomName[EDGE_NAME_SIZE];
	int acceptorIndex;
}edgeAtomPair;

#endif /* COMMONDEFINITIONS_H_ */
